/* 
    channel for Bt848 frame grabber driver

    Copyright (C) 1996,97 Marcus Metzler (mocm@metzlerbros.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _gchannel_h
#define _gchannel_h


#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <DVB.hh>
#include "gtvscreen.h"

/*
 * channel.hh
 */

const int mnamelen = 80;

class gChannel { int freq;
	char name[mnamelen + 1];
	int num;
	int color;
	int brightness;
	int hue;
	int contrast;
	int norm;
	int input;
	int pol;
	int sr;
	int fec;
	int v_pid;
	int a_pid;
	int lnb;
	int type;
	unsigned int dvb_num;
      public:
	 gChannel() {
		freq = -1;
		name[0] = '\0';
		num = -1;
		color = 0;
		brightness = 0;
		hue = 0;
		contrast = 0;
		norm = 0;
		input = 0;
		pol = NOTSET;
		sr = NOTSET;
		fec = NOTSET;
		v_pid = NOTSET;
		a_pid = NOTSET;
		lnb = NOTSET;
		type = AUTO;
		dvb_num = NOTSET;
	}
	
	gChannel(int fr, char *nam, int num) {
		int l = strlen(nam);
		freq = fr;
		if (l > mnamelen) {
			cerr <<
			    "WARNING: Name too long in channel. Name being truncated"
			    << endl;
			l = mnamelen;
		}
		name[0] = '\0';
		for (int i = 0; i < l; i++) {
			name[i] = nam[i];
		}
		name[l] = '\0';
		pol = NOTSET;
		sr = NOTSET;
		fec = NOTSET;
		v_pid = NOTSET;
		a_pid = NOTSET;
		lnb = NOTSET;
		type = AUTO;
	}

	gChannel(const Channel & chan, const Transponder & trans,
		 const Lnb & ln);

#ifdef DEBUG
	~gChannel() {
		cerr << "gChannel " << name << "  destroyed" << endl;
	}
#endif

	inline int &Freq() {
		return freq;
	} inline int &Num() {
		return num;
	} inline unsigned int &DVB_Num() {
		return dvb_num;
	} inline char *Name() {
		return name;
	} void SetName(char *nam) {
		int l = strlen(nam);
		if (l > mnamelen) {
			cerr <<
			    "WARNING: Name too long in channel. Name being truncated"
			    << endl;
			l = mnamelen;
		}
		for (int i = 0; i < l; i++) {
			name[i] = nam[i];
		}
		name[l] = '\0';
	}
	inline int &Color() {
		return color;
	} inline int &Brightness() {
		return brightness;
	} inline int &Hue() {
		return hue;
	} inline int &Contrast() {
		return contrast;
	} inline int &Norm() {
		return norm;
	} inline int &Input() {
		return input;
	} inline int &Pol() {
		return pol;
	} inline int &SR() {
		return sr;
	} inline int &FEC() {
		return fec;
	} inline int &V_pid() {
		return v_pid;
	} inline int &A_pid() {
		return a_pid;
	} inline int &LNB() {
		return lnb;
	} inline int &Type() {
		return type;
	} 
        friend ostream & operator << (ostream & stream, gChannel & x);
	friend istream & operator >> (istream & stream, gChannel & x);
};


const int channelinc = 50;

class ChannelList { int numchan;
	int maxnumc;
	gChannel *channels;

      public:
	 ChannelList();
	 ChannelList(char *filen);
	~ChannelList() {
		delete(channels);
#ifdef DEBUG
		cerr << "ChannelList destroyed" << endl;
#endif
	} void init(char *filen);
	inline int NumChan() {
		return numchan;
	} inline gChannel & operator() (int i) {
		if (i > numchan || i < 0) {
			cerr << "Channel " << i <<
			    " does not exist in Channel List" << endl;
			exit(1);
		}
		return channels[i];
	}

	void AddChannel(gChannel & chan);
	void DelChannel(char *name);
	void DelChannel(char *name, int freq);
	void DelChannel(int nb);
	friend ostream & operator << (ostream & stream, ChannelList & x);
};

#endif				/*channel.h */
