/* 
    gdvb.cc for v4l devices

    Copyright (C) 2000 Marcus Metzler (mocm@metzlerbros.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <gtk/gtk.h>
#include <DVB.hh>
#include "graphs.hh"
#include "gtvscreen.h"
#include "gdvb.hh"

extern DVB dvb;
extern Gtvscreen *gwidget;

static char savefile[256];
static char playfile[256];
static int sfileset = 0;
static int pfileset = 0;
static int lnb_lock = 0;
static GtkWidget *window1;
static void lnb_set(const Lnb & olnb);


#define TYPN 3
#define DISN 4
#define SATN 5
char *types[TYPN] = { "Single", "Universal", "Dual" };
char *diseqcs[DISN] = { "0", "1", "2", "3" };
char *sats[SATN] = { "don`t set", "Astra", "Hotbird", "Thor", "Sirius" };
enum { NOSAT, ASTRA, HOTBIRD, THOR, SIRIUS };

void widget_destroy(GtkWidget * kwidget)
{
	gtk_widget_destroy(kwidget);
	lnb_lock = 0;
}

static void lnb_edit(GtkWidget * widget, gpointer data)
{
	char *entry;
	GtkEntry *entw = GTK_ENTRY(data);
	int lnbn = -1;

	entry = gtk_entry_get_text(entw);

	for (int i = 0; i < dvb.num[LNB]; i++) {
		if (dvb.lnbs[i].name[0]) {
			if (!strncmp
			    (entry, dvb.lnbs[i].name,
			     1+strlen(dvb.lnbs[i].name))) {
				lnbn = i;
				break;
			}
		} else {
			char name[20];
			ostrstream ostr(name, 20);
			ostr << "LNB " << i << ends;
			if (!strncmp(entry, name, strlen(name))) {
				lnbn = i;
				strncpy(dvb.lnbs[i].name, name,
					strlen(name)+1);
				break;
			}
		}
	}
	if (lnbn == -1) {
		lnbn = dvb.num[LNB];
		dvb.AddLNB(lnbn + 1, 1, 9750000, 10600000, 11800000, 0,
			   NOID, NOID);
		char name[20];
		if (!strcmp(entry, "new lnb")) {
			ostrstream ostr(name, 20);
			ostr << "LNB " << lnbn << ends;
		} else
			strncpy(name, entry, strlen(entry)+1);
		strncpy((dvb.lnbs[lnbn]).name, name, strlen(name));
	}
	widget_destroy(window1);
	lnb_set(dvb.lnbs[lnbn]);
}

void lnb_setup(GtkWidget * w, gpointer data)
{
	GtkWidget *table1, *type, *button1, *button2;
	GtkWidget *lnblab;
	GList *items = NULL;

	if (!lnb_lock)
		lnb_lock = 1;
	else
		return;


	if (!dvb.num[LNB])
		status_message
		    ("After setting the LNB, please scan the satellite");

	window1 = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(window1), "LNB Setup");

	table1 = gtk_table_new(2, 2, FALSE);
	gtk_widget_show(table1);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window1)->vbox),
			   table1, TRUE, TRUE, 0);


	button1 = gtk_button_new_with_label("OK");
	gtk_widget_show(button1);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window1)->action_area),
			   button1, TRUE, TRUE, 0);

	button2 = gtk_button_new_with_label("Cancel");
	gtk_widget_show(button2);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window1)->action_area),
			   button2, TRUE, TRUE, 0);
	gtk_signal_connect_object(GTK_OBJECT(button2), "clicked",
				  (GtkSignalFunc) widget_destroy,
				  GTK_OBJECT(window1));



	lnblab = gtk_label_new("Choose LNB");
	gtk_widget_show(lnblab);
	gtk_table_attach(GTK_TABLE(table1), lnblab, 1, 2, 0, 1,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);


	type = gtk_label_new("Name");
	gtk_widget_show(type);
	gtk_table_attach(GTK_TABLE(table1), type, 0, 1, 1, 2,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);


	type = gtk_combo_new();
	gtk_object_set_data(GTK_OBJECT(window1), "Name", type);
	gtk_widget_show(type);
	gtk_table_attach(GTK_TABLE(table1), type, 1, 2, 1, 2,
			 (GtkAttachOptions) (GTK_FILL),
			 (GtkAttachOptions) (GTK_FILL), 0, 0);
	items = NULL;

	char *nlnb = "new lnb"; // for gcc 2.95 
	items = g_list_append(items,nlnb);
	for (int i = 0; i < dvb.num[LNB]; i++) {
		if (dvb.lnbs[i].name[0] != 0)
			items = g_list_append(items, dvb.lnbs[i].name);
		else {
			char name[20];
			ostrstream ostr(name, 20);
			ostr << "LNB " << i << ends;
			strncpy(dvb.lnbs[i].name,name,strlen(name)+1);
			items = g_list_append(items, name);
		}
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(type), items);
	g_list_free(items);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(type)->entry), TRUE);

	gtk_signal_connect(GTK_OBJECT(button1), "clicked",
			   GTK_SIGNAL_FUNC(lnb_edit),
			   (gpointer) GTK_COMBO(type)->entry);

	gtk_widget_show(window1);
}



void scan_sat(GtkWidget * w, gpointer data)
{

	if (!dvb.num[LNB]) {
		status_message("No LNB set, can`t scan satellite\n");
		return;
	}
	status_message("Scanning Satellites\n");
}



static void lnb_type(GtkWidget * widget, gpointer data)
{
	Lnb *lnb = (Lnb *) data;
	char *entry;

	entry = gtk_entry_get_text(GTK_ENTRY(widget));
	int i = 0;
	while (strcmp(entry, types[i]))
		i++;
	lnb->type = i;
}

static void lnb_lof1(GtkWidget * widget, gpointer data)
{
	Lnb *lnb = (Lnb *) data;

	lnb->lof1 = int (GTK_ADJUSTMENT(widget)->value * 1000);
}

static void lnb_lof2(GtkWidget * widget, gpointer data)
{
	Lnb *lnb = (Lnb *) data;

	lnb->lof2 = int (GTK_ADJUSTMENT(widget)->value * 1000);
}

static void lnb_slof(GtkWidget * widget, gpointer data)
{
	Lnb *lnb = (Lnb *) data;

	lnb->slof = int (GTK_ADJUSTMENT(widget)->value * 1000);
}

static void lnb_diseqc(GtkWidget * widget, gpointer data)
{
	Lnb *lnb = (Lnb *) data;
	char *entry;

	entry = gtk_entry_get_text(GTK_ENTRY(widget));
	lnb->diseqcnr = atoi(entry);
}


int satnum;
static void lnb_sat(GtkWidget * widget, gpointer data)
{
	char *entry;

	entry = gtk_entry_get_text(GTK_ENTRY(widget));
	int i = 0;
	while (strcmp(entry, sats[i]))
		i++;
	satnum = i;
}


Lnb *lnbs;

static void lnb_save(GtkWidget * widget, gpointer data)
{
	Lnb *lnb = (Lnb *) data;


	lnb->cpy(*lnbs);
	delete(lnbs);
	
	if (satnum) {
		int satn = dvb.num[SAT] + 1;
		switch (satnum) {
		case ASTRA:
			dvb.AddSat(satn, lnb->id, "Astra", 11700000,
				   12800000);
			break;
		case HOTBIRD:
			dvb.AddSat(satn, lnb->id, "Hotbird", 12015000,
				   12100000);
			break;
		case THOR:
			dvb.AddSat(satn, lnb->id, "Thor", 10700000,
				   12800000);
			break;
		case SIRIUS:
			dvb.AddSat(satn, lnb->id, "Sirius", 10700000,
				   12800000);
			break;
		}
	} 
	widget_destroy(window1);
	cout << *lnb << endl;
}

// create lnb window
static void lnb_set(const Lnb & olnb)
{

	GtkWidget *table1, *type, *button1, *button2;
	GtkWidget *off1, *off2, *lnblab, *slof, *diseqc, *satlab, *sat;
	GtkObject *off1_adj, *off2_adj, *slof_adj;
	GList *items = NULL;
	char wname[30];
	ostrstream ostr(wname, 30);


	satnum = 0;

	ostr << olnb.name << " Setup" << ends;
	lnbs = new Lnb(olnb);

	window1 = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(window1), wname);

	table1 = gtk_table_new(9, 2, FALSE);
	gtk_widget_show(table1);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window1)->vbox),
			   table1, TRUE, TRUE, 0);




	button1 = gtk_button_new_with_label("OK");
	gtk_widget_show(button1);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window1)->action_area),
			   button1, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(button1), "clicked",
			   GTK_SIGNAL_FUNC(lnb_save), (gpointer) & olnb);

	button2 = gtk_button_new_with_label("Cancel");
	gtk_widget_show(button2);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window1)->action_area),
			   button2, TRUE, TRUE, 0);
	gtk_signal_connect_object(GTK_OBJECT(button2), "clicked",
				  (GtkSignalFunc) widget_destroy,
				  GTK_OBJECT(window1));



	lnblab = gtk_label_new(wname);
	gtk_widget_show(lnblab);
	gtk_table_attach(GTK_TABLE(table1), lnblab, 1, 2, 0, 1,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);


	type = gtk_label_new("Type");
	gtk_widget_show(type);
	gtk_table_attach(GTK_TABLE(table1), type, 0, 1, 1, 2,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);


	type = gtk_combo_new();
	gtk_object_set_data(GTK_OBJECT(window1), "Type", type);
	gtk_widget_show(type);
	gtk_table_attach(GTK_TABLE(table1), type, 1, 2, 1, 2,
			 (GtkAttachOptions) (GTK_FILL),
			 (GtkAttachOptions) (GTK_FILL), 0, 0);
	items = NULL;
	for (int i = 0; i < TYPN; i++) {
		items = g_list_append(items, types[i]);
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(type), items);
	g_list_free(items);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(type)->entry),
			   types[olnb.type]);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(type)->entry), FALSE);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(type)->entry), "changed",
			   GTK_SIGNAL_FUNC(lnb_type), (gpointer) lnbs);



	off1 = gtk_label_new("Offset1");
	gtk_widget_show(off1);
	gtk_table_attach(GTK_TABLE(table1), off1, 0, 1, 2, 3,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);

	off1_adj =
	    gtk_adjustment_new(olnb.lof1 / 1000, 9000, 13000, 1, 10, 10);
	off1 = gtk_spin_button_new(GTK_ADJUSTMENT(off1_adj), 10, 0);
	gtk_widget_show(off1);
	gtk_table_attach(GTK_TABLE(table1), off1, 1, 2, 2, 3,
			 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			 (GtkAttachOptions) (0), 0, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(off1), TRUE);
	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(off1),
					  GTK_UPDATE_IF_VALID);
	gtk_signal_connect(off1_adj, "value_changed",
			   GTK_SIGNAL_FUNC(lnb_lof1), (gpointer) lnbs);



	off2 = gtk_label_new("Offset2");
	gtk_widget_show(off2);
	gtk_table_attach(GTK_TABLE(table1), off2, 0, 1, 3, 4,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);

	off2_adj =
	    gtk_adjustment_new(olnb.lof2 / 1000, 9000, 13000, 1, 10, 10);
	off2 = gtk_spin_button_new(GTK_ADJUSTMENT(off2_adj), 10, 0);
	gtk_widget_show(off2);
	gtk_table_attach(GTK_TABLE(table1), off2, 1, 2, 3, 4,
			 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			 (GtkAttachOptions) (0), 0, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(off2), TRUE);
	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(off2),
					  GTK_UPDATE_IF_VALID);
	gtk_signal_connect(off2_adj, "value_changed",
			   GTK_SIGNAL_FUNC(lnb_lof2), (gpointer) lnbs);


	slof = gtk_label_new("Switch ");
	gtk_widget_show(slof);
	gtk_table_attach(GTK_TABLE(table1), slof, 0, 1, 4, 5,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);

	slof_adj =
	    gtk_adjustment_new(olnb.slof / 1000, 9000, 13000, 1, 10, 10);
	slof = gtk_spin_button_new(GTK_ADJUSTMENT(slof_adj), 10, 0);
	gtk_widget_show(slof);
	gtk_table_attach(GTK_TABLE(table1), slof, 1, 2, 4, 5,
			 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			 (GtkAttachOptions) (0), 0, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(slof), TRUE);
	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(slof),
					  GTK_UPDATE_IF_VALID);
	gtk_signal_connect(slof_adj, "value_changed",
			   GTK_SIGNAL_FUNC(lnb_slof), (gpointer) lnbs);


	diseqc = gtk_label_new("Diseqc");
	gtk_widget_show(diseqc);
	gtk_table_attach(GTK_TABLE(table1), diseqc, 0, 1, 5, 6,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);


	diseqc = gtk_combo_new();
	gtk_object_set_data(GTK_OBJECT(window1), "diseqc", diseqc);
	gtk_widget_show(diseqc);
	gtk_table_attach(GTK_TABLE(table1), diseqc, 1, 2, 5, 6,
			 (GtkAttachOptions) (GTK_FILL),
			 (GtkAttachOptions) (GTK_FILL), 0, 0);
	items = NULL;
	for (int i = 0; i < DISN; i++) {
		items = g_list_append(items, diseqcs[i]);
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(diseqc), items);
	g_list_free(items);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(diseqc)->entry),
			   diseqcs[olnb.diseqcnr]);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(diseqc)->entry), FALSE);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(diseqc)->entry), "changed",
			   GTK_SIGNAL_FUNC(lnb_diseqc), (gpointer) lnbs);


	satlab = gtk_label_new("Satellite");
	gtk_widget_show(satlab);
	gtk_table_attach(GTK_TABLE(table1), satlab, 1, 2, 7, 8,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);

	sat = gtk_label_new("Name");
	gtk_widget_show(sat);
	gtk_table_attach(GTK_TABLE(table1), sat, 0, 1, 8, 9,
			 (GtkAttachOptions) (0),
			 (GtkAttachOptions) (0), 0, 0);


	sat = gtk_combo_new();
	gtk_object_set_data(GTK_OBJECT(window1), "sat", sat);
	gtk_widget_show(sat);
	gtk_table_attach(GTK_TABLE(table1), sat, 1, 2, 8, 9,
			 (GtkAttachOptions) (GTK_FILL),
			 (GtkAttachOptions) (GTK_FILL), 0, 0);
	items = NULL;
	for (int i = 0; i < SATN; i++) {
		items = g_list_append(items, sats[i]);
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(sat), items);
	g_list_free(items);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(sat)->entry), FALSE);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(sat)->entry), "changed",
			   GTK_SIGNAL_FUNC(lnb_sat), (gpointer) lnbs);

	gtk_widget_show(window1);

}


void sfile_ok_sel(GtkWidget * w, GtkFileSelection * fs)
{
	gchar *name;

	name = gtk_file_selection_get_filename(fs);
	if (name) {
		int len = strlen(name);
		strncpy(savefile, name, len);
		savefile[len] = '\0';
		gtk_widget_destroy(GTK_WIDGET(fs));
		sfileset = TRUE;
	}
}

void pfile_ok_sel(GtkWidget * w, GtkFileSelection * fs)
{
	gchar *name;

	name = gtk_file_selection_get_filename(fs);
	if (name) {
		int len = strlen(name);
		strncpy(playfile, name, len);
		playfile[len] = '\0';
		gtk_widget_destroy(GTK_WIDGET(fs));
		pfileset = TRUE;
	}
}



void load(GtkWidget * w, GtkFileSelection * fs)
{
	GtkWidget *filew;

	pfileset = FALSE;

	filew = gtk_file_selection_new("File selection");

	gtk_signal_connect(GTK_OBJECT
			   (GTK_FILE_SELECTION(filew)->ok_button),
			   "clicked", (GtkSignalFunc) pfile_ok_sel, filew);

	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION
					     (filew)->cancel_button),
				  "clicked",
				  (GtkSignalFunc) gtk_widget_destroy,
				  GTK_OBJECT(filew));

	gtk_widget_show(filew);
}

void save(GtkWidget * w, GtkFileSelection * fs)
{
	GtkWidget *filew;

	sfileset = FALSE;

	filew = gtk_file_selection_new("File selection");

	gtk_signal_connect(GTK_OBJECT
			   (GTK_FILE_SELECTION(filew)->ok_button),
			   "clicked", (GtkSignalFunc) sfile_ok_sel, filew);

	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION
					     (filew)->cancel_button),
				  "clicked",
				  (GtkSignalFunc) gtk_widget_destroy,
				  GTK_OBJECT(filew));

	gtk_widget_show(filew);
}




static void fpr(int what)
{
	char buf[80];
	ostrstream str(buf, 80);
	int fd1;
	int fd2;


	switch (what) {
	case PLAY:
		str << "Playing file: " << playfile << ends;
		status_message(buf);
		if ((fd1 = open(playfile, O_RDONLY)) < 1) {
			cerr << "Couldn't open " << playfile << endl;
		} else {
			gtvscreen_set_values(gwidget, GTV_PLAY,(gpointer) fd1);
		} 
		break;

	case RECORD:
		str << "Recording file: " << savefile << ends;
		status_message(buf);
		if ((fd2 = open(savefile, O_CREAT | O_RDWR | O_TRUNC)) < 1) {
			cerr << "Couldn't open " << savefile << endl;
			gtvscreen_set_values(gwidget, GTV_STOP, (gpointer) 0);
		} else {
			gtvscreen_set_values(gwidget, GTV_RECORD, 
					     (gpointer) fd2);
		}		
		break;
	}

}

static void frecord(GtkWidget * w, GtkFileSelection * fs)
{
	gchar *name;

	name = gtk_file_selection_get_filename(fs);
	int len = strlen(name);
	strncpy(savefile, name, len);
	savefile[len] = '\0';
	gtk_widget_destroy(GTK_WIDGET(fs));
	sfileset = TRUE;
	fpr(RECORD);
}

static void fplay(GtkWidget * w, GtkFileSelection * fs)
{
	gchar *name;

	name = gtk_file_selection_get_filename(fs);
	int len = strlen(name);
	strncpy(playfile, name, len);
	playfile[len] = '\0';
	gtk_widget_destroy(GTK_WIDGET(fs));
	pfileset = TRUE;
	fpr(PLAY);

}

void play_record(int l)
{
	GtkWidget *filew;
	filew = gtk_file_selection_new("File selection");

	switch (l) {
	case PLAY:
		if (!pfileset) {
			gtk_signal_connect(GTK_OBJECT
					   (GTK_FILE_SELECTION(filew)->
					    ok_button), "clicked",
					   (GtkSignalFunc) fplay, filew);
			gtk_signal_connect_object(GTK_OBJECT
						  (GTK_FILE_SELECTION
						   (filew)->cancel_button),
						  "clicked",
						  (GtkSignalFunc)
						  gtk_widget_destroy,
						  GTK_OBJECT(filew));

			gtk_widget_show(filew);
		} else
			fpr(l);
		break;
	case RECORD:
		if (!sfileset) {
			gtk_signal_connect(GTK_OBJECT
					   (GTK_FILE_SELECTION(filew)->
					    ok_button), "clicked",
					   (GtkSignalFunc) frecord, filew);
			gtk_signal_connect_object(GTK_OBJECT
						  (GTK_FILE_SELECTION
						   (filew)->cancel_button),
						  "clicked",
						  (GtkSignalFunc)
						  gtk_widget_destroy,
						  GTK_OBJECT(filew));

			gtk_widget_show(filew);
		} else
			fpr(l);
		break;
	}
}

