/* 
    graph.cc for v4l devices

    Copyright (C) 1999 Marcus Metzler (mocm@metzlerbros.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <gtk/gtk.h>
#include <DVB.hh>
#include <sys/wait.h>
#include "graphs.hh"
#include "gtvscreen.h"
#include "xpm.h"
#include "player_xpm.h"
#include "tree.hh"
#include "audio.hh"
#include "gdvb.hh"
#include "player.hh"

static GtkWidget *win_main, *statusbar, *aspect, *menubar;
static int listn = 0;

static int fdescr;
Gtvscreen *gwidget;
ChannelList chlist;
gint contextid;
GtkWidget *clist1, *t_widget;
GtkAccelGroup *accel_group;
GtkWidget *channel_list();
DVB dvb;
int no_crypt;

void status_message(char *buf)
{
	gint messid;
	
	
	if (statusbar && GTK_WIDGET_VISIBLE(statusbar)) {
		gtk_statusbar_pop(GTK_STATUSBAR(statusbar), contextid);
		messid = gtk_statusbar_push(GTK_STATUSBAR(statusbar), 
					    contextid, buf);
	}
}


/*                          */
/* get saved list filename  */
/*                          */
void get_gvideorc(char *path)
{
	const char *home = getenv("HOME");
	const char *file = ".gvideorc";

	strncpy(path, home, STRSIZE);
	strncat(path, "/", STRSIZE);
	strncat(path, file, STRSIZE);
}


int get_dvbrc(char *path)
{
	ifstream dvbin;

	const char *home = getenv("HOME");
	const char *file = ".dvbrc";

	strncpy(path, home, STRSIZE);
	strncat(path, "/", STRSIZE);
	strncat(path, file, STRSIZE);
	dvbin.open(path);
	if (dvbin) {
		dvbin >> dvb;
		return 1;
	}

	return 0;
}

void save_dvb(GtkWidget * w, gpointer data){
	ofstream dvbout;
	char path[80];
	const char *home = getenv("HOME");
	const char *file = ".dvbrc";
	
	strncpy(path, home, STRSIZE);
	strncat(path, "/", STRSIZE);
	strncat(path, file, STRSIZE);
	dvbout.open(path);
	if (dvbout) {
		status_message("Saving .dvbrc\n");
		dvbout << dvb;
	}
	
}


void Add_Accel(GtkWidget * widget, char *szAccel);

// give channel settings to gtvscreen widget
static void set_channel(gChannel & ch)
{
	char buf[80];
	ostrstream str(buf, 80);

	str << "Setting favorite channel " << ch.Num() << "  (" << ch.
	    Name() << " )" << ends;

	status_message(buf);

	gtvscreen_set_values(gwidget, GTV_COLOR, (gpointer) ch.Color());
	gtvscreen_set_values(gwidget, GTV_BRIGHT,
			     (gpointer) ch.Brightness());
	gtvscreen_set_values(gwidget, GTV_HUE, (gpointer) ch.Hue());
	gtvscreen_set_values(gwidget, GTV_CONTRAST,
			     (gpointer) ch.Contrast());
	gtvscreen_set_values(gwidget, GTV_NORM, (gpointer) ch.Norm());
	gtvscreen_set_values(gwidget, GTV_INPUT, (gpointer) ch.Input());

	freqtype fr;

	fr.fre = (uint) ch.Freq();
	fr.sr = ch.SR();
	fr.lnb = ch.LNB();
	fr.fec = ch.FEC();
	fr.type = ch.Type();
	fr.pol = ch.Pol();
	fr.apid = ch.A_pid();
	fr.vpid = ch.V_pid();

	if (ch.DVB_Num()) {
	        int k;
		for (k=0; k< dvb.num[CHAN]; k++) {
			if (dvb.chans[k].pnr == ch.DVB_Num())
			        break;
		}
		if (k<dvb.num[CHAN])
		        set_channel_dvb(GTK_WIDGET(gwidget),k);
	} else {
		set_audio_menu(NULL);
		gtvscreen_set_values(gwidget, GTV_FREQ, (gpointer) & fr);
	}
}



void set_aspect(GtkWidget * w, gpointer data)
{
	double ratio = double ((int) data) / 10000.0;
	gtvscreen_set_values(gwidget, GTV_CAP,(gpointer)0);

	gtk_aspect_frame_set(GTK_ASPECT_FRAME(aspect), 0.5, 0.5, ratio,
			     FALSE);
	gtvscreen_set_values(gwidget, GTV_CAP,(gpointer)1);
}

void toggle_chl(GtkWidget * w, gpointer data)
{
	static int visible = 0;
	GtkWidget *widget = GTK_WIDGET(data);


	if (visible) {
		gtk_widget_hide(widget);
		visible = 0;
	} else {
		gtk_widget_show(widget);
		visible = 1;
	}
}

// Initialize gtvscreen widget
static void init_gtvscreen(char *dev, int bpp, int norm, int remote,
			   int onoff, int ch,int fd_dev=-1)
{
	GtkWidget *widget;
	if (dev)
		gwidget = GTVSCREEN(gtvscreen_new_with_dev(dev));
	else
		if (fd_dev == -1)
			gwidget = GTVSCREEN(gtvscreen_new()); // create widget
		else
			gwidget = GTVSCREEN(gtvscreen_new_with_fd(fd_dev));

	gtvscreen_set_values(gwidget, GTV_REMOTE, (gpointer) remote);
	// set remote-access
	if (bpp)
		gtvscreen_set_values(gwidget, GTV_BPP, (gpointer) bpp);	
	// set bpp
	if (norm)
		gtvscreen_set_values(gwidget, GTV_NORM, (gpointer) norm);  
	// set tvnorm
	if (onoff)
		gtvscreen_set_values(gwidget, GTV_CAP, (gpointer) onoff);
	// set on/off 
	if (ch)
		gtvscreen_set_values(gwidget, GTV_INPUT, (gpointer) ch);
	// set input
	widget = GTK_WIDGET(gwidget);
	gtk_widget_show(widget);

	gtvscreen_get_values(gwidget, GTV_FD, &fdescr);
}

void init_graphics(char *dev, int bpp, int norm, int remote, int onoff,
		   int channel)
{
	GtkWidget *widget;
	GtkWidget *vbox, *handlebox;
	GtkWidget *menu, *menuitem;
	char filen[80];
	int fd_dev;

	no_crypt = 0;
#ifdef NAPI
	fd_dev = 0;
#else
	dvb.init(dev, "/dev/vbi0",0 );
	fd_dev = dvb.fdvb;
#endif
	if (!dev || fd_dev<0) dev = "/dev/video";
	else if (fd_dev>0) dev = (char *) NULL;

	get_gvideorc(filen);
	chlist.init(filen);

	init_gtvscreen(dev, bpp, norm, remote, onoff, channel, fd_dev);
	// create main window
	win_main = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(win_main), "gVideo");
	int maxx,maxy;
	gtvscreen_get_values(gwidget,GTV_MAXX,&maxx);
	gtvscreen_get_values(gwidget,GTV_MAXY,&maxy);
	gtk_widget_set_usize(win_main, maxx, maxy);
	gtk_window_set_policy(GTK_WINDOW(win_main), TRUE, FALSE, TRUE);	
	gtk_container_border_width(GTK_CONTAINER(win_main), 0);
	accel_group = gwidget->accel_group;
	gtk_accel_group_attach(accel_group, GTK_OBJECT(win_main));
	gtk_signal_connect(GTK_OBJECT(win_main), "destroy",
			   GTK_SIGNAL_FUNC(quit), NULL);
	gtk_signal_connect(GTK_OBJECT(win_main), "delete_event",
			   GTK_SIGNAL_FUNC(quit), NULL);


	gtk_widget_show(win_main);
	t_widget = gtk_tree_new();

	// create vbox
	vbox = gtk_vbox_new(FALSE, 3);
	gtk_container_add(GTK_CONTAINER(win_main), vbox);
	gtk_widget_show(vbox);

	//create menubar  
	menubar = gtk_menu_bar_new();
	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
	gtk_widget_show(menubar);
	menu = CreateBarSubMenu(menubar, "File");
	menuitem = CreateMenuItem(menu, "Quit", "^q", "Quit",
				  GTK_SIGNAL_FUNC(quit), (void *) "quit");

	int mpgd;
	gtvscreen_get_values(gwidget, GTV_MPGD, &mpgd);
	int mpge;
	gtvscreen_get_values(gwidget, GTV_MPGD, &mpge);
	int isdvb;
	gtvscreen_get_values(gwidget, GTV_DVB, &isdvb);

	if (mpgd)
		menuitem = CreateMenuItem(menu, "Set Play File", "^l",
					  "Set Play File",
					  GTK_SIGNAL_FUNC(load), NULL);

	if (mpge)
		menuitem = CreateMenuItem(menu, "Set Record File", "^s",
					  "Set Record File",
					  GTK_SIGNAL_FUNC(save), NULL);

	if (isdvb)
		menuitem = CreateMenuItem(menu, "Save DVB", "^d",
					  "Save .dvbrc",
					  GTK_SIGNAL_FUNC(save_dvb), NULL);

	menu = CreateBarSubMenu(menubar, "Option");
	widget = channel_list();
	menuitem = CreateMenuItem(menu, "Favorite", "l", "clist",
				  GTK_SIGNAL_FUNC(toggle_chl),
				  (void *) widget);

	if (isdvb){
		menuitem = CreateMenuItem(menu, "LNB Setup", "b", "LNB",
					  GTK_SIGNAL_FUNC(lnb_setup), 0);

		menuitem = CreateMenuItem(menu, "Sat Scan", "r", "Scan",
					  GTK_SIGNAL_FUNC(scan_sat), 0);
		gtk_widget_set_sensitive(menuitem, False);

		widget = make_tree_dialog();
		menuitem = CreateMenuItem(menu, "Show DVB", "d", "DVB Info",
					  GTK_SIGNAL_FUNC(show_dvb),
					  (void *) widget);
	}

	if (mpgd || mpge){
		widget = create_tpwindow();
		menuitem = CreateMenuItem(menu, "tuxPlayer", "^t",
					  "Use tuxPlayer",
					  GTK_SIGNAL_FUNC(show_player), 
					  (void *) widget);
	}

	menu = CreateBarSubMenu(menubar, "Aspect");
	menuitem = CreateMenuItem(menu, "4:3", "", "4:3",
				  GTK_SIGNAL_FUNC(set_aspect),
				  (void *) 13333);

	menuitem = CreateMenuItem(menu, "16:9", "", "16:9",
				  GTK_SIGNAL_FUNC(set_aspect),
				  (void *) 17777);

	CreateAudioMenu(menubar, "Audio");

	//create handlebox
	handlebox = gtk_handle_box_new();
	gtk_box_pack_start(GTK_BOX(vbox), handlebox, FALSE, FALSE, 0);

	// create toolbar (duh)
	CreateToolbar(handlebox);


	// show handlebox and pack gtvscreen widget
	gtk_widget_show(handlebox);


	// create aspect frame
	aspect = gtk_aspect_frame_new(NULL, 0.5, 0.5, 1.3333, FALSE);
	gtk_object_set_data(GTK_OBJECT(win_main), "aspect", aspect);
	gtk_widget_show(aspect);
	gtk_box_pack_start(GTK_BOX(vbox), aspect, TRUE, TRUE, 0);
	gtk_container_add(GTK_CONTAINER(aspect), GTK_WIDGET(gwidget));

	GdkColor black = { 0, 0, 0, 0 };
	gdk_color_alloc(gdk_colormap_get_system(), &black);
	GtkStyle *style, *ostyle;
	ostyle = gtk_widget_get_default_style();
	style = gtk_style_copy(ostyle);
	for (int i = 0; i < 5; i++) {
		style->bg[i] = black;
		style->fg[i] = black;
	}
	gtk_widget_set_style(GTK_WIDGET(win_main), style);

	//create statusbar
	widget = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widget)
				       , GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	gtk_container_set_border_width(GTK_CONTAINER(widget), 0);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, FALSE);
	statusbar = gtk_statusbar_new();
	gtk_object_set_data(GTK_OBJECT(win_main), "statusbar", statusbar);
	gtk_widget_show(statusbar);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(widget),
					      statusbar);
	contextid =
	    gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
					 "Statusbar");
	gtk_statusbar_push(GTK_STATUSBAR(statusbar), contextid,
			   "Welcome to gVideo");
	gtk_widget_show(widget);

	if (isdvb)
		if (get_dvbrc(filen)) {
			cout << "Found DVB input file " << filen << endl;
			make_tree();
		} else {
			cout << "Could not find DVB input file " 
			     << filen << endl;
			status_message("Please set LNB\n");
		}

	gint x,y;
	gdk_window_get_deskrelative_origin(win_main->window, &x, &y);
	gdk_window_move_resize(win_main->window,x,y,640, 480);
	
	// gtk main loop
	
	if (chlist.NumChan()){
	        set_channel(chlist(0)); //once should be enough
		set_channel(chlist(0)); // but the driver doesn`t get it !!
	}
	
#ifdef PTHREADS
	gdk_threads_enter();
	gtk_main();
	gdk_threads_leave();
#else
	gtk_main();
#endif
}



void switches(GtkWidget * widget, gpointer data)
{
	int in;
	int cap;
	int act;

	int l = (int) data;
	if (GTK_WIDGET_VISIBLE(widget)) {
		switch (l) {

			//switch to overlay mode 
		case OVERLAY:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			gtvscreen_get_values(gwidget, GTV_OVL, &cap);
			if (act != cap) {
				gtvscreen_set_values(gwidget, GTV_OVL,
						     (gpointer) act);
			}
			break;

			// switch on capturing
		case CAPOO:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			gtvscreen_get_values(gwidget, GTV_CAP, &cap);
			if (cap != act) {
				gtvscreen_set_values(gwidget, GTV_CAP,
						     (gpointer) act);
			}
			break;

			// switch to selected input
		case TV:
		case VID1:
		case VID2:
		case VID3:
		case VID4:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act) {
				in = l - TV;
				gtvscreen_set_values(gwidget, GTV_INPUT,
						     (gpointer) in);
			}
			break;
		case DOWN:
			listn--;
			if (listn < 0)
				listn = chlist.NumChan() - 1;
			gtk_clist_select_row(GTK_CLIST(clist1), listn, 0);
			gtk_clist_moveto(GTK_CLIST(clist1), listn, 0, 0,
					 0);
			break;

		case UP:
			listn++;
			if (listn > chlist.NumChan() - 1)
				listn = 0;
			gtk_clist_select_row(GTK_CLIST(clist1), listn, 0);
			gtk_clist_moveto(GTK_CLIST(clist1), listn, 0, 0,
					 0);
			break;

		case FULL:
			int isfull;
			gtvscreen_get_values(gwidget, GTV_FULLS, &isfull);
			if (isfull) {
				gtk_aspect_frame_set(GTK_ASPECT_FRAME
						     (aspect), 0.5, 0.5,
						     1.3333, FALSE);
			} else {
				gtk_aspect_frame_set(GTK_ASPECT_FRAME
						     (aspect), 0.5, 0.5,
						     1.3333, TRUE);
			}
			gtvscreen_set_values(gwidget, GTV_FULLS, NULL);
			break;

		case RECORD:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act)
				play_record(RECORD);
			break;

		case PLAY:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act)
				play_record(PLAY);
			break;

		case PAUSE:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act)
				gtvscreen_set_values(gwidget, GTV_PAUSE,
						     NULL);
			status_message("Replay paused");
			break;

		case STOP:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act){
				gtvscreen_set_values(gwidget, GTV_STOP,
						     NULL);
			}
			status_message("Replay stopped");
			break;

		case FFWD:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act)
				gtvscreen_set_values(gwidget, GTV_FFWD,
						     NULL);
			status_message("Fast Forward");
			break;

		case FBWD:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act)
				gtvscreen_set_values(gwidget, GTV_FBWD,
						     NULL);
			status_message("Fast Backward");
			break;

		case FSKP:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act)
				gtvscreen_set_values(gwidget, GTV_FSKP,
						     NULL);
			status_message("Forward Skip");
			break;


		case BSKP:
			act =
			    int (gtk_toggle_button_get_active
				 ((GtkToggleButton *) widget));
			if (act)
				gtvscreen_set_values(gwidget, GTV_BSKP,
						     NULL);
			status_message("Backward Skip");
			break;

		default:
			break;

		}
	}
}



void ovrtoggle(GtkWidget * gtv, gpointer data)
{
	int ovl, ovr;
	GtkWidget *tool = (GtkWidget *) data;
	if (GTK_WIDGET_VISIBLE(tool)) {
		ovr =
		    int (gtk_toggle_button_get_active
			 ((GtkToggleButton *) tool));
		gtvscreen_get_values(gwidget, GTV_OVL, &ovl);

		if (ovr != ovl)
			gtk_toggle_button_set_active((GtkToggleButton *)
						     tool, ovl);
	}
}

void captoggle(GtkWidget * gtv, gpointer data)
{
	int cap, car;
	GtkWidget *tool = (GtkWidget *) data;
	if (GTK_WIDGET_VISIBLE(tool)) {
		gtvscreen_get_values(gwidget, GTV_CAP, &cap);
		car =
		    int (gtk_toggle_button_get_active
			 ((GtkToggleButton *) tool));
		if (car != cap)
			gtk_toggle_button_set_active((GtkToggleButton *)
						     tool, cap);
	}
}

void input_change(GtkWidget * gtv, gpointer data)
{
	int in, state;
	GtkWidget **tool = (GtkWidget **) data;
	gtvscreen_get_values(GTVSCREEN(gtv), GTV_INPUT, &in);
	if (GTK_WIDGET_VISIBLE(tool[in])) {
		state =
		    gtk_toggle_button_get_active((GtkToggleButton *)
						 tool[in]);
		if (!state)
			gtk_toggle_button_set_active((GtkToggleButton *)
						     tool[in], TRUE);
	}
}


void stopped(GtkWidget * gtv, gpointer data)
{
	int state;
	GtkWidget *tool = (GtkWidget *) data;
	if (GTK_WIDGET_VISIBLE(tool)) {
		state =
		    gtk_toggle_button_get_active((GtkToggleButton *) tool);
		if (!state)
			gtk_toggle_button_set_active((GtkToggleButton *)
						     tool, TRUE);
	}
}

void CreateToolbar(GtkWidget * vbox_main)
{
	GtkWidget *toolbar;
	GtkWidget *tool;
	GtkWidget *pixi;



	//create the toolbar and add it to the window 
	toolbar =
	    gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_container_add(GTK_CONTAINER(vbox_main), toolbar);
	gtk_widget_show(toolbar);


	int mpgd;
	gtvscreen_get_values(gwidget, GTV_MPGD, &mpgd);
	int mpge;
	gtvscreen_get_values(gwidget, GTV_MPGD, &mpge);

	tool = NULL;
	if (mpgd){
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_fskp);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_BUTTON,
						   NULL,
						   "FSkp", "FSkp", "FSkp", 
						   pixi, 
						   (GtkSignalFunc) switches,
						   gpointer(FSKP));


//		gtk_signal_connect(GTK_OBJECT(gwidget), "chfskp",
//				   GTK_SIGNAL_FUNC(fskp), (gpointer) tool);

		tool = NULL;

		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_ffwd);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_RADIOBUTTON,
						   tool,
						   "FFwd", "FFwd", "FFwd", 
						   pixi, 
						   (GtkSignalFunc) switches,
						   gpointer(FFWD));


//		gtk_signal_connect(GTK_OBJECT(gwidget), "chffwd",
//				   GTK_SIGNAL_FUNC(ffwd), (gpointer) tool);

	}
	if (mpgd || mpge){
	  
	  pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_stop);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_RADIOBUTTON,
						   tool,
						   "Stop", "Stop", "Stop", 
						   pixi, 
						   (GtkSignalFunc) switches,
						   gpointer(STOP));
		Add_Accel(tool, "s");
		gtk_signal_connect(GTK_OBJECT(gwidget), "chstop",
				   GTK_SIGNAL_FUNC(stopped), (gpointer) tool);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tool), TRUE);
	
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_pause);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_RADIOBUTTON,
						   tool,
						   "Pause", "Pause", "Pause", 
						   pixi,
						   (GtkSignalFunc) switches,
						   gpointer(PAUSE));
		Add_Accel(tool, "p");
		gtk_signal_connect(GTK_OBJECT(gwidget), "chpause",
				   GTK_SIGNAL_FUNC(stopped), (gpointer) tool);
	}
	if (mpgd){
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_play);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_RADIOBUTTON,
						   tool,
						   "Play", "Play", "Play", 
						   pixi,
						   (GtkSignalFunc) switches,
						   gpointer(PLAY));
		Add_Accel(tool, "^p");
		gtk_signal_connect(GTK_OBJECT(gwidget), "chplay",
				   GTK_SIGNAL_FUNC(stopped), (gpointer) tool);
	}

	if (mpge){
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_record);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_RADIOBUTTON,
						   tool,
						   "Record", "Record", 
						   "Record",
						   pixi, 
						   (GtkSignalFunc) switches,
						   gpointer(RECORD));
		Add_Accel(tool, "^r");
		gtk_signal_connect(GTK_OBJECT(gwidget), "chrecord",
				   GTK_SIGNAL_FUNC(stopped), (gpointer) tool);
	}

	if (mpgd){
	  
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_fbwd);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_RADIOBUTTON,
						   tool,
						   "FBwd", "FBwd", "FBwd", 
						   pixi, 
						   (GtkSignalFunc) switches,
						   gpointer(FFWD));


//		gtk_signal_connect(GTK_OBJECT(gwidget), "chfbwd",
//				   GTK_SIGNAL_FUNC(fbwd), (gpointer) tool);

		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_bskp);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_BUTTON,
						   NULL,
						   "BSkp", "BSkp", "BSkp", 
						   pixi, 
						   (GtkSignalFunc) switches,
						   gpointer(FSKP));


//		gtk_signal_connect(GTK_OBJECT(gwidget), "chbskp",
//				   GTK_SIGNAL_FUNC(bskp), (gpointer) tool);

	}


	gtk_toolbar_prepend_space(GTK_TOOLBAR(toolbar));

	if (!gwidget->remote) {
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_full);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_BUTTON,
						   NULL,
						   "Full",
						   "Toggle Fullscreen",
						   "f", pixi,
						   (GtkSignalFunc)
						   switches,
						   gpointer(FULL));
		Add_Accel(tool, "f");

		gtk_toolbar_prepend_space(GTK_TOOLBAR(toolbar));
	}

	pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_down);
	tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
					   GTK_TOOLBAR_CHILD_BUTTON,
					   NULL,
					   "Down", "Channel Down", "-",
					   pixi, (GtkSignalFunc) switches,
					   gpointer(DOWN));
	Add_Accel(tool, "-");

	pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_up);
	tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
					   GTK_TOOLBAR_CHILD_BUTTON,
					   NULL,
					   "Up", "Channel Up", "+", pixi,
					   (GtkSignalFunc) switches,
					   gpointer(UP));

	Add_Accel(tool, "+");

	gtk_toolbar_prepend_space(GTK_TOOLBAR(toolbar));

	tool = NULL;

	int ninput;
	gtvscreen_get_values(gwidget, GTV_NINPUT, &ninput);
	int tune;
	gtvscreen_get_values(gwidget, GTV_TUNE, &tune);

	GtkWidget **intool;
	intool = new(GtkWidget *)[ninput];
	// first button of radio box for input switching : tuner
	switch (ninput) {

	case 5:
		// video 4
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_vid4);
		intool[4] =
		    gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						GTK_TOOLBAR_CHILD_RADIOBUTTON,
						tool, "Video4", "Video4",
						"Video4", pixi,
						(GtkSignalFunc) switches,
						gpointer(VID4));
		tool = intool[4];

	case 4:
		// video 3
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_vid3);
		intool[3] =
		    gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						GTK_TOOLBAR_CHILD_RADIOBUTTON,
						tool, "Video3", "Video3",
						"Video3", pixi,
						(GtkSignalFunc) switches,
						gpointer(VID3));
		tool = intool[3];

	case 3:
		// video 2
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_vid2);
		intool[2] =
		    gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						GTK_TOOLBAR_CHILD_RADIOBUTTON,
						tool, "Video2", "Video2",
						"Video2", pixi,
						(GtkSignalFunc) switches,
						gpointer(VID2));
		tool = intool[2];

	case 2:
		// video 1
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_vid1);
		intool[1] =
		    gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						GTK_TOOLBAR_CHILD_RADIOBUTTON,
						tool, "Video1", "Video1",
						"Video1", pixi,
						(GtkSignalFunc) switches,
						gpointer(VID1));
		tool = intool[1];

	case 1:
		if (tune)
			pixi = CreateWidgetFromXpm(vbox_main, 
						   (gchar **) xpm_tv);
		else
			pixi = CreateWidgetFromXpm(vbox_main, 
						   (gchar **) xpm_vid0);

		intool[0] =
		    gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						GTK_TOOLBAR_CHILD_RADIOBUTTON,
						tool, "Tuner", "Tuner",
						"Tuner", pixi,
						(GtkSignalFunc) switches,
						gpointer(TV));

		gtk_signal_connect(GTK_OBJECT(gwidget), "chinput",
				   GTK_SIGNAL_FUNC(input_change),
				   (gpointer) intool);
		tool = intool[0];

	}

	input_change(GTK_WIDGET(gwidget), intool);

	gtk_toolbar_prepend_space(GTK_TOOLBAR(toolbar));

	int remote;
	gtvscreen_get_values(gwidget, GTV_REMOTE, &remote);

	int covl;
	gtvscreen_get_values(gwidget, GTV_COVL, &covl);

	if (!remote && covl) {
		pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_over);
		tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
						   GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
						   NULL,
						   "Overlay", "Overlay",
						   "Overlay", pixi,
						   (GtkSignalFunc)
						   switches,
						   gpointer(OVERLAY));

		gtk_signal_connect(GTK_OBJECT(gwidget), "choverlay",
				   GTK_SIGNAL_FUNC(ovrtoggle),
				   (gpointer) tool);
	} else
		gtk_toolbar_prepend_space(GTK_TOOLBAR(toolbar));



	// capture button 
	pixi = CreateWidgetFromXpm(vbox_main, (gchar **) xpm_cap);
	tool = gtk_toolbar_prepend_element(GTK_TOOLBAR(toolbar),
					   GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
					   NULL,
					   "capture", "capture", "capture",
					   pixi, (GtkSignalFunc) switches,
					   gpointer(CAPOO));
	gtk_signal_connect(GTK_OBJECT(gwidget), "chcapture",
			   GTK_SIGNAL_FUNC(captoggle), (gpointer) tool);

}

void quit()
{
	int state = 0;
	int isfull;
	gtvscreen_get_values(gwidget, GTV_FULLS, &isfull);
	if (isfull)
		gtvscreen_set_values(gwidget, GTV_FULLS, NULL);

	gtvscreen_set_values(gwidget, GTV_CAP, (gpointer) state);
	gtk_exit(0);
}


void Add_Accel(GtkWidget * widget, char *szAccel)
{
	if (accel_group == NULL) {
		accel_group = gtk_accel_group_new();
		gtk_accel_group_attach(accel_group, GTK_OBJECT(win_main));
	}

	if (szAccel) {
		if (szAccel[0] == '^') {
			gtk_widget_add_accelerator(widget,
						   "clicked",
						   accel_group,
						   szAccel[1],
						   GDK_CONTROL_MASK,
						   GTK_ACCEL_VISIBLE);
		} else {
			gtk_widget_add_accelerator(widget,
						   "clicked",
						   accel_group,
						   szAccel[0],
						   0, GTK_ACCEL_VISIBLE);
		}
	}

}

// create menu item with accelerator and tooltip
GtkWidget *CreateMenuItem(GtkWidget * menu, char *szName, char *szAccel,
			  char *szTip, GtkSignalFunc func, gpointer data)
{
	GtkWidget *menuitem;
	static GtkTooltips *tooltips = (GtkTooltips *) (NULL);


	if (szName && strlen(szName)) {
		menuitem = gtk_menu_item_new_with_label(szName);
		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
				   GTK_SIGNAL_FUNC(func), data);
	} else {
		menuitem = gtk_menu_item_new();
	}

	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);

	if (accel_group == NULL) {
		accel_group = gtk_accel_group_new();
		gtk_accel_group_attach(accel_group, GTK_OBJECT(win_main));
	}

	if (szAccel) {
		if (szAccel[0] == '^') {
			gtk_widget_add_accelerator(menuitem,
						   "activate",
						   accel_group,
						   szAccel[1],
						   GDK_CONTROL_MASK,
						   GTK_ACCEL_VISIBLE);
		} else {
			gtk_widget_add_accelerator(menuitem,
						   "activate",
						   accel_group,
						   szAccel[0],
						   0, GTK_ACCEL_VISIBLE);
		}
	}

	if (szTip && strlen(szTip)) {
		if (tooltips == NULL) {
			tooltips = gtk_tooltips_new();
		}
		gtk_tooltips_set_tip(tooltips, menuitem, szTip,
				     (gchar *) NULL);
	}
	return (menuitem);
}

// create sub menu
GtkWidget *CreateBarSubMenu(GtkWidget * menu, char *szName)
{
	GtkWidget *menuitem;
	GtkWidget *submenu;

	menuitem = gtk_menu_item_new_with_label(szName);

	gtk_menu_bar_append(GTK_MENU_BAR(menu), menuitem);
	gtk_widget_show(menuitem);

	submenu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);

	return (submenu);
}

// create a widget with xpm 
GtkWidget *CreateWidgetFromXpm(GtkWidget * window, gchar ** xpm_data)
{
	GdkBitmap *mask;
	GdkPixmap *pixmap_data;
	GtkWidget *pixmap_widget;

	pixmap_data = gdk_pixmap_create_from_xpm_d(win_main->window,
						   &mask,
						   (GdkColor *) NULL,
						   (gchar **) xpm_data);

	pixmap_widget = gtk_pixmap_new(pixmap_data, mask);
	gtk_widget_show(pixmap_widget);

	return (pixmap_widget);
}


void selection_made(GtkWidget * clist,
		    gint row,
		    gint column, GdkEventButton * event, gpointer data)
{
	gchar *text;

	gtk_clist_get_text(GTK_CLIST(clist), row, column, &text);

	listn = row;
	set_channel(chlist(row));

	return;
}




void set_list(GtkCList * clist)
{
	gtk_clist_clear(clist);
	for (int i = 0; i < chlist.NumChan(); i++) {
		gchar *chan[2];
		gchar num[5];

		ostrstream snum(num, 5);
		snum << dec << setw(4) << i << ends;

		chan[1] = chlist(i).Name();
		chan[0] = num;
		gtk_clist_append(GTK_CLIST(clist), chan);
	}
}


void add_ch(gpointer data)
{
	cerr << "Sorry not yet implemented" << endl;
}

void del_ch(gpointer data)
{
	GtkCList *clist;

	clist = GTK_CLIST(data);

	chlist.DelChannel(listn);
	set_list(clist);

	if (listn == chlist.NumChan())
		if (listn >= 0)
			listn--;

	if (listn >= 0)
		gtk_clist_select_row(GTK_CLIST(clist1), listn, 0);
}

void load_ch(gpointer data)
{
	GtkCList *clist;

	clist = GTK_CLIST(data);
	char filen[80];

	get_gvideorc(filen);
	cout << "Reading channel list from " << filen << endl;
	chlist.init(filen);
	set_list(clist);

}

void save_ch(gpointer data)
{
	GtkCList *clist;

	clist = GTK_CLIST(data);
	char filen[80];
	char buf[80];
	ostrstream str(buf, 80);

	get_gvideorc(filen);
	str << "Writing channel list from " << filen << endl;
	status_message(buf);

	ofstream fout(filen);
	fout << "# This is the list of favorite channels" << endl;
	fout << "# The format is:" << endl;
	fout << "#   *   = start of a channel declaration" << endl;
	fout << "#  Channel    : <Name>" << endl;
	fout << "#  Frequency  : <frequency> (unit depends on tuner)" <<
	    endl;
	fout << "#  CHBC       : <Color> <Hue> <Brightness> <Contrast> " <<
	    endl;
	fout << "#                (numbers depend on your hardware)    " <<
	    endl;
	fout <<
	    "#  NI         : <TV norm> <Input> (if norm=-1 then SAT is expected";
	fout << endl;
	fout <<
	    "#  SAT        : <Polarization> (0 = vertical, 1 horizontal)"
	    << endl;
	fout << "#               <Signal Rate>" << endl;
	fout <<
	    "#               <FEC> (0=1/2, 1=2/3, 2=3/4, 3=4/5, 4=5/6, 5=6/7"
	    << endl;
	fout << "#                      6=7/8, 7=8/9 8=Auto" << endl;
	fout << "#               <VPID> <APID> <LNB Nr.>" << endl;
	fout << "#               <type> (Analog,DVBS,DVBT,DVBC)=(0,1,2,3)"
	    << endl;
	fout << "#               <dvbnum> id of channel in DVB structure"
	    << endl;
	fout << chlist << endl;
}

// create channel list

GtkWidget *channel_list()
{
	GtkWidget *window1;
	GtkWidget *label1;
	GtkWidget *label2;
	GtkWidget *vbox1, *scrolledwindow1, *hbox1, *hbox2;
	GtkWidget *button_add, *button_del, *button_load, *button_save;

	window1 = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_object_set_data(GTK_OBJECT(window1), "window1", window1);
	gtk_widget_set_usize(window1, 200, 600);
	gtk_window_set_title(GTK_WINDOW(window1), "Channel List");
	gtk_window_set_policy(GTK_WINDOW(window1), FALSE, FALSE, TRUE);
	gtk_window_set_wmclass(GTK_WINDOW(window1), "Chanel List", "");

	gtk_accel_group_attach(accel_group, GTK_OBJECT(window1));


	vbox1 = gtk_vbox_new(FALSE, 4);
	gtk_object_set_data(GTK_OBJECT(window1), "vbox1", vbox1);
	gtk_widget_show(vbox1);
	gtk_container_add(GTK_CONTAINER(window1), vbox1);

	scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_object_set_data(GTK_OBJECT(window1), "scrolledwindow1",
			    scrolledwindow1);
	gtk_widget_show(scrolledwindow1);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolledwindow1, TRUE, TRUE, 0);


	clist1 = gtk_clist_new(2);
	gtk_object_set_data(GTK_OBJECT(clist1), "clist1", clist1);
	gtk_widget_set_usize(clist1, 200, 500);;
	gtk_widget_show(clist1);
	gtk_container_add(GTK_CONTAINER(scrolledwindow1), clist1);
	gtk_clist_set_column_width(GTK_CLIST(clist1), 0, 25);
	gtk_clist_set_column_width(GTK_CLIST(clist1), 1, 80);
	gtk_clist_column_titles_show(GTK_CLIST(clist1));
	gtk_clist_set_shadow_type(GTK_CLIST(clist1),
				  GTK_SHADOW_ETCHED_OUT);

	label1 = gtk_label_new("Nr.");
	gtk_object_set_data(GTK_OBJECT(label1), "label1", label1);
	gtk_widget_show(label1);
	gtk_clist_set_column_widget(GTK_CLIST(clist1), 0, label1);

	label2 = gtk_label_new("Name");
	gtk_object_set_data(GTK_OBJECT(label2), "label2", label2);
	gtk_widget_show(label2);
	gtk_clist_set_column_widget(GTK_CLIST(clist1), 1, label2);


	set_list(GTK_CLIST(clist1));

	gtk_signal_connect(GTK_OBJECT(clist1), "select_row",
			   GTK_SIGNAL_FUNC(selection_made), NULL);

	hbox1 = gtk_hbox_new(FALSE, 2);
	gtk_object_set_data(GTK_OBJECT(vbox1), "hbox1", hbox1);
	gtk_widget_show(hbox1);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, TRUE, TRUE, 0);
	button_add = gtk_button_new_with_label("Add Channel");
	button_del = gtk_button_new_with_label("Delete Channel");
	gtk_widget_set_usize(button_add, 100, 5);;
	gtk_widget_set_usize(button_del, 100, 5);;
	gtk_box_pack_start(GTK_BOX(hbox1), button_add, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), button_del, TRUE, TRUE, 0);

	hbox2 = gtk_hbox_new(FALSE, 2);
	gtk_object_set_data(GTK_OBJECT(vbox1), "hbox2", hbox2);
	gtk_widget_show(hbox2);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox2, TRUE, TRUE, 0);
	button_load = gtk_button_new_with_label("Load Channels");
	button_save = gtk_button_new_with_label("Save Channels");
	gtk_widget_set_usize(button_load, 100, 5);;
	gtk_widget_set_usize(button_save, 100, 5);;
	gtk_box_pack_start(GTK_BOX(hbox2), button_load, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), button_save, TRUE, TRUE, 0);

	/* Connect our callbacks to the four buttons */
	gtk_signal_connect_object(GTK_OBJECT(button_add), "clicked",
				  GTK_SIGNAL_FUNC(add_ch),
				  GTK_OBJECT(clist1));
	gtk_signal_connect_object(GTK_OBJECT(button_del), "clicked",
				  GTK_SIGNAL_FUNC(del_ch),
				  GTK_OBJECT(clist1));
	gtk_signal_connect_object(GTK_OBJECT(button_load), "clicked",
				  GTK_SIGNAL_FUNC(load_ch),
				  GTK_OBJECT(clist1));
	gtk_signal_connect_object(GTK_OBJECT(button_save), "clicked",
				  GTK_SIGNAL_FUNC(save_ch),
				  GTK_OBJECT(clist1));

	gtk_widget_show(button_add);
	gtk_widget_show(button_del);
	gtk_widget_show(button_load);
	gtk_widget_show(button_save);




	return window1;

}
