/*
 * avpes2any tools for the Siemens Fujitsu DVB PCI card
 *
 * Copyright (C) 2000 Marcus Metzler for convergence integrated media GmbH
 * 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 * 

 * The author can be reached at marcus@convergence.de, 

 * the project's page is at http://linuxtv.org/dvb/
 */

#include "cpptools.hh"


#define HEX(N) "0x" << hex << setw(2) << setfill(int('0')) \
<< int(N) << " " << dec
#define HHEX(N,M) "0x" << hex << setw(M) << setfill(int('0')) \
<< int(N) << " " << dec
#define LHEX(N,M) "0x" << hex << setw(M) << setfill(int('0')) \
<< long(N) << " " << dec

#define MAX_SEARCH 1024 * 1024

ostream & operator << (ostream & stream, AV_PES_Packet & x){

	if (x.info){
		stream << "AV_PES Packet: ";
		if (x.p.stream_id == AV_VIDEO) stream << " Video ";
		else stream << " Audio ";
		stream << " Number: " << HEX(x.p.counter) << endl;
		stream << "  length: " << x.Length();
		stream << "  prebytes: " << HEX(x.p.flags & 0x03);
		stream << "  postbytes: " << HEX(x.p.flags & 0x0C);
		if (x.has_pts())
			stream << "   PTS: " 
			       << HHEX(ntohl(*((unsigned int *)
					       x.p.pts)),8);

		stream << endl;
		stream << endl;

		return stream;
	}

	u8 buf[MAX_AV];
	int length = cwrite_av_pes(buf,&(x.p),MAX_AV);
	stream.write(buf,length);


	return stream;
}

istream & operator >> (istream & stream, AV_PES_Packet & x){
	
	u8 sync[2];
	int found=0;
	
	while (!stream.eof() && !found){
		stream.read(&sync,2);
		if (sync[0] == 0x41 && sync[1] == 0x56) 
			found=1; 
		
	}

	stream.read(&x.p.stream_id,1);
	if (x.p.stream_id != AV_VIDEO && x.p.stream_id != AV_AUDIO)
		return stream;
	stream.read(&x.p.counter,1);
	stream.read(&x.p.reserved,1);
	stream.read(&x.p.flags,1);
	stream.read(x.p.llength,2);
	x.setlength();
	if (x.p.flags & PTS_FLAG && (x.p.stream_id == AV_VIDEO)){
		stream.read(x.p.pts,4);
		x.p.length -=4;
		stream.read(x.p.payload,x.p.length);
	} else
		stream.read(x.p.payload,x.p.length);
	if (x.p.flags & PTS_FLAG && (x.p.stream_id == AV_AUDIO))
		x.A_PTS(x.p.payload+7);

	return stream;
}



ostream & operator << (ostream & stream, PES_Packet & x){
  
	if (x.info){
		cerr << "PES Packet: " ;
		switch ( x.p.stream_id ) {
				
		case PROG_STREAM_MAP:
			cerr << "Program Stream Map";
			break;
		case PRIVATE_STREAM2:
			cerr << "Private Stream 2";
			break;
		case PROG_STREAM_DIR:
			cerr << "Program Stream Directory";
			break;
		case ECM_STREAM     :
			cerr << "ECM Stream";
			break;
		case EMM_STREAM     :
			cerr << "EMM Stream";
			break;
		case PADDING_STREAM :
			cerr << "Padding Stream";
			break;
		case DSM_CC_STREAM  :
			cerr << "DSM Stream";
			break;
		case ISO13522_STREAM:
			cerr << "ISO13522 Stream";
			break;
		case PRIVATE_STREAM1:
			cerr << "Private Stream 1";
			break;
		case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			cerr << "Audio Stream " << HEX(x.p.stream_id);
			break;
		case VIDEO_STREAM_S ... VIDEO_STREAM_E:
			cerr << "Video Stream " << HEX(x.p.stream_id);
			break;
			
		}
		cerr << " MPEG" << x.p.mpeg << endl;
		if ( x.p.mpeg == 2 ){
			cerr << "    FLAGS: ";

			if (x.p.flags1 & SCRAMBLE_FLAGS){
				cerr << " SCRAMBLE(";
				cerr << ((x.p.flags1 & SCRAMBLE_FLAGS)>>4);
				cerr << ")";
			}
			if (x.p.flags1 & PRIORITY_FLAG) 
				cerr << " PRIORITY";    
			if (x.p.flags1 & DATA_ALIGN_FLAG)
				cerr << " DATA_ALIGN";  
			if (x.p.flags1 & COPYRIGHT_FLAG) 
				cerr << " COPYRIGHT";   
			if (x.p.flags1 & ORIGINAL_FLAG) 
				cerr << " ORIGINAL";   

			if (x.p.flags2 & PTS_DTS_FLAGS){
				cerr << " PTS_DTS(";
				cerr << ((x.p.flags2 & PTS_DTS_FLAGS)>>6);
				cerr << ")";
			}
			if (x.p.flags2 & ESCR_FLAG) 
				cerr << " ESCR";
			if (x.p.flags2 & ES_RATE_FLAG)
				cerr << " ES_RATE";   
			if (x.p.flags2 & DSM_TRICK_FLAG)
				cerr << " DSM_TRICK";   
			if (x.p.flags2 & ADD_CPY_FLAG)
				cerr << " ADD_CPY";     
			if (x.p.flags2 & PES_CRC_FLAG) 
				cerr << " CRC";     
			if (x.p.flags2 & PES_EXT_FLAG)
				cerr << " EXT";     

			cerr << endl;

			if ((x.p.flags2 & PTS_DTS_FLAGS) == PTS_ONLY)
				cerr << "   PTS: " 
				     << LHEX(ntohl(x.WPTS()),8)
				     << "(h" << int(x.high_pts()) << ")"
				     << endl; 
			else if ((x.p.flags2 & PTS_DTS_FLAGS) == PTS_DTS){
				cerr << "   PTS: " 
				     << LHEX(ntohl(x.WPTS()),8)
				     << "(h" << int(x.high_pts()) << ")";
				cerr << "   DTS: " 
				     << LHEX(ntohl(x.WDTS()),8)
				     << "(h" << int(x.high_dts()) << ")"
				     << endl; 
			}
/*			
			if (x.p.flags2 & ESCR_FLAG)

			
			if (x.p.flags2 & ES_RATE_FLAG)

			
			if (x.p.flags2 & DSM_TRICK_FLAG)

			
			if (x.p.flags2 & ADD_CPY_FLAG)

			
			if (x.p.flags2 & PES_CRC_FLAG)

			
			if (x.p.flags2 & PES_EXT_FLAG){
				
				if (x.p.priv_flags & PRIVATE_DATA)
					stream.write(x.p.pes_priv_data,16);
				
				if (x.p.priv_flags & HEADER_FIELD){
					stream.write(&x.p.pack_field_length,1);
					x.p.pack_header = new 
						u8[x.p.pack_field_length];
					stream.write(x.p.pack_header,
						     x.p.pack_field_length);
				}
				
				if ( x.p.priv_flags & PACK_SEQ_CTR){
					stream.write(&x.p.pck_sqnc_cntr,1);
					stream.write(&x.p.org_stuff_length,1);
				}
				
				if ( x.p.priv_flags & P_STD_BUFFER)
					stream.write(x.p.p_std,2);
				
				if ( x.p.priv_flags & PES_EXT_FLAG2){
					stream.write(&x.p.pes_ext_lngth,1);
					x.p.pes_ext = new 
						u8[x.p.pes_ext_lngth];
					stream.write(x.p.pes_ext,
						     x.p.pes_ext_lngth);
				}
			}
		} else {
			if ((x.p.flags2 & PTS_DTS_FLAGS) == PTS_ONLY)
				stream.write(x.p.pts,5);
			else if ((x.p.flags2 & PTS_DTS_FLAGS) == 
				 PTS_DTS){
				stream.write(x.p.pts,5);
				stream.write(x.p.dts,5);
			}
*/	
		}			
		cerr << endl << endl;
		return stream;
	}

	int l = x.p.length+x.p.pes_hlength;
	u8 buf[l];
	int length = cwrite_pes(buf,&(x.p),l);
	stream.write(buf,length);
	
	return stream;
}

static unsigned int find_length(istream & stream){
	streampos p = 0;
	streampos start = 0;
	streampos q = 0;
	int found = 0;
	u8 sync4[4];

	start = stream.tellg();
	start -=2;
	stream.seekg(start);
	while ( !stream.eof() && !found ){
		p = stream.tellg();
		stream.read(&sync4,4);
		if (sync4[0] == 0x00 && sync4[1] == 0x00 && sync4[2] == 0x01) {
			switch ( sync4[3] ) {
				
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				found = 1;
				break;
			default:
				q = stream.tellg();
				break;
			}	
		} 
	}
	q = stream.tellg();
	stream.seekg(start+2);
	if (found) return (unsigned int)(q-start)-4-2;
	else return (unsigned int)(q-start-2);
	
}

istream & operator >> (istream & stream, PES_Packet & x){
	
	u8 sync4[4];
	int found=0;
	int done=0;
	streampos p = 0;
	
	while (!stream.eof() && !found) {
	        p = stream.tellg();
		stream.read(&sync4,4);
		if (sync4[0] == 0x00 && sync4[1] == 0x00 && sync4[2] == 0x01) {
			x.p.stream_id = sync4[3];

			switch ( sync4[3] ) {
				
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
				found = 1;
				stream.read(x.p.llength,2);
				x.setlength();
				if (!x.p.length){ 
					x.p.length = find_length(stream);
					x.Nlength();
				}
				stream.read(x.p.pes_pckt_data,x.p.length);
				done = 1;
				break;
			case PADDING_STREAM :
				found = 1;
				stream.read(x.p.llength,2);
				x.setlength();
				if (!x.p.length){ 
					x.p.length = find_length(stream);
					x.Nlength();
				}
				x.p.padding = x.p.length;
				stream.read(x.p.pes_pckt_data,x.p.length);
				done = 1;
				break;
				
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				stream.read(x.p.llength,2);
				x.setlength();
				if (!x.p.length){ 
					x.p.length = find_length(stream);
					x.Nlength();
				}
				found = 1;
				break;
				
			default:
				stream.seekg(p+1);
			break;
			}	
		} else stream.seekg(p+1);
	}
	
	if ( found && !done) {
		p = stream.tellg();
		stream.read(&x.p.flags1,1);
		if ( (x.p.flags1 & 0xC0) == 0x80 )
			x.p.mpeg = 2;
		else
			x.p.mpeg = 1;
		if ( x.p.mpeg == 2 ){
			stream.read(&x.p.flags2,1);
			stream.read(&x.p.pes_hlength,1);
			
			x.p.length -=x.p.pes_hlength+3;
			u8 count = x.p.pes_hlength;

			if ((x.p.flags2 & PTS_DTS_FLAGS) == PTS_ONLY){
				stream.read(x.p.pts,5);
				count -=5;
			} else 
				if ((x.p.flags2 & PTS_DTS_FLAGS) == PTS_DTS){
					stream.read(x.p.pts,5);
					stream.read(x.p.dts,5);
					count -= 10;
				}

			if (x.p.flags2 & ESCR_FLAG){
				stream.read(x.p.escr,6);
				count -= 6;
			}

			if (x.p.flags2 & ES_RATE_FLAG){
				stream.read(x.p.es_rate,3);
				count -= 6;
			}

			if (x.p.flags2 & DSM_TRICK_FLAG){
				stream.read(&x.p.trick,1);
				count -= 1;
			}

			if (x.p.flags2 & ADD_CPY_FLAG){
				stream.read(&x.p.add_cpy,1);
				count -= 1;
			}

			if (x.p.flags2 & PES_CRC_FLAG){
				stream.read(x.p.prev_pes_crc,2);
				count -= 2;
			}			

			if (x.p.flags2 & PES_EXT_FLAG){
				stream.read(&x.p.priv_flags,1);
				count -= 1;
				
				if (x.p.priv_flags & PRIVATE_DATA){
					stream.read(x.p.pes_priv_data,16);
					count -= 16;
				}

				if (x.p.priv_flags & HEADER_FIELD){
					stream.read(&x.p.pack_field_length,1);
					x.p.pack_header = new 
						u8[x.p.pack_field_length];
					stream.read(x.p.pack_header,
						    x.p.pack_field_length);
					count -= 1+x.p.pack_field_length;
				}
				
				if ( x.p.priv_flags & PACK_SEQ_CTR){
					stream.read(&x.p.pck_sqnc_cntr,1);
					stream.read(&x.p.org_stuff_length,1);
					count -= 2;
				}

				if ( x.p.priv_flags & P_STD_BUFFER){
					stream.read(x.p.p_std,2);
					count -= 2;
				}

				if ( x.p.priv_flags & PES_EXT_FLAG2){
					stream.read(&x.p.pes_ext_lngth,1);
					x.p.pes_ext = new 
						u8[x.p.pes_ext_lngth];
					stream.read(x.p.pes_ext,
						    x.p.pes_ext_lngth);
					count -= 1+x.p.pes_ext_lngth;
				}
			}
			x.p.stuffing = count;
			u8 dummy;
			for(int i = 0; i< count ;i++) 
				stream.read(&dummy,1);
			
		} else {
			u8 check;
			x.p.mpeg1_pad = 1;
			check = x.p.flags1;
			while (check == 0xFF){
				stream.read(&check,1);
				x.p.mpeg1_pad++;
			}
		    
			if ( (check & 0xC0) == 0x40){
				stream.read(&check,1);
				x.p.mpeg1_pad++;
				stream.read(&check,1);
				x.p.mpeg1_pad++;
			}
			x.p.flags2 = 0;
			x.p.length -= x.p.mpeg1_pad;

			stream.seekg(p);
			if ( (check & 0x30)){
				x.p.length ++;
				x.p.mpeg1_pad --;

				if (check == x.p.flags1){
					x.p.pes_hlength = 0;
				} else {
					x.p.mpeg1_headr = 
						new u8[x.p.mpeg1_pad];
					x.p.pes_hlength = x.p.mpeg1_pad;
					stream.read(x.p.mpeg1_headr,
						    x.p.mpeg1_pad);
				}

				x.p.flags2 = (check & 0xF0) << 2;
				if ((x.p.flags2 & PTS_DTS_FLAGS) == PTS_ONLY){
					stream.read(x.p.pts,5);
					x.p.length -= 5;
					x.p.pes_hlength += 5;
				}
				else if ((x.p.flags2 & PTS_DTS_FLAGS) == 
					 PTS_DTS){
					stream.read(x.p.pts,5);
					stream.read(x.p.dts,5);
					x.p.length -= 10;
					x.p.pes_hlength += 10;
				}
			} else {
				x.p.mpeg1_headr = new u8[x.p.mpeg1_pad];
				x.p.pes_hlength = x.p.mpeg1_pad;
				stream.read(x.p.mpeg1_headr,x.p.mpeg1_pad);
			}
		}
		stream.read(x.p.pes_pckt_data,x.p.length);
	}
	return stream;
}

ostream & operator << (ostream & stream, TS_Packet & x){

	u8 buf[TS_SIZE];
	int length = cwrite_ts(buf,&(x.p),TS_SIZE);
	stream.write(buf,length);

	return stream;
}

istream & operator >> (istream & stream, TS_Packet & x){
	u8 sync;
	int found=0;
	streampos p,q;

	sync=0;
	while (!stream.eof() && !found) {
		stream.read(&sync,1);
		if (sync == 0x47) 
			found = 1;
	}
	stream.read(x.p.pid,2);
	stream.read(&x.p.flags,1);
	x.p.count = x.p.flags & COUNT_MASK;
	 
	if (!(x.p.flags & ADAPT_FIELD) && (x.p.flags & PAYLOAD)){
		//no adapt. field only payload
		stream.read(x.p.data,184);
		x.p.rest = 184;
		return stream;
	} 

	if ( x.p.flags & ADAPT_FIELD ) {
		// adaption field
		stream.read(&x.p.adapt_length,1);
		p = stream.tellg();
		stream.read(&x.p.adapt_flags,1);

		if ( x.p.adapt_flags & PCR_FLAG )
			stream.read( x.p.pcr,6);

		if ( x.p.adapt_flags & OPCR_FLAG )
			stream.read( x.p.opcr,6);

		if ( x.p.adapt_flags & SPLICE_FLAG )
			stream.read( &x.p.splice_count,1);

		if( x.p.adapt_flags & TRANS_PRIV){
			stream.read(&x.p.priv_dat_len,1);
			x.p.priv_dat = new u8[x.p.priv_dat_len];
			stream.read(x.p.priv_dat,x.p.priv_dat_len);
		}
			
		if( x.p.adapt_flags & ADAP_EXT_FLAG){
			stream.read(&x.p.adapt_ext_len,1);
			stream.read(&x.p.adapt_eflags,1);
			if( x.p.adapt_eflags & LTW_FLAG)
				stream.read(x.p.ltw,2);
			
			if( x.p.adapt_eflags & PIECE_RATE)
				stream.read(x.p.piece_rate,3);
			
			if( x.p.adapt_eflags & SEAM_SPLICE)
				stream.read(x.p.dts,5);
		}
		q = stream.tellg();
		x.p.stuffing = x.p.adapt_length -(q-p);
		x.p.rest = 183-x.p.adapt_length;
		stream.seekg(q+x.p.stuffing);
		if (x.p.flags & PAYLOAD) // payload
			stream.read(x.p.data,x.p.rest);
		else 
			stream.seekg(q+x.p.rest);
	}
	return stream;
}


ostream & operator << (ostream & stream, PS_Packet & x){

	u8 buf[PS_MAX];
	int length = cwrite_ps(buf,&(x.p),PS_MAX);
	stream.write(buf,length);

	return stream;
}

istream & operator >> (istream & stream, PS_Packet & x){
	u8 headr[4];
	int found=0;
	streampos p = 0;
	streampos q = 0;
	int count = 0;

	p = stream.tellg();
	while (!stream.eof() && !found && count < MAX_SEARCH) {
		stream.read(&headr,4);
		if (headr[0] == 0x00 && headr[1] == 0x00 && headr[2] == 0x01)
			if ( headr[3] == 0xBA ) 
				found = 1;
			else if ( headr[3] == 0xB9 ) break;
			else stream.seekg(p+1);
		count++;
	}
	
	if (found){
		stream.read(x.p.scr,6);
		if (x.p.scr[0] & 0x40)
			x.p.mpeg = 2;
		else
			x.p.mpeg = 1;

		if (x.p.mpeg == 2){
			stream.read(x.p.mux_rate,3);
			stream.read(&x.p.stuff_length,1);
			p = stream.tellg();
			stream.seekg(p+(x.p.stuff_length & 3));
		} else {
			x.p.mux_rate[0] = x.p.scr[5]; //mpeg1 scr is only 5 bytes
			stream.read(x.p.mux_rate+1,2);
		}
			
		p=stream.tellg();
		stream.read(headr,4);
		if (headr[0] == 0x00 && headr[1] == 0x00 && 
		    headr[2] == 0x01 && headr[3] == 0xBB ) {
			stream.read(x.p.sheader_llength,2);
			x.setlength();
			if (x.p.mpeg == 2){
				stream.read(x.p.rate_bound,3);
				stream.read(&x.p.audio_bound,1);
				stream.read(&x.p.video_bound,1);
				stream.read(&x.p.reserved,1);
			}
			stream.read(x.p.data,x.p.sheader_length);
		} else {
			stream.seekg(p);
			x.p.sheader_length = 0;
		}

		int i = 0;
		int done = 0;
		q = stream.tellg();
		PES_Packet pes;
		do {
			p=stream.tellg();
			stream.read(headr,4);
			stream.seekg(p);
			if ( headr[0] == 0x00 && headr[1] == 0x00 
			     && headr[2] == 0x01 && headr[3] != 0xBA){
				pes.init();
				stream >> pes;
			        i++;
			} else done = 1;
		} while (!stream.eof() && !done);
		x.p.npes = i;
		stream.seekg(q);
	} 
 	return stream;
}

void extract_audio_from_PES(istream &in, ostream &out){
	PES_Packet pes;
	
	while(!in.eof()){
		pes.init();
		in >> pes ;
		if (pes.Stream_ID() == 0xC0)
			out << pes;
	}
}

void extract_video_from_PES(istream &in, ostream &out){
	PES_Packet pes;
	
	while(!in.eof()){
		pes.init();
		in >> pes ;
		if (pes.Stream_ID() == 0xE0)
			out << pes;
	}
}

void extract_es_audio_from_PES(istream &in, ostream &out){
	PES_Packet pes;
	
	while(!in.eof()){
		pes.init();
		in >> pes ;
		if (pes.Stream_ID() == 0xC0)
		  out.write(pes.Data(),pes.Length());
	}
}

void extract_es_video_from_PES(istream &in, ostream &out){
	PES_Packet pes;
	
	while(!in.eof()){
		pes.init();
		in >> pes ;
		if (pes.Stream_ID() == 0xE0)
		  out.write(pes.Data(),pes.Length());
	}
}

void extract_audio_from_AV_PES(istream &in, ostream &out){
	AV_PES_Packet aps;
	
	while(!in.eof()){
		aps.init();
		in >> aps ;
		if (aps.Stream_ID() == AV_AUDIO)
			out.write(aps.Data(),aps.Length());
	}
}


void extract_video_from_AV_PES(istream &in, ostream &out){
	
	AV_PES_Packet aps;
	
	while(!in.eof()){
		aps.init();
		in >> aps ;
		if (aps.Stream_ID() == AV_VIDEO)
			out.write(aps.Data(),aps.Length());
	}
}


static void write_video(AV_PES_Packet &apsv, u8 &countv,ostream &out){
	int gap = apsv.Length() & 3;
	apsv.Gap() = gap;
	apsv.Counter() = countv;
	apsv.Length()-=gap;
	out << apsv;
	if (apsv.Length()) countv++;
	if (gap) memmove(apsv.Data(), apsv.Data()+apsv.Length(),gap);
	apsv.Flags() = gap;
	apsv.Length() = gap;
}

static int write_vpes(PES_Packet &pes, AV_PES_Packet &apsv, ostream &out,
		      int p){
	static u8 countv = 0;
	static int markv = 0;
	int lengthv = pes.Length()-markv;
	u8 *posv = pes.Data()+markv;

	if(p == 4){
		if (apsv.Length() > (apsv.Flags()& 0x03)){
			apsv.Flags() &= ~PTS_FLAG;
			write_video(apsv,countv,out);
			apsv.Flags() |= PTS_FLAG;
		}
	}

	if (apsv.Flags() & PTS_FLAG) p = 4;
	if (lengthv+apsv.Length() < MAX_AV-p ){
		memcpy(apsv.Data()+apsv.Length(),posv,lengthv);
		apsv.Length() += lengthv;
		markv = 0;
		return 1;
	} else {
		int l= MAX_AV-apsv.Length()-p;
		memcpy(apsv.Data()+apsv.Length(),posv,l);
		markv += l;
		apsv.Length() += l;		
		write_video(apsv,countv,out);
		return 0;
	}
}

static void write_audio(AV_PES_Packet &apsa, u8 &counta, ostream &out){
	apsa.Counter() = counta;
	out << apsa;
	if (apsa.Length()) counta++;
	apsa.Flags() = 0;
	apsa.Length() = 0;
}

static int write_apes(u8 *apes,AV_PES_Packet &apsa, 
		      int &length,ostream &out) {
	static int marka = 0;
	static u8 counta = 0;
	int lengtha = length-marka;
	u8 *posa = apes+marka;
	
	if (lengtha+apsa.Length() <= MAX_AVA){
		memcpy(apsa.Data()+apsa.Length(),posa,lengtha);
		apsa.Length() += lengtha;
		write_audio(apsa, counta,out);
		marka = 0;
		return 1;
	} else {
		int l = MAX_AVA-apsa.Length(); 
		memcpy(apsa.Data()+apsa.Length(),posa,l);
		marka += l;
		apsa.Length() += l; 
		write_audio(apsa, counta,out);
		return 0;
	}
}

static void pes_to_AV_PES(AV_PES_Packet &apsv,AV_PES_Packet &apsa,
			 istream &in,ostream &out){
	PES_Packet pes;
	int p = 0;
	int length;

	pes.init();
	in >> pes;
	int done = 0;
	switch (pes.Stream_ID()){
	case VIDEO_STREAM_S ... VIDEO_STREAM_E:
		if (pes.has_pts()){
			pes.AV_PTS(apsv);
			p = 4;
		} else 	p = 0;

		
		while (!done){
			done = write_vpes(pes,apsv,out,p);
			p = 0;
		}
		break;

	case AUDIO_STREAM_S ... AUDIO_STREAM_E:
		
		length = pes.Length()+pes.HLength()+9;
		if (length){
			u8 *apes;
			apes =new u8 [length];
			cwrite_pes(apes,pes.P(),length);
			if (pes.has_pts() && pes.MPEG() == 2)
				apsa.Flags() |= PTS_FLAG;
			
			
			while (!done){
				done = write_apes(apes,apsa
						  ,length,out);
			}
			delete [] apes;
		}
		break;
	
	default:
		done = 1;
	}
}

void PES_to_AV_PES(istream &in, ostream &out){
	AV_PES_Packet apsv;
	AV_PES_Packet apsa;

	apsv.Stream_ID() = AV_VIDEO;
	apsa.Stream_ID() = AV_AUDIO;
	while(!in.eof()){
		pes_to_AV_PES(apsv,apsa,in,out);
	}
	if (apsv.Length()){
		out << apsv;
	}
	if (apsa.Length()){
		out << apsa;
	}
}

void PS_to_AV_PES(istream &in, ostream &out){

	AV_PES_Packet apsv;
	AV_PES_Packet apsa;
	PS_Packet ps;


	apsv.Stream_ID() = AV_VIDEO;
	apsa.Stream_ID() = AV_AUDIO;
	while(!in.eof()){
		ps.init();
		in >> ps;
		for(int j=0;j < ps.NPES();j++){
			pes_to_AV_PES(apsv,apsa,in,out);
		}
	}
	if (apsv.Length()){
		out << apsv;
	}
	if (apsa.Length()){
		out << apsa;
	}
}


#define BUFL  65536+65536

void AV_PES_to_PES(istream &in, ostream &out){
	PES_Packet vpes;
	AV_PES_Packet aps;
	int posa = 0;
	int posv = 0;
	int npesha = 0;
	char bufa[BUFL];
	char bufv[BUFL];
	

	while (!in.eof()) {
		aps.init();
		in >> aps;
		switch ( aps.Stream_ID()){
		case AV_AUDIO:
		{
			if (aps.Length()+posa < BUFL){
				memcpy(bufa+posa,aps.Data(),aps.Length());
				posa+=aps.Length();
			} else {
				cerr << "Buffer overflow in AVPES2PES" << endl;
				exit(1);
			}
			int found = 0;
			int j = 0;
			for (j = 4*npesha; j < posa-4 && found == 0 ; j++){
				if (bufa[j] == 0x00 && bufa[j+1] == 0x00 &&
				    bufa[j+2] == 0x01){
					switch ( u8(bufa[j+3]) ) {
					case AUDIO_STREAM_S ... AUDIO_STREAM_E:
						found = 1;
						break;
					default:
						found = 0;
						break;
					}
				}
			}
			j--;
			
			if (found){
				npesha++;
				if (npesha < 2){
					memmove(bufa,bufa+j,posa-j);
					posa -= j;
				} else {
					istrstream ins(bufa,j+4);
					PES_Packet pes;
					ins >> pes;
					out << pes;
					memmove(bufa,bufa+j,posa-j);
					posa -= j;
					npesha = 1;
				} 
			}
			
		}
		break;
		
		case AV_VIDEO:
			if (aps.has_pts()){
				if (posv > 0){
					vpes.init();
					vpes.MPEG() = 2;
					vpes.Get_PTS(aps);
					vpes.Length() = posv;
					vpes.Nlength();
					vpes.HLength()=0x05;
					memcpy(vpes.Data(),bufv,posv);
					posv = 0;
					vpes.Stream_ID() = 0xE0;
					out << vpes;
				}
				vpes.init();
				vpes.Get_PTS(aps);
			}
				
			if (aps.Length()+posv < BUFL){
				memcpy(bufv+posv,aps.Data(),aps.Length());
				posv+=aps.Length();
			} else {
				cerr << "Buffer overflow in AVPES2PES" << endl;
				exit(1);
			}
			
			if (posv > BUFL/2){
				vpes.Length() = posv;
				vpes.Nlength();
				memcpy(vpes.Data(),bufv,posv);
				posv = 0;
				vpes.Stream_ID() = 0xE0;
				out << vpes;
				vpes.init();
			}				
			
			break;
			
		}
		
	}
	
}

void TS_to_AV_PES(istream &in, ostream &out,short pid1, short pid2){

	PES_Packet vpes;
	TS_Packet ts;
	int posa = 0;
	int npesha = 0;
	char bufa[BUFL];
	int posb = 0;
	int npeshb = 0;
	char bufb[BUFL];
	AV_PES_Packet apsv;
	AV_PES_Packet apsa;

	apsv.Stream_ID() = AV_VIDEO;
	apsa.Stream_ID() = AV_AUDIO;

	while (!in.eof()) {
		ts.init();
		in >> ts;
		if ( ts.PID() == pid1){
			if ( ts.FLAG1() & PAY_START ){
				if (npesha){
					memcpy(bufa+posa,ts.Data(),4);
					istrstream ins(bufa,posa+4);
					pes_to_AV_PES(apsv,apsa,ins,out);
				}
				npesha = 1;
				posa = 0;
			}
				
			if (ts.Rest()+posa < BUFL){
				memcpy(bufa+posa,ts.Data(),ts.Rest());
				posa+=ts.Rest();
			} else {
				cerr << "Buffer overflow in TS2PES" << endl;
				exit(1);
			}
		} else if ( ts.PID() == pid2 ){
			if ( ts.FLAG1() & PAY_START ){
				if (npeshb){
					memcpy(bufb+posb,ts.Data(),4);
					istrstream ins(bufb,posb+4);
					pes_to_AV_PES(apsv,apsa,ins,out);
				}
				npeshb = 1;
				posb = 0;
			}
				
			if (ts.Rest()+posb < BUFL){
				memcpy(bufb+posb,ts.Data(),ts.Rest());
				posb+=ts.Rest();
			} else {
				cerr << "Buffer overflow in TS2PES" << endl;
				exit(1);
			}
		}
	}
	if (apsv.Length()){
		out << apsv;
	}
	if (apsa.Length()){
		out << apsa;
	}
}


void TS_to_PES(istream &in, ostream &out,short pid1, short pid2){

	PES_Packet vpes;
	TS_Packet ts;
	int posa = 0;
	int npesha = 0;
	char bufa[BUFL];
	int posb = 0;
	int npeshb = 0;
	char bufb[BUFL];

	while (!in.eof()) {
		ts.init();
		in >> ts;
		if ( ts.PID() == pid1){
			if ( ts.FLAG1() & PAY_START ){
				if (npesha){
					memcpy(bufa+posa,ts.Data(),4);
					istrstream ins(bufa,posa+4);
					PES_Packet pes;
					ins >> pes;
					out << pes;
				}
				npesha = 1;
				posa = 0;
			}
				
			if (ts.Rest()+posa < BUFL){
				memcpy(bufa+posa,ts.Data(),ts.Rest());
				posa+=ts.Rest();
			} else {
				cerr << "Buffer overflow in TS2PES" << endl;
				exit(1);
			}
		} else if ( ts.PID() == pid2 ){
			if ( ts.FLAG1() & PAY_START ){
				if (npeshb){
					memcpy(bufb+posb,ts.Data(),4);
					istrstream ins(bufb,posb+4);
					PES_Packet pes;
					ins >> pes;
					out << pes;
				}
				npeshb = 1;
				posb = 0;
			}
				
			if (ts.Rest()+posb < BUFL){
				memcpy(bufb+posb,ts.Data(),ts.Rest());
				posb+=ts.Rest();
			} else {
				cerr << "Buffer overflow in TS2PES" << endl;
				exit(1);
			}
		}
	}
}

#define MAX_PID 20
int TS_PIDS(istream &in, ostream &out){
	int pid[MAX_PID];
	TS_Packet ts;
	int npid=0;
	
	for (int i=0 ; i<MAX_PID; i++)
		pid[i] = -1;
	while (!in.eof()) {
		ts.init();
		in >> ts;
		int j;
		int found = 0;
		for (j=0;j<npid;j++){
			if ( ts.PID() == pid[j] )
				found = 1;
		}
		if (! found){ 
			out << ts.PID() << " ";
			pid[npid] = ts.PID();
			npid++;
			if (npid == MAX_PID) return -1;
		}
	}
	out << endl;
	return 0;
}

// TODO
void AV_PES_to_PS(istream &in, ostream &out){
	PES_Packet vpes;
	PES_Packet apes;
	PS_Packet  ps;
	AV_PES_Packet *avp;

	ps.P()->audio_bound = 0x04;
	ps.P()->video_bound = 0xE1;
	ps.P()->reserved    = 0xFF;
	
	while (!in.eof()){
		avp = new AV_PES_Packet;
		in >> *avp;
		switch(avp->Stream_ID()){
		case AV_VIDEO:
			break;
		case AV_AUDIO:
			
			break;
		}
	}
	
}


pid_t pidin;

int ifilter (istream &in, FILTER function){

	int fd[2];
	
	if ( pipe(fd) < 0 ){
		cerr << "Pipe error" << endl;
		exit(1);
	}
	
	if ( (pidin = fork()) < 0 ){
		cerr << "Fork error" << endl;
		exit(1);
	} else if (pidin>0) {      /* parent */
		close(fd[1]);
		
		return fd[0];
	} else {
		close(fd[0]);
		ofstream out(fd[1]);
		function(in,out);
		close(fd[1]);
		return -1;
	}
	
  
}

pid_t pidout;

int ofilter (ostream &out, FILTER function){

	int fd[2];

	if ( pipe(fd) < 0 ){
		cerr << "Pipe error" << endl;
		exit(1);
	}
	
	if ( (pidout = fork()) < 0 ){
		cerr << "Fork error" << endl;
		exit(1);
	} else if (pidout>0) {      /* parent */
		close(fd[0]);
		return fd[1];
	} else {
		close(fd[1]);
		ifstream in(fd[0]);
		function(in,out);
		close(fd[1]);
		return -1;
	}
	
	
  
}


static void *ifilter_thread(void *args){
	thread_args *targs = (thread_args *) args;

	int status = pthread_detach(pthread_self());
	if (status)
		cerr << "Thread detach " <<  status << endl;
	ofstream out(targs->fd[1]);
	ifstream in(targs->in);
	targs->function(in,out);

	in.close();
	out.close();
	close(targs->fd[0]);
	close(targs->fd[1]);

	delete (targs->fd);
	delete (targs);
	pthread_exit(NULL);
	return NULL;
}

int itfilter (int fdin, FILTER function){
	pthread_t thread1;
	int *fd = new int[2];
	thread_args * targs= new thread_args;
	void *args = (void * )targs;

	targs->function = function;
	targs->in = fdin;
	targs->fd = fd;

	if ( pipe(fd) < 0 ){
		cerr << "Pipe error" << endl;
		exit(1);
	}

	int status = pthread_create(&thread1, NULL, ifilter_thread, args);
	if (status)
		cerr << "Thread detach " << status << endl;

	return fd[0];
}

static void *ofilter_thread(void *args){
	thread_args *targs = (thread_args *) args;

	int status = pthread_detach(pthread_self());
	if (status)
		cerr << "Thread detach " <<  status << endl;

	ofstream out(targs->out);
	ifstream in(targs->fd[0]);
	targs->function(in,out);

	in.close();
	out.close();
	close(targs->fd[0]);
	close(targs->fd[1]);

	delete (targs->fd);
	delete (targs);
	pthread_exit(NULL);
	return NULL;
}


int otfilter (int fdout, FILTER function){
	pthread_t thread1;
	int *fd = new int[2];
	thread_args * targs= new thread_args;
	void *args = (void * )targs;

	targs->function = function;
	targs->out = fdout;
	targs->fd = fd;

	if ( pipe(fd) < 0 ){
		cerr << "Pipe error" << endl;
		exit(1);
	}

	int status = pthread_create(&thread1, NULL, ofilter_thread, args);

	if (status)
		cerr << "Thread detach " << status << endl;

	

	return fd[1];
}


int stream_type(int fd){
	ifstream fin(fd);
	return stream_type(fin);
}


int tv_norm(istream &stream){
	u8 headr[4];
	int found=0;
	streampos p = 0;
	streampos q = 0;
	int hsize,vsize;
	int form;

	q = stream.tellg();
	while (!stream.eof() && !found) {
		p = stream.tellg();
		stream.read(headr,4);
		if (headr[0] == 0x00 && headr[1] == 0x00 && headr[2] == 0x01)
			if ( headr[3] == 0xB3 ){ 
				found = 1;
			}
		if (! found) stream.seekg(p+1);
	}
	stream.read(headr,4);

	hsize = (headr[1] &0xF0) >> 4 | headr[0] << 4;
	vsize = (headr[1] &0x0F) << 8 | headr[2];
	cerr << "SIZE: " << hsize << "x" << vsize << endl;
	
	switch((headr[3]&0xF0 >>4)){
	case 1:
		cerr << "ASPECT: 1:1" << endl;
		break;
	case 2:
		cerr << "ASPECT: 4:3" << endl;
		break;
	case 3:
		cerr << "ASPECT: 16:9" << endl;
		break;
	case 4:
		cerr << "ASPECT: 2.21:1" << endl;
		break;
	}

	switch (int(headr[3]&0x0F)){
	case 1:
		cerr << "FRAMERATE: 23.976" << endl;
		form = pDUNNO;
		break;
	case 2:
		cerr << "FRAMERATE: 24" << endl;
		form = pDUNNO;
		break;
	case 3:
		cerr << "FRAMERATE: 25" << endl;
		form = pPAL;
		break;
	case 4:
		cerr << "FRAMERATE: 29.97" << endl;
		form = pNTSC;
		break;
	case 5:
		cerr << "FRAMERATE: 30" << endl;
		form = pNTSC;
		break;
	case 6:
		cerr << "FRAMERATE: 50" << endl;
		form = pPAL;
		break;
	case 7:
		cerr << "FRAMERATE: 60" << endl;
		form = pNTSC;
		break;
	}

	int mpeg = 0;
	found = 0;
	while (!stream.eof() && !found) {
		p = stream.tellg();
		stream.read(headr,4);
		if (headr[0] == 0x00 && headr[1] == 0x00 && headr[2] == 0x01)
			if ( headr[3] == 0xB5 ){ 
				mpeg = 2;
				found = 1;
			} else {
				mpeg = 1;
				found = 1;
			}				
		if (! found) stream.seekg(p+1);
	}

	stream.seekg(q);
	return (form | mpeg << 4);
}



int stream_type(istream &fin){
	u8 headr[4];
	streampos p=fin.tellg();

	AV_PES_Packet apes;
	fin >> apes;
	fin.read(headr,4);
	fin.seekg(p);
	if (fin && headr[0] == 0x41 && headr[1] == 0x56){
		if (headr[2] == AV_VIDEO || headr[2] == AV_AUDIO){
			return AV_PES_STREAM;
		}
	}
	
	TS_Packet ts;
	fin >> ts;
	fin.read(headr,1);
	fin.seekg(p);
	if(fin && headr[0] == 0x47){
		return TS_STREAM;
	}

	PS_Packet ps;
	fin >> ps;
	PES_Packet pes;
	for(int j=0;j < ps.NPES();j++){
		fin >> pes;
	}
	fin.read(headr,4);
	fin.seekg(p);
	if (fin && headr[0] == 0x00 && headr[1] == 0x00
	    && headr[2] == 0x01 && headr[3] == 0xBA){
		return PS_STREAM;
	}
	
	fin >> pes;
	fin.read(headr,4);
			fin.seekg(p);
	if (fin && headr[0] == 0x00 && headr[1] == 0x00
	    && headr[2] == 0x01 ){
		int found = 0;
		switch ( headr[3] ) {
			
		case PROG_STREAM_MAP:
		case PRIVATE_STREAM2:
		case PROG_STREAM_DIR:
		case ECM_STREAM     :
		case EMM_STREAM     :
		case PADDING_STREAM :
		case DSM_CC_STREAM  :
		case ISO13522_STREAM:
		case PRIVATE_STREAM1:
		case AUDIO_STREAM_S ... AUDIO_STREAM_E:
		case VIDEO_STREAM_S ... VIDEO_STREAM_E:
			found = 1;
			break;
		}
		if (found){
			return PES_STREAM;
		}
	}

	

	return -1;
}


void analyze(istream &fin)
{
	PS_Packet ps;
	PES_Packet pes;
	int fc = 0;
	char *frames[3] = {"I-Frame","P-Frame","B-Frame"};
			
	while(fin){
		u32 pts;
		fin >> ps;
		cerr << "SCR base: " << hex << setw(5) 
		     << setfill(int('0')) \
		     << ps.SCR_base() << " " << dec
		     << "ext : " << ps.SCR_ext();

		cerr << "   MUX rate: " << ps.MUX()*50*8/1000000.0
		     << " Mbit/s   ";
		cerr << "RATE bound: " << ps.Rate()*50*8/1000000.0
		     << " Mbit/s" << endl;
		cerr << "                             Audio bound: " 
		     << hex << "0x" 
		     << int(ps.P()->audio_bound);
		cerr << "         Video bound: " << hex << "0x" 
		     << int(ps.P()->video_bound)
		     << dec
		     << endl;
		cerr << endl;

		for (int i=0; i < ps.NPES(); i++){
			pes.init();
			fin >> pes;
			pts2pts((u8 *)&pts,pes.PTS());
			pes.Info() = 1;
			cerr  << pes;

			u8 *buf = pes.P()->pes_pckt_data;
			int c = 0;
			int l;
			switch (pes.P()->stream_id){
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				l=pes.P()->length;
				break;
			default:
				l = 0;
				break;
			}
			while ( c < l - 6){
				if (buf[c] == 0x00 && 
				    buf[c+1] == 0x00 &&
				    buf[c+2] == 0x01 && 
				    buf[c+3] == 0xB8) {
					c += 4;
					cerr << "TIME   hours: " 
					     << int((buf[c]>>2)& 0x1F)
					     << "  minutes: "
					     << int(((buf[c]<<4)& 0x30)|
						    ((buf[c+1]>>4)& 0x0F))
					     << " seconds: "
					     << int(((buf[c+1]<<3)& 0x38)|
						    ((buf[c+2]>>5)& 0x0F))
					     << endl;
				}
				
				if ( buf[c] == 0x00 && 
				     buf[c+1] == 0x00 &&
				     buf[c+2] == 0x01 && 
				     buf[c+3] == 0x00) {
					fc++;
					c += 4;
					cerr << "picture: " 
					     << fc 
					     << " ("
					     << frames[((buf[c+1]&0x38) >>3)-1]
					     << ")" << endl << endl;
				} else c++;
			}
		}
	}
}
