/*
 * avpes2any tools for the Siemens Fujitsu DVB PCI card
 *
 * Copyright (C) 2000 Marcus Metzler for convergence integrated media GmbH
 * 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 * 

 * The author can be reached at marcus@convergence.de, 

 * the project's page is at http://linuxtv.org/dvb/
 */

#include "ctools.h"

#define MAX_SEARCH 1024 * 1024


/*

      AV_PES
  
*/
u32 trans_pts_dts(u8 *pts){
	u32 w_dts;
	u8 *av_dts = (u8 *)(&w_dts);
	
	av_dts[0] = ((pts[0] & 0x06) << 4) | 
		((pts[1] & 0xFC) >> 2); 
	av_dts[1] = ((pts[1] & 0x03) << 6) |
		((pts[2] & 0xFC) >> 2); 
	av_dts[2] = ((pts[2] & 0x02) << 6) |
		((pts[3] & 0xFE) >> 1);
	av_dts[3] = ((pts[3] & 0x01) << 7) |
		((pts[4] & 0xFE) >> 1);
	return w_dts;
}


void init_av_pes (av_pes_packet *p){
	p->length = 0;
	p->reserved = AV_RES;
	p->counter = 0;
	p->flags = 0x00;
}


void nlength_av_pes(av_pes_packet *p){
	short *ll = (short *) p->llength;
	short l = p->length;
	if ((p->flags & PTS_FLAG) && (p->stream_id == AV_VIDEO)) l+= 4;
	*ll = htons(l);
}


void a_pts(u8 *apts, av_pes_packet *p){
	p->pts[0] = ((apts[0] & 0x06) << 4) | 
		((apts[1] & 0xFC) >> 2); 
	p->pts[1] = ((apts[1] & 0x03) << 6) |
		((apts[2] & 0xFC) >> 2); 
	p->pts[2] = ((apts[2] & 0x02) << 6) |
		((apts[3] & 0xFE) >> 1);
	p->pts[3] = ((apts[3] & 0x01) << 7) |
		((apts[4] & 0xFE) >> 1);
}

void setlength_av_pes(av_pes_packet *p){
	short *ll;
	ll = (short *) p->llength;
	p->length = ntohs(*ll);
}


int cwrite_av_pes(u8 *buf, av_pes_packet *p, long length){
	long count;
	u8 sync[2] = { 0x41, 0x56} ;
	
	count = 0;
	if (length < MAX_AV) {
		fprintf(stderr,"Wrong buffer size in cwrite_av_pes\n");
		exit(1);
	}

	if (! p->length) return 0;
	nlength_av_pes(p);
	buf[0] = sync[0];
	buf[1] = sync[1];
	count = 2;
	if (p->gap){
		p->flags |= (p->gap & 0x03) << 2;
	}
	buf[count] = p->stream_id;
	count++;
	buf[count] = p->counter;
	count++;
	buf[count] = p->reserved;
	count++;
	buf[count] = p->flags;
	count++;
	buf[count] = p->llength[0];
	count++;
	buf[count] = p->llength[1];
	count++;

	if ((p->flags & PTS_FLAG) && (p->stream_id == AV_VIDEO)){
		memcpy(buf+count,p->pts,4);
		count += 4;
		memcpy(buf+count,p->payload,p->length);
		count += p->length;
	} else {
		memcpy(buf+count,p->payload,p->length);
		count += p->length;
	}

	return count;

}

void write_av_pes(int fd, av_pes_packet *p){
	long length;
	u8 buf[MAX_AV+8];
	
	length = cwrite_av_pes(buf,p,MAX_AV+8);
	write(fd,buf,length);
}

int read_av_pes(int f, av_pes_packet *p){
	u8 sync[2];
	int found=0;
	int neof = 1;

	while (neof > 0 && !found){
		neof = read(f,(void *)&sync,2);
		if (sync[0] == 0x41 && sync[1] == 0x56) 
			found=1; 
		
	}

	neof = read(f,(void *)&p->stream_id,1);
	if (p->stream_id != AV_VIDEO && p->stream_id != AV_AUDIO)
		return neof;
	neof = read(f,(void *)&p->counter,1);
	neof = read(f,(void *)&p->reserved,1);
	neof = read(f,(void *)&p->flags,1);
	neof = read(f,(void *)p->llength,2);
	setlength_av_pes(p);
	if (p->flags & PTS_FLAG && (p->stream_id == AV_VIDEO)){
		neof = read(f,(void *)p->pts,4);
		p->length -=4;
		neof = read(f,(void *)p->payload,p->length);
	} else
		neof = read(f,(void *)p->payload,p->length);
	if (p->flags & PTS_FLAG && (p->stream_id == AV_AUDIO))
		a_pts(p->payload+7,p);
	return neof;
}

void cread_av_pes(char *buf, av_pes_packet *p,long length){
	u8 sync[2];
	int found=0;
	long count=0;
	
	while ( count < length && !found){
		memcpy(&sync,buf+count,2);
		count+=2;
		if (sync[0] == 0x41 && sync[1] == 0x56) 
			found=1; 		
	}

	memcpy(&p->stream_id,buf+count,1);
	count++;
	if (p->stream_id != AV_VIDEO && p->stream_id != AV_AUDIO)
		return;
	memcpy(&p->counter,buf+count,1);
	count++;
	memcpy(&p->reserved,buf+count,1);
	count++;
	memcpy(&p->flags,buf+count,1);
	count++;
	memcpy(p->llength,buf+count,2);
	count+=2;
	setlength_av_pes(p);
	if (p->flags & PTS_FLAG && (p->stream_id == AV_VIDEO)){
		memcpy(p->pts,buf+count,4);
		count+=4;
		p->length -=4;
		memcpy(p->payload,buf+count,p->length);
		count+=p->length;
	} else {
		memcpy(p->payload,buf+count,p->length);
		count+=p->length;
	}
	if (p->flags & PTS_FLAG && (p->stream_id == AV_AUDIO))
		a_pts(p->payload+7,p);
}



/*

      PES
  
*/


void init_pes(pes_packet *p){
	p->stream_id = 0;
	p->llength[0] = 0;
	p->llength[1] = 0;
	p->length = 0;
	p->flags1 = 0x80;
	p->flags2 = 0;
	p->pes_hlength = 0;
	p->trick = 0;
	p->add_cpy = 0;
	p->priv_flags = 0;
	p->pack_field_length = 0;
	p->pack_header = (u8 *) NULL;
	p->pck_sqnc_cntr = 0;
	p->org_stuff_length = 0;
	p->pes_ext_lngth = 0;
	p->pes_ext = (u8 *) NULL;
	p->pes_pckt_data = (u8 *) NULL;
	p->padding = 0;
	p->mpeg = 2; // DEFAULT MPEG2
	p->mpeg1_pad = 0;
	p->mpeg1_headr = NULL;
	p->stuffing = 0;
}

void kill_pes(pes_packet *p){
	if (p->pack_header)
		free(p->pack_header);
	if (p->pes_ext)
		free(p->pes_ext);
	if (p->pes_pckt_data)
		free(p->pes_pckt_data);
	if (p->mpeg1_headr)
		free(p->mpeg1_headr);
	init_pes(p);
}

void setlength_pes(pes_packet *p){
 	short *ll;
	ll = (short *) p->llength;
	p->length = ntohs(*ll);
}

static void setl_pes(pes_packet *p){
	setlength_pes(p);
	if (p->length)
		p->pes_pckt_data = (u8 *)malloc(p->length);
}

void nlength_pes(pes_packet *p){
	if (p->length <= 0xFFFF){
		short *ll = (short *) p->llength;
		short l = p->length;
		*ll = htons(l);
	} else {
		p->llength[0] =0x00;
		p->llength[1] =0x00;
	}
}

static void nl_pes(pes_packet *p)
{
	nlength_pes(p);
	p->pes_pckt_data = (u8 *) malloc(p->length);
}

void pts2pts(u8 *av_pts, u8 *pts)
{
  
	av_pts[0] = ((pts[0] & 0x06) << 5) | 
		((pts[1] & 0xFC) >> 2); 
	av_pts[1] = ((pts[1] & 0x03) << 6) |
		((pts[2] & 0xFC) >> 2); 
	av_pts[2] = ((pts[2] & 0x02) << 6) |
			((pts[3] & 0xFE) >> 1);
	av_pts[3] = ((pts[3] & 0x01) << 7) |
		((pts[4] & 0xFE) >> 1);
	
}


void av_pts(av_pes_packet *aps, pes_packet *p){
	u8 *av_pts = aps->pts;
	
	if (p->flags2 & PTS_ONLY){
		av_pts[0] = ((p->pts[0] & 0x06) << 4) | 
			((p->pts[1] & 0xFC) >> 2); 
		av_pts[1] = ((p->pts[1] & 0x03) << 6) |
			((p->pts[2] & 0xFC) >> 2); 
		av_pts[2] = ((p->pts[2] & 0x02) << 6) |
			((p->pts[3] & 0xFE) >> 1);
		av_pts[3] = ((p->pts[3] & 0x01) << 7) |
			((p->pts[4] & 0xFE) >> 1);
		aps->flags |= PTS_FLAG;
	} else {
		av_pts[0] = 0;
		av_pts[1] = 0;
		av_pts[2] = 0;
		av_pts[3] = 0;
		
		aps->flags &= ~PTS_FLAG;
	}
}


void get_pts(av_pes_packet *aps,pes_packet *p, int high){
	u8 *av_pts = aps->pts;
	
	if (aps->flags & PTS_FLAG){
		p->pts[0] = 0x31 | (high << 4) | 
			((av_pts[0] & 0xC0) >>4);
		p->pts[1] = ((av_pts[0] & 0x3F) << 2) |
			((av_pts[1] & 0xC0) >> 6);
		p->pts[2] = 0x01 | ((av_pts[1] & 0x3F) << 2) |
			((av_pts[2] & 0x80) >> 6);
		p->pts[3] = ((av_pts[2] & 0x7F) << 1) |
			((av_pts[3] & 0x80) >> 7);
		p->pts[4] = 0x01 | ((av_pts[3] & 0x7F) << 1);
		
		p->flags2 |= PTS_ONLY;
	} else {
		p->pts[0] = 0;
		p->pts[1] = 0;
		p->pts[2] = 0;
		p->pts[3] = 0;
		p->pts[4] = 0;
		p->flags2 &= ~PTS_DTS;
	}
}



int cwrite_pes(u8 *buf, pes_packet *p, long length){
	int count,i;
	u8 dummy;
	int more = 0;
	u8 headr[3] = { 0x00, 0x00 , 0x01};

	if (length <  p->length+p->pes_hlength){
		fprintf(stderr,"Wrong buffer size in cwrite_pes\n");
		exit(1);
	}


	memcpy(buf,headr,3);
	count = 3;
	buf[count] = p->stream_id;
	count++;

	switch ( p->stream_id ) {
				
	case PROG_STREAM_MAP:
	case PRIVATE_STREAM2:
	case PROG_STREAM_DIR:
	case ECM_STREAM     :
	case EMM_STREAM     :
	case PADDING_STREAM :
		buf[count] = p->llength[0];
		count++;
		buf[count] = p->llength[1];
		count++;
		memcpy(buf+count,p->pes_pckt_data,p->length);
		count += p->length;
		break;
	case DSM_CC_STREAM  :
	case ISO13522_STREAM:
	case PRIVATE_STREAM1:
	case AUDIO_STREAM_S ... AUDIO_STREAM_E:
	case VIDEO_STREAM_S ... VIDEO_STREAM_E:
		buf[count] = p->llength[0];
		count++;
		buf[count] = p->llength[1];
		count++;
		more = 1;
		break;
	}	
	

	if ( more ) {
		if ( p->mpeg == 2 ){
			memcpy(buf+count,&p->flags1,1);
			count++;
			memcpy(buf+count,&p->flags2,1);
			count++;
			memcpy(buf+count,&p->pes_hlength,1);
			count++;
			
			if ((p->flags2 & PTS_DTS_FLAGS) == PTS_ONLY){
				memcpy(buf+count,p->pts,5);
				count += 5;
			} else 
				if ((p->flags2 & PTS_DTS_FLAGS) == PTS_DTS){
					memcpy(buf+count,p->pts,5);
					count += 5;
					memcpy(buf+count,p->dts,5);
					count += 5;
				}
			if (p->flags2 & ESCR_FLAG){
				memcpy(buf+count,p->escr,6);
				count += 6;
			}
			if (p->flags2 & ES_RATE_FLAG){
				memcpy(buf+count,p->es_rate,3);
				count += 3;
			}
			if (p->flags2 & DSM_TRICK_FLAG){
				memcpy(buf+count,&p->trick,1);
				count += 3;
			}
			if (p->flags2 & ADD_CPY_FLAG){
				memcpy(buf+count,&p->add_cpy,1);
				count++;
			}
			if (p->flags2 & PES_CRC_FLAG){
				memcpy(buf+count,p->prev_pes_crc,2);
				count += 2;
			}
			if (p->flags2 & PES_EXT_FLAG){
				memcpy(buf+count,&p->priv_flags,1);
				count++;

				if (p->priv_flags & PRIVATE_DATA){
					memcpy(buf+count,p->pes_priv_data,16);
					count += 16;
				}
				if (p->priv_flags & HEADER_FIELD){
					memcpy(buf+count,&p->pack_field_length,
					       1);
					count++;
					memcpy(buf+count,p->pack_header,
						     p->pack_field_length);
					count += p->pack_field_length;

				}
				
				if ( p->priv_flags & PACK_SEQ_CTR){
					memcpy(buf+count,&p->pck_sqnc_cntr,1);
					count++;
					memcpy(buf+count,&p->org_stuff_length,
					       1);
					count++;
				}
				
				if ( p->priv_flags & P_STD_BUFFER){
					memcpy(buf+count,p->p_std,2);
					count += 2;
				}
				if ( p->priv_flags & PES_EXT_FLAG2){
					memcpy(buf+count,&p->pes_ext_lngth,1);
					count++;
					memcpy(buf+count,p->pes_ext,
						     p->pes_ext_lngth);
					count += p->pes_ext_lngth;
				}
			}
			dummy = 0xFF;
			for (i=0;i<p->stuffing;i++) {
				memcpy(buf+count,&dummy,1);
				count++;
			}
		} else {
			if (p->mpeg1_pad){
				memcpy(buf+count,p->mpeg1_headr,p->mpeg1_pad);
				count += p->mpeg1_pad;
			}
			if ((p->flags2 & PTS_DTS_FLAGS) == PTS_ONLY){
				memcpy(buf+count,p->pts,5);
				count += 5;
			}
			else if ((p->flags2 & PTS_DTS_FLAGS) == 
				 PTS_DTS){
				memcpy(buf+count,p->pts,5);
					count += 5;
				memcpy(buf+count,p->dts,5);
					count += 5;
			}
		}			
		memcpy(buf+count,p->pes_pckt_data,p->length);
		count += p->length;
	}

	return count;

}

void write_pes(int fd, pes_packet *p){
	long length;
	u8 *buf;
	int l = p->length+p->pes_hlength;
	
	buf = (u8 *) malloc(l);
	length = cwrite_pes(buf,p,l);
	write(fd,buf,length);
	free(buf);
}

static unsigned int find_length(int f){
	long p = 0;
	long start = 0;
	long q = 0;
	int found = 0;
	u8 sync4[4];
	int neof = 1;

	start = lseek(f,0,SEEK_CUR);
	start -=2;
        lseek(f,start,SEEK_SET);
	while ( neof > 0 && !found ){
		p = lseek(f,0,SEEK_CUR);
		neof = read(f,&sync4,4);
		if (sync4[0] == 0x00 && sync4[1] == 0x00 && sync4[2] == 0x01) {
			switch ( sync4[3] ) {
				
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				found = 1;
				break;
			default:
				q = lseek(f,0,SEEK_CUR);
				break;
			}	
		} 
	}
	q = lseek(f,0,SEEK_CUR);
	lseek(f,start+2,SEEK_SET);
	if (found) return (unsigned int)(q-start)-4-2;
	else return (unsigned int)(q-start-2);
	
}


void cread_pes(char *buf, pes_packet *p){
	
	u8 count, dummy, check;
	int i;
	long po = 0;
	int c=0;

	switch ( p->stream_id ) {
		
	case PROG_STREAM_MAP:
	case PRIVATE_STREAM2:
	case PROG_STREAM_DIR:
	case ECM_STREAM     :
	case EMM_STREAM     :
		memcpy(p->pes_pckt_data,buf+c,p->length);
		return;
		break;
	case PADDING_STREAM :
		p->padding = p->length;
		memcpy(p->pes_pckt_data,buf+c,p->length);
		return;
		break;			
	case DSM_CC_STREAM  :
	case ISO13522_STREAM:
	case PRIVATE_STREAM1:
	case AUDIO_STREAM_S ... AUDIO_STREAM_E:
	case VIDEO_STREAM_S ... VIDEO_STREAM_E:
		break;
	default:
		return;
		break;
	}	
	
	po = c;
	memcpy(&p->flags1,buf+c,1);
	c++;
	if ( (p->flags1 & 0xC0) == 0x80 ) p->mpeg = 2;
	else p->mpeg = 1;
	
	if ( p->mpeg == 2 ){
		memcpy(&p->flags2,buf+c,1);
		c++;
		memcpy(&p->pes_hlength,buf+c,1);
		c++;
		
		p->length -=p->pes_hlength+3;
		count = p->pes_hlength;
		
		if ((p->flags2 & PTS_DTS_FLAGS) == PTS_ONLY){
			memcpy(p->pts,buf+c,5);
			c += 5;
			count -=5;
		} else 
			if ((p->flags2 & PTS_DTS_FLAGS) == PTS_DTS){
				memcpy(p->pts,buf+c,5);
				c += 5;
				memcpy(p->dts,buf+c,5);
				c += 5;
				count -= 10;
			}
		
		if (p->flags2 & ESCR_FLAG){
			memcpy(p->escr,buf+c,6);
			c += 6;
			count -= 6;
		}
		
		if (p->flags2 & ES_RATE_FLAG){
			memcpy(p->es_rate,buf+c,3);
			c += 6;
			count -= 6;
		}

		if (p->flags2 & DSM_TRICK_FLAG){
			memcpy(&p->trick,buf+c,1);
			c += 6;
			count -= 1;
		}
		
		if (p->flags2 & ADD_CPY_FLAG){
			memcpy(&p->add_cpy,buf+c,1);
			c++;
			count -= 1;
		}
		
		if (p->flags2 & PES_CRC_FLAG){
			memcpy(p->prev_pes_crc,buf+c,2);
			c += 2;
			count -= 2;
		}			
		
		if (p->flags2 & PES_EXT_FLAG){
			memcpy(&p->priv_flags,buf+c,1);
			c++;
			count -= 1;
			
			if (p->priv_flags & PRIVATE_DATA){
				memcpy(p->pes_priv_data,buf+c,16);
				c += 16;
				count -= 16;
			}
			
			if (p->priv_flags & HEADER_FIELD){
				memcpy(&p->pack_field_length,buf+c,1);
				c++;
				p->pack_header = (u8 *)
					malloc(p->pack_field_length);
				memcpy(p->pack_header,buf+c,
				       p->pack_field_length);
				c += p->pack_field_length;
				count -= 1+p->pack_field_length;
			}
			
			if ( p->priv_flags & PACK_SEQ_CTR){
				memcpy(&p->pck_sqnc_cntr,buf+c,1);
				c++;
				memcpy(&p->org_stuff_length,buf+c,1);
				c++;
				count -= 2;
			}
			
			if ( p->priv_flags & P_STD_BUFFER){
				memcpy(p->p_std,buf+c,2);
				c += 2;
				count -= 2;
			}

			if ( p->priv_flags & PES_EXT_FLAG2){
				memcpy(&p->pes_ext_lngth,buf+c,1);
				c++;
				p->pes_ext = (u8 *)
					malloc(p->pes_ext_lngth);
				memcpy(p->pes_ext,buf+c,
				       p->pes_ext_lngth);
				c += p->pes_ext_lngth;
				count -= 1+p->pes_ext_lngth;
			}
		}
		p->stuffing = count;
		for(i = 0; i< count ;i++){ 
			memcpy(&dummy,buf+c,1);
			c++;
		}
	} else {
		p->mpeg1_pad = 1;
 		check = p->flags1;
		while (check == 0xFF){
			memcpy(&check,buf+c,1);
			c++;
			p->mpeg1_pad++;
		}
		
		if ( (check & 0xC0) == 0x40){
			memcpy(&check,buf+c,1);
			c++;
			p->mpeg1_pad++;
			memcpy(&check,buf+c,1);
			c++;
			p->mpeg1_pad++;
		}
		p->flags2 = 0;
		p->length -= p->mpeg1_pad;
		
		c = po;
		if ( (check & 0x30)){
			p->length ++;
			p->mpeg1_pad --;
			
			if (check == p->flags1){
				p->pes_hlength = 0;
			} else {
				p->mpeg1_headr = (u8 *)
					malloc(p->mpeg1_pad);
				p->pes_hlength = p->mpeg1_pad;
				memcpy(p->mpeg1_headr,buf+c,
				       p->mpeg1_pad);
				c += p->mpeg1_pad;
			}
			
			p->flags2 = (check & 0xF0) << 2;
			if ((p->flags2 & PTS_DTS_FLAGS) == PTS_ONLY){
				memcpy(p->pts,buf+c,5);
				c += 5;
				p->length -= 5;
				p->pes_hlength += 5;
			}
			else if ((p->flags2 & PTS_DTS_FLAGS) == 
				 PTS_DTS){
				memcpy(p->pts,buf+c,5);
				c += 5;
				memcpy(p->dts,buf+c,5);
				c += 5;
				p->length -= 10;
				p->pes_hlength += 10;
			}
		} else {
			p->mpeg1_headr = (u8 *) malloc(p->mpeg1_pad);
			p->pes_hlength = p->mpeg1_pad;
			memcpy(p->mpeg1_headr,buf+c,
			       p->mpeg1_pad);
			c += p->mpeg1_pad;
		}
	}
	memcpy(p->pes_pckt_data,buf+c,p->length);
}


int read_pes(int f, pes_packet *p){
	
	u8 sync4[4];
	int found=0;
	long po = 0;
	int neof = 1;
	u8 *buf;

	while (neof > 0 && !found) {
	        po = lseek(f,0,SEEK_CUR);
		if (po < 0) return -1;
		if ((neof = read(f,&sync4,4)) < 4) return -1;
		if (sync4[0] == 0x00 && sync4[1] == 0x00 && sync4[2] == 0x01) {
			p->stream_id = sync4[3];
			switch ( sync4[3] ) {
				
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				if((neof = read(f,p->llength,2)) < 2)
					return -1;
				setl_pes(p);
				if (!p->length){ 
					p->length = find_length(f);
					nl_pes(p);
				}
				found = 1;
				break;
				
			default:
				if (lseek(f,po+1,SEEK_SET) < po+1) return -1;
			break;
			}	
		} else if(lseek(f,po+1,SEEK_SET) < po+1) return -1;
	}

	if (!found || !p->length) return 0;
	
	if (p->length >0){
		buf = (u8 *) malloc(p->length);
		if((neof = read(f,buf,p->length))< p->length) return -1;
		cread_pes((char *)buf,p);
		free(buf);
	} else return 0;

	return neof;
}

/*

   Transport Stream

*/

void init_ts(ts_packet *p){
	p->pid[0] = 0;
	p->pid[1] = 0;
	p->flags = 0;
	p->count = 0;
	p->adapt_length = 0;
	p->adapt_flags = 0;
	p->splice_count = 0;
	p->priv_dat_len = 0;
	p->priv_dat = NULL;
	p->adapt_ext_len = 0;
	p->adapt_eflags = 0;
	p->rest = 0;
	p->stuffing = 0;
}

void kill_ts(ts_packet *p){
	if (p->priv_dat)
		free(p->priv_dat);
	init_ts(p);
}


unsigned short get_pid(u8 *pid)
{
	unsigned short *pp;
	u8 ppid[2];
	ppid[0] = pid[0] & PID_MASK_HI;
	ppid[1] = pid[1];
	pp = (unsigned short *) ppid;
	return ntohs(*pp);
}

unsigned short pid_ts(ts_packet *p)
{
  return get_pid(p->pid);
}

int cwrite_ts(u8 *buf, ts_packet *p, long length){
	long count,i;
	u8 sync,dummy;

	sync = 0x47;
	memcpy(buf,&sync,1);
	count = 1;
	memcpy(buf+count,p->pid,2);
	count += 2;
	memcpy(buf+count,&p->flags,1);
	count++;

	 
	if (! (p->flags & ADAPT_FIELD) && (p->flags & PAYLOAD)){
		memcpy(buf+count,p->data,184);
		count += 184;
	} else {
		memcpy(buf+count,&p->adapt_length,1);
		count++;
		memcpy(buf+count,&p->adapt_flags,1);
		count++;

		if ( p->adapt_flags & PCR_FLAG ){
			memcpy(buf+count, p->pcr,6);
			count += 6;
		}
		if ( p->adapt_flags & OPCR_FLAG ){
			memcpy(buf+count, p->opcr,6);
			count += 6;
		}
		if ( p->adapt_flags & SPLICE_FLAG ){
			memcpy(buf+count, &p->splice_count,1);
			count++;
		}
		if( p->adapt_flags & TRANS_PRIV){
			memcpy(buf+count,&p->priv_dat_len,1);
			count++;
			memcpy(buf+count,p->priv_dat,p->priv_dat_len);
			count += p->priv_dat_len;
		}
			
		if( p->adapt_flags & ADAP_EXT_FLAG){
			memcpy(buf+count,&p->adapt_ext_len,1);
			count++;
			memcpy(buf+count,&p->adapt_eflags,1);
			count++;
		
			if( p->adapt_eflags & LTW_FLAG){
				memcpy(buf+count,p->ltw,2);
				count += 2;
			}
			if( p->adapt_eflags & PIECE_RATE){
				memcpy(buf+count,p->piece_rate,3);
				count += 3;
			}
			if( p->adapt_eflags & SEAM_SPLICE){
				memcpy(buf+count,p->dts,5);
				count += 5;
			}
		}
		dummy = 0xFF;
		for(i=0; i < p->stuffing ; i++){
			memcpy(buf+count,&dummy,1);
			count++;
		}
		if (p->flags & PAYLOAD){
			memcpy(buf+count,p->data,p->rest);
			count += p->rest;
		}
	} 


	return count;
}

void write_ts(int fd, ts_packet *p){
	long length;
	u8 buf[TS_SIZE];

	length = cwrite_ts(buf,p,TS_SIZE);
	write(fd,buf,length);
}

int read_ts (int f, ts_packet *p){
	u8 sync;
	int found=0;
	long po,q;
	int neof = 1;

	sync=0;
	while (neof > 0 && !found) {
		neof = read(f,&sync,1);
		if (sync == 0x47) 
			found = 1;
	}
	neof = read(f,p->pid,2);
	neof = read(f,&p->flags,1);
	p->count = p->flags & COUNT_MASK;
	 
	if (!(p->flags & ADAPT_FIELD) && (p->flags & PAYLOAD)){
		//no adapt. field only payload
		neof = read(f,p->data,184);
		p->rest = 184;
		return neof;
	} 

	if ( p->flags & ADAPT_FIELD ) {
		// adaption field
		neof = read(f,&p->adapt_length,1);
		po = lseek(f,0,SEEK_CUR);
		neof = read(f,&p->adapt_flags,1);

		if ( p->adapt_flags & PCR_FLAG )
			neof = read(f, p->pcr,6);

		if ( p->adapt_flags & OPCR_FLAG )
			neof = read(f, p->opcr,6);

		if ( p->adapt_flags & SPLICE_FLAG )
			neof = read(f, &p->splice_count,1);

		if( p->adapt_flags & TRANS_PRIV){
			neof = read(f,&p->priv_dat_len,1);
			p->priv_dat = (u8 *) malloc(p->priv_dat_len);
			neof = read(f,p->priv_dat,p->priv_dat_len);
		}
			
		if( p->adapt_flags & ADAP_EXT_FLAG){
			neof = read(f,&p->adapt_ext_len,1);
			neof = read(f,&p->adapt_eflags,1);
			if( p->adapt_eflags & LTW_FLAG)
				neof = read(f,p->ltw,2);
			
			if( p->adapt_eflags & PIECE_RATE)
				neof = read(f,p->piece_rate,3);
			
			if( p->adapt_eflags & SEAM_SPLICE)
				neof = read(f,p->dts,5);
		}
		q = lseek(f,0,SEEK_CUR);
		p->stuffing = p->adapt_length -(q-po);
		p->rest = 183-p->adapt_length;
		lseek(f,q+p->stuffing,SEEK_SET);
		if (p->flags & PAYLOAD) // payload
			neof = read(f,p->data,p->rest);
		else 
			lseek(f,q+p->rest,SEEK_SET);
	}
	return neof;
}

void cread_ts (char *buf, ts_packet *p, long length){
	u8 sync;
	int found=0;
	long po,q;
	long count;
	
	sync=0;
	while (count < length  && !found) {
		sync=buf[count];
		count++;
		if (sync == 0x47) 
			found = 1;
	}
	memcpy(p->pid,buf+count,2);
	count += 2;
	p->flags = buf[count];
	count++;
	p->count = p->flags & COUNT_MASK;
	 
	if (!(p->flags & ADAPT_FIELD) && (p->flags & PAYLOAD)){
		//no adapt. field only payload
		memcpy(p->data,buf+count,184);
		p->rest = 184;
		return;
	} 

	if ( p->flags & ADAPT_FIELD ) {
		// adaption field
		p->adapt_length = buf[count];
		count++;
		po = count;
		memcpy(&p->adapt_flags,buf+count,1);
		count++;

		if ( p->adapt_flags & PCR_FLAG ){
			memcpy( p->pcr,buf+count,6);
			count += 6;
		}
		if ( p->adapt_flags & OPCR_FLAG ){
			memcpy( p->opcr,buf+count,6);
			count += 6;
		}
		if ( p->adapt_flags & SPLICE_FLAG ){
			memcpy( &p->splice_count,buf+count,1);
			count++;
		}
		if( p->adapt_flags & TRANS_PRIV){
			memcpy(&p->priv_dat_len,buf+count,1);
			count++;
			p->priv_dat = (u8 *) malloc(p->priv_dat_len);
			memcpy(p->priv_dat,buf+count,p->priv_dat_len);
			count += p->priv_dat_len;
		}
			
		if( p->adapt_flags & ADAP_EXT_FLAG){
			memcpy(&p->adapt_ext_len,buf+count,1);
			count++;
			memcpy(&p->adapt_eflags,buf+count,1);
			count++;
			if( p->adapt_eflags & LTW_FLAG){
				memcpy(p->ltw,buf+count,2);
				count += 2;
			}
			if( p->adapt_eflags & PIECE_RATE){
				memcpy(p->piece_rate,buf+count,3);
				count += 3;
			}
			if( p->adapt_eflags & SEAM_SPLICE){
				memcpy(p->dts,buf+count,5);
				count += 5;
			}
		}
		q = count;
		p->stuffing = p->adapt_length -(q-po);
		p->rest = 183-p->adapt_length;
		count = q+p->stuffing;
		if (p->flags & PAYLOAD){ // payload
			memcpy(p->data,buf+count,p->rest);
			count += p->rest;
		} else 
			count = q+p->rest;
	}
}


/*

   Program Stream

*/


void init_ps(ps_packet *p)
{
	p->stuff_length=0xF8;
	p->data = NULL;
	p->sheader_length = 0;
	p->audio_bound = 0;
	p->video_bound = 0;
	p->npes = 0;
	p->mpeg = 2;
}

void kill_ps(ps_packet *p)
{
	if (p->data)
		free(p->data);
	init_ps(p);
}

void setlength_ps(ps_packet *p)
{
	short *ll;
	ll = (short *) p->sheader_llength;
	if (p->mpeg == 2)
		p->sheader_length = ntohs(*ll) - 6;
	else 
		p->sheader_length = ntohs(*ll);
}	

static void setl_ps(ps_packet *p)
{
	setlength_ps(p);
	p->data = (u8 *) malloc(p->sheader_length);
}

int mux_ps(ps_packet *p)
{
	u32 mux = 0;
	u8 *i = (u8 *)&mux;

	i[1] = p->mux_rate[0];
	i[2] = p->mux_rate[1];
	i[3] = p->mux_rate[2];
	mux = ntohl(mux);
	mux = (mux >>2);
	return mux;
}

int rate_ps(ps_packet *p)
{
	u32 rate=0;
	u8 *i= (u8 *) &rate;

	i[1] = p->rate_bound[0] & 0x7F;
	i[2] = p->rate_bound[1];
	i[3] = p->rate_bound[2];
	
	rate = ntohl(rate);
	rate = (rate >> 1);
	return rate;
}


u32 scr_base_ps(ps_packet *p) // only 32 bit!!
{
	u32 base = 0;
	u8 *buf = (u8 *)&base;
	
	buf[0] |= (long int)((p->scr[0] & 0x18) << 3);
	buf[0] |= (long int)((p->scr[0] & 0x03) << 4);
	buf[0] |= (long int)((p->scr[1] & 0xF0) >> 4);
		 
	buf[1] |= (long int)((p->scr[1] & 0x0F) << 4);
	buf[1] |= (long int)((p->scr[2] & 0xF0) >> 4);

	buf[2] |= (long int)((p->scr[2] & 0x08) << 4);
	buf[2] |= (long int)((p->scr[2] & 0x03) << 5);
	buf[2] |= (long int)((p->scr[3] & 0xF8) >> 3);

	buf[3] |= (long int)((p->scr[3] & 0x07) << 5);
	buf[3] |= (long int)((p->scr[4] & 0xF8) >> 3);

	base = ntohl(base);
	return base;
}

u16 scr_ext_ps(ps_packet *p)
{
	short ext = 0;

	ext = (short)(p->scr[5] >> 1);
	ext += (short) (p->scr[4] &  0x03) * 128;

	return ext;
}

int cwrite_ps(u8 *buf, ps_packet *p, long length)
{
	long count,i;
	u8 headr1[4] = {0x00, 0x00, 0x01, 0xBA };
	u8 headr2[4] = {0x00, 0x00, 0x01, 0xBB };
	u8 buffy = 0xFF;

	
	memcpy(buf,headr1,4);
	count = 4;
	if (p->mpeg == 2){
		memcpy(buf+count,p->scr,6);
		count += 6;
		memcpy(buf+count,p->mux_rate,3);
		count += 3;
		memcpy(buf+count,&p->stuff_length,1);
		count++;
		for(i=0; i< (p->stuff_length & 3); i++){
			memcpy(buf+count,&buffy,1);
			count++;
		}
	} else {
		memcpy(buf+count,p->scr,5);
		count += 5;
		memcpy(buf+count,p->mux_rate,3);
		count += 3;
	}
	if (p->sheader_length){
		memcpy(buf+count,headr2,4);
		count += 4;
		memcpy(buf+count,p->sheader_llength,2);
		count += 2;
		if ( p->mpeg == 2){
			memcpy(buf+count,p->rate_bound,3);
			count += 3;
			memcpy(buf+count,&p->audio_bound,1);
			count++;
			memcpy(buf+count,&p->video_bound,1);
			count++;
			memcpy(buf+count,&p->reserved,1);
			count++;
		}
		memcpy(buf+count,p->data,p->sheader_length);
		count += p->sheader_length;
	}

	return count;
}

void write_ps(int fd, ps_packet *p){
	long length;
	u8 buf[PS_MAX];

	length = cwrite_ps(buf,p,PS_MAX);
	write(fd,buf,length);
}

int read_ps (int f, ps_packet *p){
	u8 headr[4];
	pes_packet pes;
	int i,done;
	int found=0;
	long po = 0;
	long q = 0;
	long count = 0;
	int neof = 1;

	po = lseek(f,0,SEEK_CUR);
	while (neof > 0 && !found && count < MAX_SEARCH) {
		neof = read(f,&headr,4);
		if (headr[0] == 0x00 && headr[1] == 0x00 && headr[2] == 0x01){
			if ( headr[3] == 0xBA ) 
				found = 1;
			else 
				if ( headr[3] == 0xB9 ) break;
				else lseek(f,po+1,SEEK_SET);
		}
		count++;
	}
	
	if (found){
		neof = read(f,p->scr,6);
		if (p->scr[0] & 0x40)
			p->mpeg = 2;
		else
			p->mpeg = 1;

		if (p->mpeg == 2){
			neof = read(f,p->mux_rate,3);
			neof = read(f,&p->stuff_length,1);
			po = lseek(f,0,SEEK_CUR);
			lseek(f,po+(p->stuff_length & 3),SEEK_SET);
		} else {
			p->mux_rate[0] = p->scr[5]; //mpeg1 scr is only 5 bytes
			neof = read(f,p->mux_rate+1,2);
		}
			
		po = lseek(f,0,SEEK_CUR);
		neof = read(f,headr,4);
		if (headr[0] == 0x00 && headr[1] == 0x00 && 
		    headr[2] == 0x01 && headr[3] == 0xBB ) {
			neof = read(f,p->sheader_llength,2);
			setl_ps(p);
			if (p->mpeg == 2){
				neof = read(f,p->rate_bound,3);
				neof = read(f,&p->audio_bound,1);
				neof = read(f,&p->video_bound,1);
				neof = read(f,&p->reserved,1);
			}
			neof = read(f,p->data,p->sheader_length);
		} else {
			lseek(f,po,SEEK_SET);
			p->sheader_length = 0;
		}

		i = 0;
		done = 0;
		q = lseek(f,0,SEEK_CUR);
		do {
			po = lseek(f,0,SEEK_CUR);
			neof = read(f,headr,4);
			lseek(f,po,SEEK_SET);
			if ( headr[0] == 0x00 && headr[1] == 0x00 
			     && headr[2] == 0x01 && headr[3] != 0xBA){
				init_pes(&pes);
				neof = read_pes(f,&pes);
			        i++;
			} else done = 1;
			kill_pes(&pes);
		} while ( neof > 0 && !done);
		p->npes = i;
		lseek(f,q,SEEK_SET);
	} 
	return neof;
}

void cread_ps (char *buf, ps_packet *p, long length){
	u8 *headr;
	pes_packet pes;
	int i,done;
	int found=0;
	long po = 0;
	long q = 0;
	long count = 0;
	long c = 0;
	
	po = c;
	while ( count < length && !found && count < MAX_SEARCH) {
		headr = (u8 *)buf+c;
		c += 4;
		if (headr[0] == 0x00 && headr[1] == 0x00 && headr[2] == 0x01){
			if ( headr[3] == 0xBA ) 
				found = 1;
			else 
				if ( headr[3] == 0xB9 ) break;
				else c = po+1;
		}
		count++;
	}
	
	if (found){
		memcpy(p->scr,buf+c,6);
		c += 6;
		if (p->scr[0] & 0x40)
			p->mpeg = 2;
		else
			p->mpeg = 1;

		if (p->mpeg == 2){
			memcpy(p->mux_rate,buf+c,3);
			c += 3;
			memcpy(&p->stuff_length,buf+c,1);
			c++;
			po = c;
			c = po+(p->stuff_length & 3);
		} else {
			p->mux_rate[0] = p->scr[5]; //mpeg1 scr is only 5 bytes
			memcpy(p->mux_rate+1,buf+c,2);
			c += 2;
		}
			
		po = c;
		headr = (u8 *)buf+c;
		c += 4;
		if (headr[0] == 0x00 && headr[1] == 0x00 && 
		    headr[2] == 0x01 && headr[3] == 0xBB ) {
			memcpy(p->sheader_llength,buf+c,2);
			c += 2;
			setl_ps(p);
			if (p->mpeg == 2){
				memcpy(p->rate_bound,buf+c,3);
				c += 3;
				memcpy(&p->audio_bound,buf+c,1);
				c++;
				memcpy(&p->video_bound,buf+c,1);
				c++;
				memcpy(&p->reserved,buf+c,1);
				c++;
			}
			memcpy(p->data,buf+c,p->sheader_length);
			c += p->sheader_length;
		} else {
			c = po;
			p->sheader_length = 0;
		}

		i = 0;
		done = 0;
		q = c;
		do {
			headr = (u8 *)buf+c;
			if ( headr[0] == 0x00 && headr[1] == 0x00 
			     && headr[2] == 0x01 && headr[3] != 0xBA){
				init_pes(&pes);
				//	cread_pes(buf+c,&pes);
			        i++;
			} else done = 1;
			kill_pes(&pes);
		} while (c < length && !done);
		p->npes = i;
		c = q;
	} 
}







/*
  conversion
*/

static u8 write_videoc(av_pes_packet *apsv, u8 countv, int fd){
	int gap = apsv->length & 3;
	apsv->gap = gap;
	apsv->counter = countv;
	apsv->length -= gap;
	write_av_pes(fd, apsv);
	if (apsv->length) countv += 1;
	if (gap) memmove(apsv->payload, apsv->payload+apsv->length,gap);
	apsv->flags = gap;
	apsv->length = gap;
	return countv;
}

static int write_vpesc(int lengthv, u8 *posv, av_pes_packet *apsv, int fd,
		      int p){
	static u8 countv = 0;
	static int markv = 0;

	lengthv -= markv;
	posv += markv;


	if(p == 4){
		if (apsv->length > (apsv->flags & 0x03)){
			apsv->flags &= ~PTS_FLAG;
			countv = write_videoc(apsv,countv,fd);
			apsv->flags |= PTS_FLAG;
		}
	}

	if (apsv->flags & PTS_FLAG) p = 4;
	if (lengthv+apsv->length < MAX_AV-p ){
		memcpy(apsv->payload+apsv->length,posv,lengthv);
		apsv->length += lengthv;
		markv = 0;
		return 1;
	} else {
		int l= MAX_AV-apsv->length-p;
		memcpy(apsv->payload+apsv->length,posv,l);
		markv += l;
		apsv->length += l;		
		countv = write_videoc(apsv,countv,fd);
		return 0;
	}
}

static u8 write_audioc(av_pes_packet *apsa, u8 counta, int fd){
	apsa->counter = counta;
	write_av_pes(fd, apsa);
	if (apsa->length) counta++;
	apsa->flags = 0;
	apsa->length = 0;
	return counta;
}

static int write_apesc(u8 *apes,av_pes_packet *apsa, 
		      int length,int fd) {
	static int marka = 0;
	static u8 counta = 0;
	int lengtha = length-marka;
	u8 *posa = apes+marka;
	
	if (lengtha+apsa->length <= MAX_AVA){
		memcpy(apsa->payload+apsa->length,posa,lengtha);
		apsa->length += lengtha;
		counta = write_audioc(apsa, counta,fd);
		marka = 0;
		return 1;
	} else {
		int l = MAX_AVA-apsa->length; 
		memcpy(apsa->payload+apsa->length,posa,l);
		marka += l;
		apsa->length += l; 
		counta = write_audioc(apsa, counta,fd);
		return 0;
	}
}

static int  pes_to_AV_PESc(av_pes_packet *apsv,av_pes_packet *apsa,
			 int in,int out){
	pes_packet pes;
	int p = 0;
	int length,done;
	u8 *apes;
	int neof;

	init_pes(&pes);
	neof = read_pes(in,&pes);
	done = 0;
	switch (pes.stream_id){
	case VIDEO_STREAM_S ... VIDEO_STREAM_E:
		if (pes.flags2 & PTS_DTS){
			av_pts(apsv,&pes);
			p = 4;
		} else 	p = 0;

		
		while (!done){
			done = write_vpesc(pes.length,pes.pes_pckt_data
					   ,apsv,out,p);
			p = 0;
		}
		break;

	case AUDIO_STREAM_S ... AUDIO_STREAM_E:
		
		length = pes.length+pes.pes_hlength+9;
		if (length){
			apes = (u8 *) malloc(length);
			cwrite_pes(apes,&pes,length);
			if ((pes.flags2 & PTS_DTS) && pes.mpeg == 2)
				apsa->flags |= PTS_FLAG;

			while (!done){
				done = write_apesc(apes,apsa
						  ,length,out);
			}
			free(apes);
		}
		break;
	
	default:
		done = 1;
	}
	kill_pes(&pes);
	return neof;
}


static void kwrite_video(av_pes_packet *apsv){
	int gap = apsv->length & 3;

	apsv->gap = gap;
	apsv->length -= gap;
	write_av_pes(STDOUT_FILENO, apsv);
	if (gap) memmove(apsv->payload, apsv->payload+apsv->length,gap);
	apsv->flags = gap;
	apsv->length = gap;
	apsv->counter++;
}

static void kwrite_audio(av_pes_packet *apsa){
	write_av_pes(STDOUT_FILENO, apsa);
	apsa->flags = 0;
	apsa->length = 0;
	apsa->counter ++;
}

void init_p2a(p2a *p){
	p->found = 0;
	p->cid = 0;
	p->mpeg = 0;
	init_av_pes(&p->apsv);
	init_av_pes(&p->apsa);
	p->apsv.stream_id = AV_VIDEO;
	p->apsa.stream_id = AV_AUDIO;
	p->done = 0;
	p->fd1 = -1;
	p->av = 0;
}


void kpes_to_av_pes(u8 *buf, int count, p2a *p)
{

	int l,max;
	unsigned short *pl;
	int c=0;

	u8 headr[3] = { 0x00, 0x00, 0x01} ;

	while (c < count && p->found <4 ){
		switch ( p->found ){
		case 0:
		case 1:
			if (buf[c] == 0x00) p->found++;
			else p->found = 0;
			c++;
			break;
		case 2:
			if (buf[c] == 0x01) p->found++;
			else if (buf[c] == 0){
				p->found = 2;
			} else p->found = 0;
			c++;
			break;
		case 3:
			p->cid = 0;
			switch (buf[c]){
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
				p->done = 1;
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
				p->found++;
				p->cid = buf[c];
				c++;
				break;
			default:
				p->found = 0;
				break;
			}
			break;
			
		default:

			break;
		}
	}
	if (c == count) return;

	if (p->found == 4){
			
		if (count-c > 1){
			pl = (unsigned short *) (buf+c);
			p->plength = ntohs(*pl);
			p->plen[0] = buf[c];
			c++;
			p->plen[1] = buf[c];
			c++;
			p->found+=2;
		} else {
			p->plen[0] = buf[c];
			p->found++;
			return;
		}
	}

	if (p->found == 5){
		p->plen[1] = buf[c];
		c++;
		pl = (unsigned short *) p->plen;
		p->plength = ntohs(*pl);
		p->found++;
	}

 
	if (c == count) return;

	while (!p->done && (c < count && p->found > 5 && (p->mpeg == 0 ||
					  (p->mpeg == 1 && p->found < 7) ||
					  (p->mpeg == 2 && p->found < 9)))){
		if ( p->found == 6 ){
			p->flag1 = buf[c];
			c++;
			p->found++;
			if ( (p->flag1 & 0xC0) == 0x80 ) p->mpeg = 2;
			else {
				p->which = 0;
				p->mpeg = 1;
				p->flag2 = 0;				
			}
		}
		if (c == count) return;
	       
		if ( p->found == 7 && p->mpeg == 2){
			p->flag2 = buf[c];
			c++;
			p->found++;
		}
		if (c == count) return;
		if ( p->found == 8  && p->mpeg == 2){
			p->hlength = buf[c];
			c++;
			p->found++;
		}
	}
	if ( p->done || (p->mpeg == 2 && p->found >= 9)  || 
	     (p->mpeg == 1 && p->found >= 7) ){
		switch (p->cid){
			
		case VIDEO_STREAM_S ... VIDEO_STREAM_E:
			if (p->mpeg == 2 && (p->flag2 & PTS_ONLY) &&  
			    p->found < 14){
				while (c < count && p->found < 14){
					p->pts[p->found-9] = buf[c];
					c++;
					p->found++;
				}
				if (c == count) return;
				if (p->apsv.length > (p->apsv.flags & 0x03))
					kwrite_video(&p->apsv);
				pts2pts(p->apsv.pts,p->pts);
				p->apsv.flags |= PTS_FLAG;
			}
			while (p->mpeg == 2 && c < count && 
			       p->found < 9+p->hlength){
				c++;
				p->found++;
			}


			if (p->mpeg == 1 && p->which < 2000){

				if (p->found == 7) p->check = p->flag1;
				while (!p->which && c < count && 
				       p->check == 0xFF){
					p->check = buf[c];
					c++;
					p->found++;
				}

				if ( c == count) return;
				
				if ( (p->check & 0xC0) == 0x40 && !p->which){
					p->check = buf[c];
					c++;
					p->found++;
					p->which = 1;
					if ( c == count) return;
					p->check = buf[c];
					c++;
					p->found++;
					p->which = 2;
					if ( c == count) return;
				}

				if (p->which == 1){
					p->check = buf[c];
					c++;
					p->found++;
					p->which = 2;
					if ( c == count) return;
				}
				
				if ( (p->check & 0x30) && p->check != 0xFF){
					p->flag2 = (p->check & 0xF0) << 2;
					p->pts[0] = p->check;
					p->which = 3;
				} 

				if ( c == count) return;
				if (p->which > 2){
					if ((p->flag2 & PTS_DTS_FLAGS)
					    == PTS_ONLY){
						while (c < count && 
						       p->which < 7){
							p->pts[p->which-2] =
								buf[c];
							c++;
							p->found++;
							p->which++;
						}
						if ( c == count) return;
					} else if ((p->flag2 & PTS_DTS_FLAGS) 
						   == PTS_DTS){
						while (c < count && 
						       p->which< 12){
							if (p->which< 7)
								p->pts[p->which
								      -2] =
									buf[c];
							c++;
							p->found++;
							p->which++;
						}
						if ( c == count) return;
					}
					if (p->which >= 7){
						if (p->apsv.length > 
						    (p->apsv.flags & 0x03))
							kwrite_video(&p->apsv);
						pts2pts(p->apsv.pts,p->pts);
						p->apsv.flags |= PTS_FLAG;
					}
					p->which = 2000;

				}
							
			}
			while (c < count && (p->found < p->plength+6
					     || !p->plength)){
				max = MAX_AV - ((p->apsv.flags & PTS_FLAG)>>2) 
				  - p->apsv.length;
				l = count-c;
				if (p->plength && l+p->found > p->plength+6)
					l = p->plength+6-p->found;
				if (l < max){
					memcpy(p->apsv.payload+p->apsv.length,
					       buf+c,l);
					p->apsv.length += l;
					p->found += l;
					c += l;
				} else {
					memcpy(p->apsv.payload+p->apsv.length,
					       buf+c,max);
					p->apsv.length += max;
					p->found += max;
					c += max;
					kwrite_video(&p->apsv);
				}
			}					
				
			break;
		case AUDIO_STREAM_S ... AUDIO_STREAM_E:			
			if (p->mpeg == 2 && p->found == 9){
				memcpy(p->apsa.payload,headr,3);
				p->apsa.payload[3] = p->cid;
				memcpy(p->apsa.payload+4,p->plen,2);
				p->apsa.payload[6] = p->flag1;
				p->apsa.payload[7] = p->flag2;
				p->apsa.payload[8] = p->hlength;
				if (p->flag2 & PTS_ONLY && p->mpeg == 2)
					p->apsa.flags |= PTS_FLAG;
				p->apsa.length = 9;
			}

			if (p->mpeg == 1 && p->found == 7){
				memcpy(p->apsa.payload,headr,3);
				p->apsa.payload[3] = p->cid;
				memcpy(p->apsa.payload+4,p->plen,2);
				p->apsa.payload[6] = p->flag1;
				p->apsa.length = 7;
				if (c == count) return;
			}

			while (c < count && p->found < p->plength+6){
				max = MAX_AVA - p->apsa.length;
				l = count -c;
				if (l+p->found > p->plength+6)
					l = p->plength+6-p->found;
				if (l < max){
					memcpy(p->apsa.payload+p->apsa.length,
					       buf+c,l);
					p->apsa.length += l;
					p->found += l;
					c += l;
					if (p->found == p->plength+6)
						kwrite_audio(&p->apsa);
				} else {
					memcpy(p->apsa.payload+p->apsa.length,
					       buf+c,max);
					p->apsa.length += max;
					p->found += max;
					c += max;
					kwrite_audio(&p->apsa);
				}	
			}				
			break;
		case PROG_STREAM_MAP:
		case PRIVATE_STREAM2:
		case PROG_STREAM_DIR:
		case ECM_STREAM     :
		case EMM_STREAM     :
		case PADDING_STREAM :
		case DSM_CC_STREAM  :
		case ISO13522_STREAM:
		case PRIVATE_STREAM1:
			if ( p->found + count - c < p->plength+6){
				p->found += count-c;
				c = count;
			} else {
				c += p->plength+6 - p->found;
				p->found = p->plength+6;
			}
			break;
		}

		if (p->plength && p->found == p->plength+6) {
			p->found = 0;
			p->done = 0;
			p->plength = 0;
			if (c < count)
				kpes_to_av_pes(buf+c, count-c, p);
		}
	}
	return;
}

void pes_to_av_pes(int in, int out){
	av_pes_packet apsv;
	av_pes_packet apsa;
	int neof = 1;

	init_av_pes(&apsv);
	init_av_pes(&apsa);
	apsv.stream_id = AV_VIDEO;
	apsa.stream_id = AV_AUDIO;
	while(neof > 0)
		neof = pes_to_AV_PESc(&apsv,&apsa,in,out);
	if (apsv.length){
		write_av_pes(out, &apsv);
	}
	if (apsa.length){
		write_av_pes(out, &apsa);
	}
}

void ps_to_av_pes(int in, int out){
	av_pes_packet apsv;
	av_pes_packet apsa;
	ps_packet ps;
	int j;
	int neof = 1;


	init_av_pes(&apsv);
	init_av_pes(&apsa);
	apsv.stream_id = AV_VIDEO;
	apsa.stream_id = AV_AUDIO;
	while(neof > 0){
		init_ps(&ps);
		neof = read_ps(in,&ps);
		for(j=0;j < ps.npes;j++){
			neof = pes_to_AV_PESc(&apsv,&apsa,in,out);
		}
		kill_ps(&ps);
	}
	if (apsv.length){
		write_av_pes(out, &apsv);
	}
	if (apsa.length){
		write_av_pes(out, &apsa);
	}
}


static void save_pesstuff(p2a *pesto,t2a *tsto,int pid)
{
	tsto->p[pid].found = pesto->found;
	tsto->p[pid].cid = pesto->cid;
	tsto->p[pid].plength = pesto->plength;
	tsto->p[pid].plen[0] = pesto->plen[0];
	tsto->p[pid].plen[1] = pesto->plen[1];
	tsto->p[pid].flag1 = pesto->flag1;
	tsto->p[pid].flag2 = pesto->flag2;
	tsto->p[pid].hlength = pesto->hlength;
	memcpy(tsto->p[pid].pts,pesto->pts,5);
	tsto->p[pid].check = pesto->check;
	tsto->p[pid].which = pesto->which;
	tsto->p[pid].done =  pesto->done;
	tsto->p[pid].fd1 =  pesto->fd1;
	tsto->p[pid].av =  pesto->av;
}

static void recover_pesstuff(p2a *pesto,t2a *tsto,int pid)
{
	pesto->found = tsto->p[pid].found;
	pesto->cid = tsto->p[pid].cid;
	pesto->plength = tsto->p[pid].plength;
	pesto->plen[0] = tsto->p[pid].plen[0];
	pesto->plen[1] = tsto->p[pid].plen[1];
	pesto->flag1 = tsto->p[pid].flag1;
	pesto->flag2 = tsto->p[pid].flag2;
	pesto->hlength = tsto->p[pid].hlength;
	memcpy(pesto->pts,tsto->p[pid].pts,5);
	pesto->check = tsto->p[pid].check;
	pesto->which = tsto->p[pid].which;
	pesto->done =  tsto->p[pid].done;
	pesto->fd1 =  tsto->p[pid].fd1;
	pesto->av =  tsto->p[pid].av;
}

static void init_pes_stuff(pes_stuff *p)
{
	p->found = 0;
	p->cid = 0;
	p->plength = 0;
	p->plen[0] = 0;
	p->plen[1] = 0;
	p->flag1 = 0;
	p->flag2 = 0;
	p->hlength = 0;
	p->pts[0] = 0;
	p->pts[1] = 0;
	p->pts[2] = 0;
	p->pts[3] = 0;
	p->pts[4] = 0;
	p->check = 0;
	p->which = 0;
	p->done =  0;
}

void init_t2a(t2a *p, int pid1, int pid2)
{
	p->pid[0] = pid1;
	p->pid[1] = pid2;
	p->start[0] = 0;
	p->start[1] = 0;
	p->found = 0;
	init_pes_stuff(&p->p[0]);
	init_pes_stuff(&p->p[1]);
	p->flags = 0;
	p->adapt_length = 0;
}



void ts_to (u8 *buf, int count, t2a *p, p2a *pesto,void (*func)(u8 *buf, int count, p2a *p))
{
	int c=0;
	unsigned short *pl;
	int l;
	int pid;
	
	while (c < count && p->found <1 ){
		if ( buf[c] == 0x47) p->found = 1;
		c++;
	}

	if (c == count) return;
	if (p->found == 1){
			
		if (count-c > 1){
			p->cpid[0] = buf[c];
			c++;
			p->cpid[1] = buf[c];
			c++;
			p->found+=2;
			p->tpid = get_pid(p->cpid);
		} else {
			p->cpid[0] = buf[c];
			c++;
			p->found++;
			return;
		}

	}
	if (c == count) return;

	if (p->found == 2){
		p->cpid[1] = buf[c];
		c++;
		pl = (unsigned short *) p->cpid;
		p->tpid = get_pid(p->cpid);
		p->found++;
	}

	
	if (c == count) return;
	
	if (p->found == 3){
		p->flags = buf[c];
		c++;
		p->found++;
		
		pid = -1;
		if (p->tpid == p->pid[0]){
			pid = 0;
		} 
		if (p->tpid == p->pid[1]){
			pid = 1;
		}

		if ( pid >= 0 && (p->cpid[0] & PAY_START)){
			p->start[pid] = 1;
			if ( !p->p[pid].plength){
				p->p[pid].found = 0;
				p->p[pid].done = 0;
			}
		}

	}

	if (c == count) return;

	if ( p->found > 3 && (p->flags & ADAPT_FIELD) ) {
		// adaption field
		if (p->found == 4){
			p->adapt_length = buf[c];
			c++;
			p->found++;
			if (c == count) return;
		} else {
			while (c < count && p->found < p->adapt_length+5){
				c++;
				p->found++;
			}
		}
	}

	if (c == count) return;

	if (p->found > 4 + p->adapt_length - (int)(!(p->flags & ADAPT_FIELD)) 
			     && (p->flags & PAYLOAD)){
		l = count - c;
		if (l+p->found > 188) l = 188-p->found;

		pid = -1;
		if (p->tpid == p->pid[0]){
			pid = 0;
		} 
		if (p->tpid == p->pid[1]){
			pid = 1;
		}

		if (pid >= 0 && p->start[pid]){
			recover_pesstuff(pesto,p,pid);
			func(buf+c, l, pesto);
			save_pesstuff(pesto,p,pid);
		}

		c+=l;
		p->found += l;
	}
	if (p->found > 187){
		p->found = 0;
		p->flags = 0;
		p->adapt_length = 0;
	}
	if (c < count) 
		ts_to(buf+c, count-c, p, pesto, func);
}

void ts_av_pes(u8 *buf, int count, t2a *p, p2a *pesto)
{
	ts_to(buf,count,p,pesto,kpes_to_av_pes);
}

void out_pes(u8 *buf, int count, p2a *p)
{
	u8  pbuf[MAX_PLENGTH];
	u8  apbuf[MAX_PLENGTH];
	static int first =1;
	static int afirst =1;
	static int c=0;
	static int ac=0;
	
		
	if ( p->av == VIDEO){
		if (first){
			p->plength = MAX_PLENGTH;
			p->found = 1;
			first = 0;
		}
		if (p->found == 0 && c > 6){
			short *ll = (short *) &pbuf[4];
			short l = c-6;
			*ll = htons(l);
			write(p->fd1,pbuf,c);
			c = 0;
			p->plength = MAX_PLENGTH;
			p->found = 1;
		} 
		if(c+count < MAX_PLENGTH){
			memcpy(pbuf+c,buf,count);
			c+=count;
		}
	}
	if ( p->av == AUDIO){
		if (afirst){
			p->plength = MAX_PLENGTH;
			p->found = 1;
			afirst = 0;
		}
		
		if (p->found == 0 && ac > 6){
			write(p->fd1,apbuf,ac);
			ac = 0;
			p->plength = MAX_PLENGTH;
			p->found = 1;
		} 
		if(ac+count < MAX_PLENGTH){
			memcpy(apbuf+ac,buf,count);
			ac+=count;
		}
	}
}

void ts_demux(u8 *buf, int count, t2a *p, p2a *pesto)
{
	ts_to(buf,count,p,pesto,out_pes);
}



void init_trans(trans *p)
{
	int i;

	p->found = 0;
	p->pes = 0;
	p->is_full = 0;
	p->pes_start = 0;
	p->pes_started = 0;
	p->set = 0;

	for (i = 0; i < MASKL*MAXFILT ; i++){
		p->mask[i] = 0;
		p->filt[i] = 0;
	}
	for (i = 0; i < MAXFILT ; i++){
		p->sec[i].found = 0;
		p->sec[i].length = 0;
	}	
}

int set_trans_filt(trans *p, int filtn, u16 pid, u8 *mask, u8 *filt, int pes)
{
	int i;
	int off;

	if ( filtn > MAXFILT-1 || filtn<0 ) return -1;
	p->pid[filtn] = pid;
	if (pes) p->pes |= (tflags)(1 << filtn);
	else {
		off = MASKL*filtn;
		p->pes &= ~((tflags) (1 << filtn) );
		for (i = 0; i < MASKL ; i++){
			p->mask[off+i] = mask[i];
			p->filt[off+i] = filt[i];
		}
	}		
	p->set |= (tflags) (1 << filtn);
	return 0;
}

void clear_trans_filt(trans *p,int filtn)
{
	int i;

	p->set &= ~((tflags) (1 << filtn) );
	p->pes &= ~((tflags) (1 << filtn) );
	p->is_full &= ~((tflags) (1 << filtn) );
	p->pes_start &= ~((tflags) (1 << filtn) );
	p->pes_started &= ~((tflags) (1 << filtn) );

	for (i = MASKL*filtn; i < MASKL*(filtn+1) ; i++){
		p->mask[i] = 0;
		p->filt[i] = 0;
	}
	p->sec[filtn].found = 0;
	p->sec[filtn].length = 0;
}

int filt_is_set(trans *p, int filtn)
{
	if (p->set & ((tflags)(1 << filtn))) return 1;
	return 0;
}

int pes_is_set(trans *p, int filtn)
{
	if (p->pes & ((tflags)(1 << filtn))) return 1;
	return 0;
}

int pes_is_started(trans *p, int filtn)
{
	if (p->pes_started & ((tflags)(1 << filtn))) return 1;
	return 0;
}

int pes_is_start(trans *p, int filtn)
{
	if (p->pes_start & ((tflags)(1 << filtn))) return 1;
	return 0;
}

int filt_is_ready(trans *p,int filtn)
{
	if (p->is_full & ((tflags)(1 << filtn))) return 1;
	return 0;
}

void trans_filt(u8 *buf, int count, trans *p)
{
	int c=0;
	//fprintf(stderr,"trans_filt\n");
	

	while (c < count && p->found <1 ){
		if ( buf[c] == 0x47) p->found = 1;
		c++;
		p->packet[0] = 0x47;
	}
	if (c == count) return;
	
	while( c < count && p->found < 188 && p->found > 0 ){
		p->packet[p->found] = buf[c];
		c++;
		p->found++;
	}
	if (p->found == 188){
		p->found = 0;
		filter(p);
	}

	if (c < count) trans_filt(buf+c,count-c,p);
} 


void filter(trans *p)
{
	int l,c;
	int tpid;
	u8 flag,flags;
	u8 adapt_length = 0;
	u8 cpid[2];


	//	fprintf(stderr,"filter\n");

	cpid[0] = p->packet[1];
	cpid[1] = p->packet[2];
	tpid = get_pid(cpid);

	flag = cpid[0];
	flags = p->packet[3];
	
	if ( flags & ADAPT_FIELD ) {
		// adaption field
		adapt_length = p->packet[4];
	}

	c = 5 + adapt_length - (int)(!(flags & ADAPT_FIELD));
	if (flags & PAYLOAD){
		for ( l = 0; l < MAXFILT ; l++){
			if ( filt_is_set(p,l) ) {
				if ( p->pid[l] == tpid) {
					if ( pes_is_set(p,l) ){
						if (cpid[0] & PAY_START){
							p->pes_started |= 
								(tflags) 
								(1 << l);
							p->pes_start |= 
								(tflags) 
								(1 << l);
						} else {
							p->pes_start &= ~ 
								((tflags) 
								(1 << l));
						}
						pes_filter(p,l,c);
					} else {
						sec_filter(p,l,c);
					}	
				}
			}
		}
	}
}	


void pes_filter(trans *p, int filtn, int off)
{
	int count,c;
	u8 *buf;

	if (filtn < 0 || filtn >= MAXFILT) return; 

	count = 188 - off;
	c = 188*filtn;
	buf = p->packet+off;
	if (pes_is_started(p,filtn)){
		p->is_full |= (tflags) (1 << filtn);
		memcpy(p->transbuf+c,buf,count);
		p->transcount[filtn] = count;
	}
}

section *get_filt_sec(trans *p, int filtn)
{
	section *sec;
	
	sec = &p->sec[filtn];
	p->is_full &= ~((tflags) (1 << filtn) );
	return sec;
}

int get_filt_buf(trans *p, int filtn,u8 **buf)
{
	*buf = p->transbuf+188*filtn;
	p->is_full &= ~((tflags) (1 << filtn) );
	return p->transcount[filtn];
}




void sec_filter(trans *p, int filtn, int off)
{
	int i,j;
	int error;
	int count,c;
	u8 *buf, *secbuf;
	section *sec;

	//	fprintf(stderr,"sec_filter\n");

	if (filtn < 0 || filtn >= MAXFILT) return; 

	count = 188 - off;
	c = 0;
	buf = p->packet+off;
	sec = &p->sec[filtn];
	secbuf = sec->payload;
	if(!filt_is_ready(p,filtn)){
		p->is_full &= ~((tflags) (1 << filtn) );
		sec->found = 0;
		sec->length = 0;
	}
		
	if ( !sec->found ){
		c = buf[c]+1;
		if (c >= count) return;
		sec->id = buf[c];
		secbuf[0] = buf[c];
		c++;
		sec->found++;
		sec->length = 0;
	}
	
	while ( c < count && sec->found < 3){
		secbuf[sec->found] = buf[c];
		c++;
		sec->found++;
	}
	if (c == count) return;
	
	if (!sec->length && sec->found == 3){
		sec->length |= ((secbuf[1] & 0x0F) << 8); 
		sec->length |= (secbuf[2] & 0xFF);
	}
	
	while ( c < count && sec->found < sec->length+3){
		secbuf[sec->found] = buf[c];
		c++;
		sec->found++;
	}

	if ( sec->length && sec->found == sec->length+3 ){
		error=0;
		for ( i = 0; i < MASKL; i++){
			if (i > 0 ) j=2+i;
			else j = 0;
			error += (sec->payload[j]&p->mask[MASKL*filtn+i])^
				(p->filt[MASKL*filtn+i]&
				 p->mask[MASKL*filtn+i]);
		}
		if (!error){
			p->is_full |= (tflags) (1 << filtn);
		}
		if (buf[0]+1 < c ) c=count;
	}
	
	if ( c < count ) sec_filter(p, filtn, off);

}


void ts_to2(int fd, int fd1, int fd2, void (*func)(u8 *buf, int count, p2a *p))
{
	trans p;
	u8 mask[16];
	u8 filt[16];
	int i,count,c;
	u8 *cbuf;
	u8 buf[188];
	p2a p1;
	p2a p2;
	unsigned short pid1 = 0;
	unsigned short pid2 = 0;
	unsigned short pid = 0;
	unsigned short ilength;
	section *sec;

	init_trans(&p);
	init_p2a(&p1);
	init_p2a(&p2);
	p1.fd1 = fd1;
	p2.fd1 = fd2;
	p1.av  = VIDEO;
	p2.av  = AUDIO;

	for ( i = 0; i<16; i++){
		mask[i] = 0x00;
		filt[i] = 0x00;
	}
	mask[0]= 0xFF;
	filt[0]= 0x00;
	
	clear_trans_filt(&p,0);
	set_trans_filt(&p,0,0,mask,filt,0);

	count = 1;
	while (count > 0 && pid == 0){
		count = read(fd,buf,188);
		trans_filt(buf, count, &p);
		if (filt_is_ready(&p,0)){
			sec = get_filt_sec(&p,0);
			for (i = 8; i< sec->length-1 && pid == 0; i+=4){
				if (sec->payload[i] != 0 || 
				    sec->payload[i+1] !=0){
					pid = get_pid(sec->payload+i+2);
				}
			}
		}
	}
		
	
	mask[0]= 0xFF;
	filt[0]= 0x02;
	clear_trans_filt(&p,0);
	set_trans_filt(&p,0,pid,mask,filt,0);
		
	while (count > 0 && pid1 == 0 && pid2 == 0){
		count = read(fd,buf,188);
		trans_filt(buf, count, &p);
		if (filt_is_ready(&p,0)){
			sec = get_filt_sec(&p,0);
			pid = 0;
//			write(STDOUT_FILENO,sec->payload,sec->length+3);
			ilength = (unsigned short)((sec->payload[10]&3)<<8);
			ilength |= (sec->payload[11]&0xFF);
			for (i = 12+ilength; i< sec->length-1; i+=5){
				if (sec->payload[i] == 0x02){
					pid1 = get_pid(sec->payload+i+1);
				}
				if (sec->payload[i] == 0x03 ||
				    sec->payload[i] == 0x04){
					pid2 = get_pid(sec->payload+i+1);
				}
				if (pid1 && pid2) break;
				i+=((sec->payload[i+3]&15)<<8)|
					sec->payload[i+4];

			}
		}
	}
		


	set_trans_filt(&p,0,pid1,NULL,NULL,1);
	set_trans_filt(&p,1,pid2,NULL,NULL,1);
			
	while (count > 0){
		count = read(fd,buf,188);
		trans_filt(buf, count, &p);

		if (filt_is_ready(&p,0)){
			if ( pes_is_start(&p,0) ){
				if (p1.plength == MAX_PLENGTH){
					p1.found = 0;
				}
			}
			c = get_filt_buf(&p,0,&cbuf);
			func(cbuf, c,&p1);
		}

		if (filt_is_ready(&p,1)){
			c = get_filt_buf(&p,1,&cbuf);
			if ( pes_is_start(&p,1) ){
				if (p2.plength == MAX_PLENGTH){
					p2.found = 0;
				}
			}
			func(cbuf, c,&p2);
		}
	}
} 

void ts_av_pes2(int fd)
{
	ts_to2(fd,0,0,kpes_to_av_pes);
}

void ts_demux2(int fd, int fd1, int fd2)
{
	ts_to2(fd,fd1,fd2,out_pes);
}




void get_pespts(u8 *av_pts,u8 *pts)
{
	
	pts[0] = 0x21 | 
		((av_pts[0] & 0xC0) >>5);
	pts[1] = ((av_pts[0] & 0x3F) << 2) |
		((av_pts[1] & 0xC0) >> 6);
	pts[2] = 0x01 | ((av_pts[1] & 0x3F) << 2) |
		((av_pts[2] & 0x80) >> 6);
	pts[3] = ((av_pts[2] & 0x7F) << 1) |
		((av_pts[3] & 0x80) >> 7);
	pts[4] = 0x01 | ((av_pts[3] & 0x7F) << 1);
}


void init_a2p(a2p *p)
{
	int i;

	p->fd = STDOUT_FILENO;
	p->type = PES_STREAM;
	p->found = 0;
	p->length = 0;
	p->plength = 0;
	p->cid = 0;
	p->flags = 0;
	p->headr = 0;
	p->alength=0;
	p->vlength=0;
	p->pid0 = 0;
	p->pid1 = 0;
	p->pidv = 0;
	p->pida = 0;

	for ( i= 0; i < 4; i++){
		p->last_av_pts[i] = 0;
		p->av_pts[i] = 0;
		p->scr[i] = 0;
	}
}

void dec_pts(u8 *pts, int num)
{
	u32 *s1,s2;
	
	s1 = (u32 *) pts;
	s2 = ntohl (*s1);
	s2 -= num;
	*s1 = htonl(s2);
} 

void inc_scr(a2p *p)
{
	u32 *s1,s2,*s3;
	
	s1 = (u32 *) p->scr;
	s3 = (u32 *) p->last_av_pts;
	s2 = ntohl(*s3);
	s2 -= 9000;
	*s1 = htonl(s2);
//	fprintf (stderr,"scr: %d\n",s2);
}


void av_pes_to_pes(u8 *buf,int count, a2p *p)
{
	int c=0;
	int l,i;
	u8 pts[5];
	ps_packet ps;
	
	u8 tspid0[188] = { 0x47,0x40,0x00,0x10,0x00,0x00,0xb0,0x11,0x00,0x00,0xcb,0x00,0x00,0x00,0x00,0xe0,0x10,0x00,0x01,0xe4,0x00,0x2a,0xd6,0x1a,0xfb,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};

	u8 tspid1[188] = { 0x47,0x44,0x00,0x10,0x00,0x02,0xb0,0x1c,0x00,0x01,0xcb,0x00,0x00,0xe0,0xa0,0xf0,0x05,0x48,0x03,0x01,0x00,0x00,0x02,0xe0,0xa0,0xf0,0x00,0x03,0xe0,0x50,0xf0,0x00,0xae,0xea,0x4e,0x48,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};


	while (c < count && p->found < 2 ){
		if (buf[c] == 0x41 ) p->found = 1;
		c++;
		if (p->found && buf[c] == 0x56) {
			p->found = 2;
			c++;
		} else p->found = 0;
	}
	
	if ( c == count ) return;
 
	if ( p->found == 2 ){
		p->cid = buf[c];
		c++;
		p->found++;
	}
	
	if ( c == count ) return;

        if ( p->found > 2 ){
		while ( c < count && p->found < 5){
			c++;
			p->found++;
		}
	}
	
	if ( c == count ) return;
	
	if ( p->found == 5 ){
		p->flags = buf[c];
		c++;
		p->found++;
	}

	if ( c == count ) return;

	if ( p->found == 6 ){
		while ( c < count && p->found < 8){
			if ( p->found == 6 ) {
				p->length =(int)( ( buf[c] & 0xFF ) << 8);
				c++;
				p->found++;
			} else {
				p->length |= ( buf[c] & 0xFF );
				c++;
				p->found++;
			}
		}
	}

	
	if ( p->found > 7 ){
		while ( p->type == TS_STREAM &&
			c < count && p->found < p->length+8){
			switch (p->cid){
			case 0x01:
				if ( p->found == 8){
					tspid0[3] |= (p->pid0++) 
						& 0x0F ;
					tspid0[3] |= (p->pid1++) 
						& 0x0F ;
					write(p->fd,tspid0,188);
					write(p->fd,tspid1,188);
				}
				
				if (!(p->flags & PTS_FLAG)&& !p->headr){
					int pesstart = 0;
					if (p->plength+count-c > MAX_PLENGTH){
						pesstart = 1;
						p->plength = 0;
					}
					p->vlength = w_tsh(0xE0,p->length,
							   NULL,p->vbuf,p,
							   pesstart);
					p->headr = 1;
					if (pesstart) fprintf(stderr,"PESSTART\n");
				}
				
				if ( (p->flags & PTS_FLAG) && !p->headr){
					if ( p->found < 12 ){
						p->av_pts[p->found-8] = buf[c];
						c++;
						p->found++;
					}
					
					if ( p->found == 12 ){
						get_pespts(p->av_pts,pts);
						
						p->vlength = w_tsh(
							0xE0,p->length,
							pts, p->vbuf,p,1);
						p->plength = 0;

						p->headr = 1;
						memcpy(p->last_av_pts,
						       p->av_pts,4);
					}
				}
				


				if ( p->headr ){
					l = count - c;
					if ( l + p->found > p->length+8 )
						l = p->length - p->found+8;
					
					if (p->vlength+l < 188){
						memcpy(p->vbuf+p->vlength, 
						       buf+c, l);
						p->vlength += l;
						c += l;
						p->found += l;
						if (p->found == p->length+8){
							// adaptation field
							p->vbuf[3] |= 0x30;
							l = 188-p->vlength+4;
							memmove(p->vbuf+l,
								p->vbuf+4,
								p->vlength-4);
							p->vbuf[4] = l-5;
							if (l-5)
								p->vbuf[5] = 
									0x00;
							for ( i = 6; i < l; 
							      i++)
								p->vbuf[i] = 
									0xFF;
							p->plength += 
								p->vlength-4; 
							write(p->fd,p->vbuf,
							      188);
							p->vlength = 0;
							p->headr = 0;
							p->flags = 0;
						}
					} else {
						l = 188 - p->vlength;
						memcpy(p->vbuf+p->vlength, 
						       buf+c, l);
						p->vlength += l;
						c += l;
						p->found += l;
						p->plength += p->vlength-4; 
						write(p->fd,p->vbuf,
						      p->vlength);
						p->vlength = 0;
						p->headr = 0;
						p->flags = 0;
					}
				}
				break;

			case 0x02:
				
				if ( !p->headr ){
					int pesstart = 0;
					
					if ( ((p->flags & PTS_FLAG) ||
					      (count -c >4 &&
					       buf[c] == 0x00 &&
					       buf[c+1] == 0x00 &&
					       buf[c+2] == 0x01 &&
					       buf[c+3] == 0xC0))
						) pesstart = 1;					
					p->alength = w_tsh(0xC0,p->length,
							   NULL,p->abuf,p,
							   pesstart);
					p->headr = 1;
				}

				if ( p->headr ){
					l = count - c;
					if ( l + p->found > p->length+8 )
						l = p->length - p->found+8;
					
					if (p->alength+l < 188){
						memcpy(p->abuf+p->alength, 
						       buf+c, l);
						p->alength += l;
						c += l;
						p->found += l;
						if (p->found == p->length+8){
							// adaptation field
							p->abuf[3] |= 0x30;
							l = 188-p->alength+4;
							memmove(p->abuf+l,
								p->abuf+4,
								p->alength-4);
							p->abuf[4] = l-5;
							if (l-5)
								p->abuf[5] = 
									0x00;
							for ( i = 6; i < l; 
							      i++)
								p->abuf[i] = 
									0xFF;
							write(p->fd,p->abuf,
							      188);
							p->alength = 0;
							p->headr = 0;
							p->flags = 0;
						}
					} else {
						l = 188 - p->alength;
						memcpy(p->abuf+p->alength, 
						       buf+c, l);
						p->alength += l;
						c += l;
						p->found += l;
						
						write(p->fd,p->abuf,
						      p->alength);
						p->alength = 0;
						p->headr = 0;
						p->flags = 0;
					}
				}
				break;
			}
		}
	
		while ( p->type != TS_STREAM &&
			c < count && p->found < p->length+8){
			switch (p->cid){
			case 0x01:
				if ( p->found == 8 && (p->flags & PTS_FLAG) 
				     && p->vlength ){
					l = p->vlength - 6;
					p->vbuf[4] = 0;
					p->vbuf[5] = 0;
					p->vbuf[4] |= ((u8)(l >> 8) & 0xFF); 
					p->vbuf[5] |= ((u8)(l) & 0xFF); 
					if (p->type == PS_STREAM){
						inc_scr(p);
						write_ps_headr(&ps,p->scr
							       ,p->fd);
					}
					write(p->fd,p->vbuf,p->vlength);
					p->vlength=0;
					p->headr = 0;
				}
				
				if (!(p->flags & PTS_FLAG)&& !p->headr){
					p->vlength = w_pesh(0xE0,p->length,
							    NULL,p->vbuf);
					p->headr = 1;
				}
				
				if ( (p->flags & PTS_FLAG) && !p->headr){
					if ( p->found < 12 ){
						p->av_pts[p->found-8] = buf[c];
						c++;
						p->found++;
					}
					
					if ( p->found == 12 ){
						get_pespts(p->av_pts,pts);
					
						p->vlength = w_pesh(
							0xE0,p->length,
							pts, p->vbuf);
						
						p->headr = 1;
						memcpy(p->last_av_pts,
						       p->av_pts,4);
					}
				}

				if ( p->headr ){
					l = count - c;
					if ( l + p->found > p->length+8 )
						l = p->length - p->found+8;

					if (p->vlength+l < MAX_PLENGTH){
						memcpy(p->vbuf+p->vlength, 
						       buf+c, l);
						p->vlength += l;
						c += l;
						p->found += l;
					} else {
						inc_scr(p);
						l = p->vlength - 6;
						p->vbuf[4] = 0;
						p->vbuf[5] = 0;
						p->vbuf[4] |= ((u8)(l >> 8) 
							       & 0xFF); 
						p->vbuf[5] |= ((u8)(l) 
							       & 0xFF); 
						write(p->fd,p->vbuf,
						     p->vlength);
						p->vlength=0;
						p->headr = 0;
					}
				}
				break;

			case 0x02:
				l = count - c;
				if ( l + p->found > p->length+8 )
					l = p->length - p->found +8;
				
				if ( p->found == 8 && 
				     p->alength ){
					p->abuf[4] = 0;
					p->abuf[5] = 0;
					p->abuf[4] |= 
						(u8)(((p->alength-6) >> 8) & 
						     0xFF); 
					p->abuf[5] |= 
						(u8)((p->alength-6) & 
						     0xFF); 
					write(p->fd,p->abuf,
					      p->alength);
					if (
						(count -c >4 &&
						 buf[c] == 0x00 &&
						 buf[c+1] == 0x00 &&
						 buf[c+2] == 0x01 &&
						 buf[c+3] == 0xC0)
						){
						p->alength=0;
					} else {
						p->alength = w_pesh(
							0xC0,0,
							NULL, p->abuf);
					}
				}
				
				memcpy(p->abuf+p->alength,buf+c,l);
				p->alength += l;
				c += l;
				p->found += l;

				break;
			}
		}
	}
	
		  
	
	
	if ( p->length+8 == p->found ) {
		p->found = 0;
		p->length = 0;
		p->cid = 0;
		p->flags = 0;
	}
	if ( c < count )
		av_pes_to_pes( buf+c, count-c, p);
}

int w_tsh(u8 id,int length ,u8 *pts, u8 *obuf,a2p *p, int startpes)
{
	int c = 0;
	u8 tsheadv[4] = { 0x47, 0x00, 0xa0, 0x10}; 
	u8 tsheada[4] = { 0x47, 0x00, 0x50, 0x10}; 

	if (id == 0xE0){
		if (startpes) tsheadv[1] = 0x40;
		tsheadv[3] |= (p->pidv++ & 0x0F);
		memcpy(obuf+c,tsheadv,4);
		c+=4;
		if (startpes) c += w_pesh(id,0,pts,obuf+c);
	} 

	if (id == 0xC0){
		if (startpes) tsheada[1] = 0x40;
		tsheada[3] |= (p->pida++ & 0x0F) ;
		memcpy(obuf+c,tsheada,4);
		c+=4;	
	}
	
	return c;
}

int w_pesh(u8 id,int length ,u8 *pts, u8 *obuf)
{
	u8 le[2];
	u8 dummy[3];
	int c;
	u8 headr[3] = {0x00, 0x00, 0x01};
	
	c = 0;
	memcpy(obuf+c,headr,3);
	c += 3;
	memcpy(obuf+c,&id,1);
	c++;

	le[0] = 0;
	le[1] = 0;
	if (length){
		if (pts) length+=8;
		else length+=3;
	}
	le[0] |= ((u8)(length >> 8) & 0xFF); 
	le[1] |= ((u8)(length) & 0xFF); 
	memcpy(obuf+c,le,2);
	c += 2;

	dummy[0] = 0x80;
	dummy[1] = 0;
	dummy[2] = 0;
	if (pts){
		dummy[1] |= PTS_ONLY;
		dummy[2] = 0x05;
	}
	memcpy(obuf+c,dummy,3);
	c += 3;

	if (pts){
		memcpy(obuf+c,pts,5);
		c += 5;
	}
	
	return c;
}


void write_ps_headr(ps_packet *p,u8 *pts,int fd)
{
	u8 buf[40];
	init_ps(p);
	
	p->mpeg = 2;
// SCR = 0
	p->scr[0] = 0x44;
	p->scr[1] = 0x00;
	p->scr[2] = 0x04;
	p->scr[3] = 0x00;
	p->scr[4] = 0x04;
	p->scr[5] = 0x01;

// SCR = PTS
	p->scr[0] = 0x44 | ((pts[0] >> 3)&0x18) | ((pts[0] >> 4)&0x03);
	p->scr[1] = 0x00 | ((pts[0] << 4)&0xF0) | ((pts[1] >> 4)&0x0F);
	p->scr[2] = 0x04 | ((pts[1] << 4)&0xF0) | ((pts[2] >> 4)&0x08)
		         | ((pts[2] >> 5)&0x03);
	p->scr[3] = 0x00 | ((pts[2] << 3)&0xF8) | ((pts[3] >> 5)&0x07);
	p->scr[4] = 0x04 | ((pts[3] << 3)&0xF8);
	p->scr[5] = 0x01;

// MUX Rate = 10080000 bits/sec (DVD)
	p->mux_rate[0] = 0x01;
	p->mux_rate[1] = 0x89;
	p->mux_rate[2] = 0xC3;
	p->stuff_length = 0xF8;

	p->rate_bound[0] = 0x80;
	p->rate_bound[1] = 0xC4;
	p->rate_bound[2] = 0xE1;


	p->sheader_llength[0] = 0x00;
	p->sheader_llength[1] = 0x12;

	setl_ps(p);
	
	p->audio_bound = 0x00;
	p->video_bound = 0xe1;
	p->reserved = 0xFF;

	p->data[0] =  0xB9;
	p->data[1] =  0xE0;
	p->data[2] =  0xE8;
	p->data[3] =  0xB8;
	p->data[4] =  0xC0;
	p->data[5] =  0x20;
	p->data[6] =  0xBD;
	p->data[7] =  0xE0;
	p->data[8] =  0x44;
	p->data[9] =  0xBF;
	p->data[10] = 0xE0;
	p->data[11] = 0x02;
	
	

	
	cwrite_ps(buf, p, 38);
	write(fd,buf,38);
}


void es_to_av_pes(u8 *buf, int l)
{
        long int count, size;
        av_pes_packet apsv;


        init_av_pes(&apsv);
        apsv.stream_id = AV_VIDEO;
        count = 0;
        while (count < l){
                size = l-count;
                if ( size > MAX_AV) size = MAX_AV;
                count += size;
                apsv.length = size;
                memcpy(apsv.payload,buf,size);
		write_av_pes(STDOUT_FILENO, &apsv);
        }

}





void twrite(u8 const *buf)
{
	int l = TS_SIZE;
	int c = 0;
	int w;


	while (l){
		w = write(STDOUT_FILENO,buf+c,l);
		if (w>=0){
			l-=w;
			c+=w;
		}
	}
}

void init_p2t(p2t_t *p, void (*fkt)(u8 const *buf))
{
	memset(p->pes,0,TS_SIZE);
	p->counter = 0;
	p->pos = 0;
	p->frags = 0;
	if (fkt) p->t_out = fkt;
	else p->t_out = twrite;
}

void clear_p2t(p2t_t *p)
{
	memset(p->pes,0,TS_SIZE);
	p->counter = 0;
	p->pos = 0;
	p->frags = 0;
}


long int find_pes_header(u8 const *buf, long int length, int *frags)
{
	int c = 0;
	int found = 0;

	*frags = 0;

	while (c < length-3 && !found) {
		if (buf[c] == 0x00 && buf[c+1] == 0x00 && 
		    buf[c+2] == 0x01) {
			switch ( buf[c+3] ) {
			case 0xBA:
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				found = 1;
				break;
				
			default:
				c++;
				break;
			}	
		} else c++;
	}
	if (c == length-3 && !found){
		if (buf[length-1] == 0x00) *frags = 1;
		if (buf[length-2] == 0x00 &&
		    buf[length-1] == 0x00) *frags = 2;
		if (buf[length-3] == 0x00 &&
		    buf[length-2] == 0x00 &&
		    buf[length-1] == 0x01) *frags = 3;
		return -1;
	}

	return c;
}

void pes_to_ts( u8 const *buf, long int length, u16 pid, p2t_t *p)
{
	int c,c2,l,add;
	int check,rest;

	c = 0;
	c2 = 0;
	if (p->frags){
		check = 0;
		switch(p->frags){
		case 1:
			if ( buf[c] == 0x00 && buf[c+1] == 0x01 ){
				check = 1;
				c += 2;
			}
			break;
		case 2:
			if ( buf[c] == 0x01 ){
				check = 1;
				c++;
			}
			break;
		case 3:
			check = 1;
		}
		if(check){
			switch ( buf[c] ) {
				
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				p->pes[0] = 0x00;
				p->pes[1] = 0x00;
				p->pes[2] = 0x01;
				p->pes[3] = buf[c];
				p->pos=4;
				memcpy(p->pes+p->pos,buf+c,TS_SIZE-4-p->pos);
				c += TS_SIZE-4-p->pos;
				p_to_t(p->pes,TS_SIZE-4,pid,&p->counter,
				       p->t_out);
				clear_p2t(p);
				break;
				
			default:
				c=0;
				break;
			}
		}
		p->frags = 0;
	}
		
	if (p->pos){
		c2 = find_pes_header(buf+c,length-c,&p->frags);
		if (c2 >= 0 && c2 < TS_SIZE-4-p->pos){
			l = c2+c;
		} else l = TS_SIZE-4-p->pos;
		memcpy(p->pes+p->pos,buf,l);
		c += l;
		p->pos += l;
		p_to_t(p->pes,p->pos,pid,&p->counter,
		       p->t_out);
		clear_p2t(p);
	}
			
	add = 0;
	while (c < length){
		c2 = find_pes_header(buf+c+add,length-c-add,&p->frags);
		if (c2 >= 0) {
			c2 += c+add;
			if (c2 > c){
				p_to_t(buf+c,c2-c,pid,&p->counter,
				       p->t_out);
				c = c2;
				clear_p2t(p);
				add = 0;
			} else add = 1;
		} else {
			l = length-c;
			rest = l % (TS_SIZE-4);
			l -= rest;
			p_to_t(buf+c,l,pid,&p->counter,
			       p->t_out);
			memcpy(p->pes,buf+c+l,rest);
			p->pos = rest;
			c = length;
		}
	}
}


int write_ts_header(u16 pid, u8 *counter, int pes_start, u8 *buf, u8 length)
{
	int i;
	int c = 0;
	int fill;
	u8 tshead[4] = { 0x47, 0x00, 0x00, 0x10}; 
        

	fill = TS_SIZE-4-length;
        if (pes_start) tshead[1] = 0x40;
	if (fill) tshead[3] = 0x30;
        tshead[1] |= (u8)((pid & 0x1F00) >> 8);
        tshead[2] |= (u8)(pid & 0x00FF);
        tshead[3] |= ((*counter)++ & 0x0F) ;
        memcpy(buf,tshead,4);
	c+=4;


	if (fill){
		buf[4] = fill-1;
		c++;
		if (fill >1){
			buf[5] = 0x00;
			c++;
		}
		for ( i = 6; i < fill+4; i++){
			buf[i] = 0xFF;
			c++;
		}
	}

        return c;
}


void p_to_t( u8 const *buf, long int length, u16 pid, u8 *counter, 
	    void (*ts_write)(u8 const *))
{
  
	int l, pes_start;
	u8 obuf[TS_SIZE];
	long int c = 0;
	pes_start = 0;
	if ( length > 3 && 
	     buf[0] == 0x00 && buf[1] == 0x00 && buf[2] == 0x01 )
		switch (buf[3]){
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				pes_start = 1;
				break;
				
			default:
				break;
		}			

	while ( c < length ){
		memset(obuf,0,TS_SIZE);
		if (length - c >= TS_SIZE-4){
			l = write_ts_header(pid, counter, pes_start
					     , obuf, TS_SIZE-4);
			memcpy(obuf+l, buf+c, TS_SIZE-l);
			c += TS_SIZE-l;
		} else { 
			l = write_ts_header(pid, counter, pes_start
					     , obuf, length-c);
			memcpy(obuf+l, buf+c, TS_SIZE-l);
			c = length;
		}
		ts_write(obuf);
		pes_start = 0;
	}
}



void init_dmx(dmx_t *p,void (*a_out)(u8 const *buf,long int l),
	      void (*v_out)(u8 const *buf,long int l))
{
	memset(p->pes,0,PBUFFER);
	p->type = NOPES;
	p->frags = 0;
	p->length = 0;
	p->a_out = a_out;
	p->v_out = v_out;
}

void clear_dmx(dmx_t *p)
{
	memset(p->pes,0,PBUFFER);
	p->type = NOPES;
	p->frags = 0;
	p->length = 0;
}


void pes_out( u8 const *buf, long int length,dmx_t *p)
{
	short *ll;

	//fprintf (stderr," buf[3]: %x  ",buf[3]);
	if ( length > 5 && 
	     buf[0] == 0x00 && buf[1] == 0x00 && buf[2] == 0x01 ){
		//fprintf (stderr,"NEW TYPE %x  ",buf[3]);
		switch (buf[3]){
		case 0xBA:
			if (!p->length){
				p->type = NOPES;
				p->length = 0;
			}
			break;

		case PROG_STREAM_MAP:
		case PRIVATE_STREAM2:
		case PROG_STREAM_DIR:
		case ECM_STREAM     :
		case EMM_STREAM     :
		case PADDING_STREAM :
		case DSM_CC_STREAM  :
		case ISO13522_STREAM:
		case PRIVATE_STREAM1:
		case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			if ((buf[4] || buf[5]) && !p->length){
				p->type = AUDIO;
				ll = (short *) (buf+4);
				p->length = ntohs(*ll)+6;
			}
			break;
		case VIDEO_STREAM_S ... VIDEO_STREAM_E:
			if ((buf[4] || buf[5]) && !p->length){
				p->type = VIDEO;
				ll = (short *) (buf+4);
				p->length = ntohs(*ll)+6;
			}
			break;
		default:
			break;
		}			
	}
	switch(p->type){
	case AUDIO:
		p->length -= length;
		p->a_out(buf,length);
		break;
	case VIDEO:
		p->length -= length;
		p->v_out(buf,length);
		break;
	}
	//fprintf (stderr," %d\n\n",p->type);
}

void dmx_pes( u8 const *buf, long int length, dmx_t *p)
{

	long int c, c2;
	int fr,l;
	int add;
	static long int co = 0;

	c = 0;
	c2 = 0;
	fr = 0;

	if(p->frags){
		if (p->frags < 7 && length+p->frags > 7){
			memcpy(p->pes+p->frags,buf,7-p->frags);
			c+= 7-p->frags;
			p->frags = 7;
		}

		if (p->frags > 6) 
			c2 = find_pes_header(p->pes, p->frags-2, &fr);
		else {
			memcpy(p->pes+p->frags,buf,length);
			p->frags += length;
			return;
			// Not enough data 
		}


		if (c2 < 0 && fr > c-2){
			if (length+fr < 7){
				memcpy(p->pes+p->frags,buf,length);
				p->frags += length;
				return;
				// Not enough data 
			}
			memcpy(p->pes+p->frags,buf+c,7-fr);
			c += 7-fr;
			p->frags += 7-fr;
			c2 = find_pes_header(p->pes, p->frags-2, &fr);
		}
		
		if ( c2 < 0){
			//fprintf(stderr,"frags c2<0:  %x  type: %d\n",co+c,p->type);
			pes_out( p->pes, p->frags-c, p);
			c = 0;
		} else {
			//fprintf(stderr,"frags c2>=0:  %x  type: %d c2: %d\n",co+c,p->type,c2);
			if (c2) pes_out( p->pes, c2, p);
			pes_out( p->pes+c2,p->frags-c2,p);
		}
	}
	p->frags = 0;
	add = 0;
	while (c < length){
		//fprintf(stderr,"K:  %x  c: %d add: %d l: %d\n",co+c,c, add,length);
		if ( (l=length-c-add-2) < 1) c2 = -1;
                else {
			c2 = find_pes_header(buf+c+add,l,&p->frags);
			p->frags+=2;
		}
                if (c2 >= 0) {
                        c2 += c+add;
                        if (c2 > c){
				//fprintf(stderr,"c2>c:  %x  type: %d\n",co+c,p->type);
                                pes_out(buf+c,c2-c,p);
                                c = c2;
                                add = 0;
                        } else add = 1;
                } else {
                        l = length-c-p->frags;
			if (l < 6) {
				p->frags += l;
			} else {
				pes_out(buf+c,l,p);
				c+=l;
			}
			//fprintf(stderr,"c2<0:  %x  type: %d  l: %d frags: %d\n",co+c,p->type,l,p->frags);
                        memcpy(p->pes,buf+c,p->frags);
                        c += p->frags;
                }
        }
	co += length;
}




void copy_ptslm(PTS_List *a, PTS_List *b)
{
	a->pos  = b->pos;
	a->PTS  = b->PTS;
	a->pes  = b->pos;
	a->spos = b->PTS;
}

void clear_ptslm(PTS_List *a)
{
	a->pos  = 0;
	a->PTS  = 0;
	a->pes  = 0;
	a->spos = 0;	
}

void init_ptsl(PTS_List *ptsl)
{
	int i;
	for (i=0;i< MAX_PTS;i++){
		clear_ptslm(&ptsl[i]);
	}
}

int del_pts(PTS_List *ptsl, int pos, int nr)
{
	int i;
	int del = 0;

	for( i = 0; i < nr-1; i++){
		if(pos > ptsl[i].pos && pos >= ptsl[i+1].pos) del++;
	}

	if(del)
		for( i = 0; i < nr-del; i++){
			copy_ptslm(&ptsl[i], &ptsl[i+del]);
		}

	return nr-del;
}

int add_pts(PTS_List *ptsl, u32 pts, int pos, int spos, int nr, long pes)
{
	int i;

	if (nr == MAX_PTS) {
		nr = del_pts(ptsl, ptsl[1].pos+1, nr);
	} else nr++;
	i = nr-1;
	
	ptsl[i].pos  = pos;
	ptsl[i].spos = spos;
	ptsl[i].PTS  = pts;
	ptsl[i].pes  = pes;
	return nr;
}

void add_vpts(Remux *rem, u8 *pts)
{
	rem->vptsn = add_pts(rem->vpts_list,ntohl(trans_pts_dts(pts))
			     ,rem->vwrite,rem->awrite,rem->vptsn,rem->vpes);
}

void add_apts(Remux *rem, u8 *pts)
{
	rem->aptsn = add_pts(rem->apts_list,ntohl(trans_pts_dts(pts))
			     ,rem->awrite,rem->vwrite,rem->aptsn,rem->apes);
}

void del_vpts(Remux *rem)
{
	rem->vptsn = del_pts(rem->vpts_list, rem->vread, rem->vptsn);
}

void del_apts(Remux *rem)
{
	rem->aptsn = del_pts(rem->apts_list, rem->aread, rem->aptsn);
}


void copy_framelm(FRAME_List *a, FRAME_List *b)
{
	a->type  = b->type;
	a->pos   = b->pos;
	a->FRAME = b->FRAME;
	a->time  = b->time;
}

void clear_framelm(FRAME_List *a)
{
	a->type  = 0;
	a->pos   = 0;
	a->FRAME = 0;
	a->time  = 0; 
}

void init_framel(FRAME_List *framel)
{
	int i;
	for (i=0;i< MAX_FRAME;i++){
		clear_framelm(&framel[i]);
	}
}

int del_frame(FRAME_List *framel, int pos, int nr)
{
	int i;
	int del = 0;

	for( i = 0; i < nr-1; i++){
		if(pos > framel[i].pos && pos >= framel[i+1].pos) del++;
	}

	if(del)
		for( i = 0; i < nr-del; i++){
			copy_framelm(&framel[i], &framel[i+del]);
		}

	return nr-del;
}

int add_frame(FRAME_List *framel, u32 frame, int pos, int type, int nr, 
	      u32 time)
{
	int i;

	if (nr == MAX_FRAME) {
		nr = del_frame(framel, framel[1].pos+1, nr);
	} else nr++;
	i = nr-1;
	
	framel[i].type  = type;
	framel[i].pos   = pos;
	framel[i].FRAME = frame;
	framel[i].time  = time;
	return nr;
}

void add_vframe(Remux *rem, u32 frame, long int pos, int type, int time)
{
	rem->vframen = add_frame(rem->vframe_list, frame, pos, type,
				 rem->vframen, time);
}

void add_aframe(Remux *rem, u32 frame, long int pos)
{
	rem->aframen = add_frame(rem->aframe_list, frame, pos, 0, 
				 rem->aframen, 0);
}

void del_vframe(Remux *rem)
{
	rem->vframen = del_frame(rem->vframe_list, rem->vread, rem->vframen);
}

void del_aframe(Remux *rem)
{
	rem->aframen = del_frame(rem->aframe_list, rem->aread, rem->aframen);
}


void find_vframes( Remux *rem, u8 *buf, int l)
{
 	int c = 0;
	int type;
	char *frames[3] = {"I-Frame","P-Frame","B-Frame"};
	u32 time = 0;
	int hour;
	int min;
	int sec;

	while ( c < l - 6){
		if (buf[c] == 0x00 && 
		    buf[c+1] == 0x00 &&
		    buf[c+2] == 0x01 && 
		    buf[c+3] == 0xB8) {
			c += 4;
			hour = (int)((buf[c]>>2)& 0x1F);
			min  = (int)(((buf[c]<<4)& 0x30)| 
				     ((buf[c+1]>>4)& 0x0F));
			sec  = (int)(((buf[c+1]<<3)& 0x38)|
				      ((buf[c+2]>>5)& 0x0F));
  
			time = 3600*hour + 60*min + sec;
/*
			fprintf(stderr,"         TIME hours: %3d", hour);
			fprintf(stderr,"  minutes: %2d", min);
			fprintf(stderr," seconds: %2d\r", sec);
*/			
		}
		if ( buf[c] == 0x00 && 
		     buf[c+1] == 0x00 &&
		     buf[c+2] == 0x01 && 
		     buf[c+3] == 0x00) {
			c += 4;
			type = ((buf[c+1]&0x38) >>3);
//			fprintf(stderr," %s \r",frames[type-1]);
			rem->vframe++;
			add_vframe( rem, rem->vframe, rem->vwrite+c, type, 
				    time);
		} else c++;
	}
}

void find_aframes( Remux *rem, u8 *buf, int l)
{
 	int c = 0;

	while ( c < l - 2){
		if ( buf[c] == 0xFF && 
		     (buf[c+1] & 0xF8) == 0xF8 ) {
			c += 2;
			rem->aframe++;
			add_aframe( rem, rem->aframe, rem->awrite+c);
		} else c++;
	}
}

int refill_buffy(Remux *rem)
{
	pes_packet pes;
	int count = 0;
	int acount, vcount;
	ringbuffy *vbuf = &rem->vid_buffy;
	ringbuffy *abuf = &rem->aud_buffy;
	int fin = rem->fin;

	acount = abuf->size-ring_rest(abuf);
	vcount = vbuf->size-ring_rest(vbuf);
	
	
	while ( acount > MAX_PLENGTH && vcount > MAX_PLENGTH && count < 10){
		int neof;
		count++;
		init_pes(&pes);
		if ((neof = read_pes(fin,&pes)) <= 0) return -1;
		switch(pes.stream_id){
		case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			rem->apes++;
			if(pes.flags2 & PTS_DTS)
				add_apts(rem, pes.pts);
			find_aframes( rem, pes.pes_pckt_data, pes.length);
			ring_write(abuf,(char *)pes.pes_pckt_data,pes.length);
			rem->awrite += pes.length;
			break;
		case VIDEO_STREAM_S ... VIDEO_STREAM_E:
			rem->vpes++;
			if(pes.flags2 & PTS_DTS)
				add_vpts(rem, pes.pts);
			find_vframes( rem, pes.pes_pckt_data, pes.length);
			ring_write(vbuf,(char *)pes.pes_pckt_data,pes.length);
			rem->vwrite += pes.length;
			break;
		}
		acount = abuf->size-ring_rest(abuf);
		vcount = vbuf->size-ring_rest(vbuf);
		kill_pes(&pes);
	}
	return 0;
}

int vring_read( Remux *rem, u8 *buf, int l)
{
	int c = 0;
	int r = 0;

	if (ring_rest(&rem->vid_buffy) <= l)
		r = refill_buffy(rem);
	if (r) return -1;

	c = ring_read(&rem->vid_buffy, (char *) buf, l);
	rem->vread += c;
	del_vpts(rem);
	del_vframe(rem);
	return c;
}

int aring_read( Remux *rem, u8 *buf, int l)
{
	int c = 0;
	int r = 0;

	if (ring_rest(&rem->aud_buffy) <= l)
		r = refill_buffy(rem);
	if (r) return -1;
	
	c = ring_read(&rem->aud_buffy, (char *)buf, l);
	rem->aread += c;
	del_apts(rem);
	del_aframe(rem);
	return c;
}

int vring_peek( Remux *rem, u8 *buf, int l, long off)
{
	int c = 0;
	
	if (ring_rest(&rem->vid_buffy) <= l)
		refill_buffy(rem);

	c = ring_peek(&rem->vid_buffy, (char *) buf, l, off);
	return c;
}

int aring_peek( Remux *rem, u8 *buf, int l, long off)
{
	int c = 0;

	if (ring_rest(&rem->aud_buffy) <= l)
		refill_buffy(rem);

	c = ring_peek(&rem->aud_buffy, (char *)buf, l, off);
	return c;
}


int get_video_info(Remux *rem)
{
	u8 buf[12];
	u8 *headr;
	int found = 0;
        int sw;
	long off = 0;
	int form = -1;
	ringbuffy *vid_buffy = &rem->vid_buffy;
	VideoInfo *vi = &rem->video_info;

	while (found < 4 && ring_rest(vid_buffy)){
		u8 b[3];

		vring_peek( rem, b, 4, 0);
		if ( b[0] == 0x00 && b[1] == 0x00 && b[2] == 0x01
		     && b[3] == 0xb3) found = 4;
		else {
			off++;
			vring_read( rem, b, 1);
		}
	}

	if (! found) return -1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x01; buf[3] = 0xb3;
	headr = buf+4;
	if(vring_peek(rem, buf, 12, 0) < 12) return -1;

	vi->horizontal_size	= ((headr[1] &0xF0) >> 4) | (headr[0] << 4);
	vi->vertical_size	= ((headr[1] &0x0F) << 8) | (headr[2]);
    
        sw = (int)((headr[3]&0xF0) >> 4) ;

        switch( sw ){
	case 1:
		fprintf(stderr,"Videostream: ASPECT: 1:1");
		vi->aspect_ratio = 100;        
		break;
	case 2:
		fprintf(stderr,"Videostream: ASPECT: 4:3");
                vi->aspect_ratio = 133;        
		break;
	case 3:
		fprintf(stderr,"Videostream: ASPECT: 16:9");
                vi->aspect_ratio = 177;        
		break;
	case 4:
		fprintf(stderr,"Videostream: ASPECT: 2.21:1");
                vi->aspect_ratio = 221;        
		break;

        case 5 ... 15:
		fprintf(stderr,"Videostream: ASPECT: reserved");
                vi->aspect_ratio = 0;        
		break;

        default:
                vi->aspect_ratio = 0;        
                return -1;
	}

        fprintf(stderr,"  Size = %dx%d",vi->horizontal_size,vi->vertical_size);

        sw = (int)(headr[3]&0x0F);

        switch ( sw ) {
	case 1:
		fprintf(stderr,"  FRate: 23.976 fps");
                vi->framerate = 24000/1001;
		form = -1;
		break;
	case 2:
		fprintf(stderr,"  FRate: 24 fps");
                vi->framerate = 24;
		form = -1;
		break;
	case 3:
		fprintf(stderr,"  FRate: 25 fps");
                vi->framerate = 25;
		form = VIDEO_MODE_PAL;
		break;
	case 4:
		fprintf(stderr,"  FRate: 29.97 fps");
                vi->framerate = 30000/1001;
		form = VIDEO_MODE_NTSC;
		break;
	case 5:
		fprintf(stderr,"  FRate: 30 fps");
                vi->framerate = 30;
		form = VIDEO_MODE_NTSC;
		break;
	case 6:
		fprintf(stderr,"  FRate: 50 fps");
                vi->framerate = 50;
		form = VIDEO_MODE_PAL;
		break;
	case 7:
		fprintf(stderr,"  FRate: 60 fps");
                vi->framerate = 60;
		form = VIDEO_MODE_NTSC;
		break;
	}


	vi->bit_rate = 400*((headr[4] << 10) | (headr[5] << 2) | 
			    ((headr[6] & 0xC0) >> 6));
	
        fprintf(stderr,"  BRate: %.2f Mbit/s",(vi->bit_rate)/1000000.);
	
        fprintf(stderr,"\n");
        vi->video_format = form;

	/*
	marker_bit (&video_bs, 1);
	vi->vbv_buffer_size	= getbits (&video_bs, 10);
	vi->CSPF		= get1bit (&video_bs);
	*/
	return form;
}


int get_audio_info( Remux *rem)
{
	static unsigned int bitrates[3][16] =
	{{0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,0},
	 {0,32,48,56,64,80,96,112,128,160,192,224,256,320,384,0},
	 {0,32,40,48,56,64,80,96,112,128,160,192,224,256,320,0}};

	static double freq[4] = {44.1, 48, 32, 0};

	u8 *headr;
	u8 buf[3];
	long off = 0;
	int found = 0;
	ringbuffy *aud_buffy = &rem->aud_buffy;
	AudioInfo *ai = &rem->audio_info;
	
	while(!ring_rest(aud_buffy) && !refill_buffy(rem));
	while (found < 2 && ring_rest(aud_buffy)){
		u8 b[2];
		refill_buffy(rem);
		aring_peek( rem, b, 2, 0);

		if ( b[0] == 0xff && (b[1] & 0xf8) == 0xf8)
			found = 2;
		else {
			off++;
			aring_read( rem, b, 1);
		}
	}	

	if (!found) return -1;
	
	if (aring_peek(rem, buf, 3, 0) < 1) return -1;
	headr = buf+2;

	ai->layer = (buf[1] & 0x06) >> 1;

        fprintf(stderr,"Audiostream: Layer: %d", 4-ai->layer);


	ai->bit_rate = bitrates[(3-ai->layer)][(headr[0] >> 4 )]*1000;

	if (ai->bit_rate == 0)
		fprintf (stderr,"  Bit rate: free");
	else if (ai->bit_rate == 0xf)
		fprintf (stderr,"  BRate: reserved");
	else
		fprintf (stderr,"  BRate: %d kb/s", ai->bit_rate/1000);
	

	ai->frequency = (headr[0] & 0x0c ) >> 2;
	if (ai->frequency == 3)
		fprintf (stderr, "  Freq: reserved\n");
	else
		fprintf (stderr,"  Freq: %2.1f kHz\n", 
			 freq[ai->frequency]);

	return 0;
}


int write_pes_header(u8 id,int length , long PTS, u8 *obuf)
{
	u8 le[2];
	u8 dummy[3];
	u8 *pts;
	u8 ppts[5];
	long lpts;
	int c;
	u8 headr[3] = {0x00, 0x00, 0x01};
	
	lpts = htonl(PTS);
	pts = (u8 *) &lpts;
	
	get_pespts(pts,ppts);

	c = 0;
	memcpy(obuf+c,headr,3);
	c += 3;
	memcpy(obuf+c,&id,1);
	c++;

	le[0] = 0;
	le[1] = 0;
	length -= 6;

	le[0] |= ((u8)(length >> 8) & 0xFF); 
	le[1] |= ((u8)(length) & 0xFF); 
	memcpy(obuf+c,le,2);
	c += 2;

	dummy[0] = 0x80;
	dummy[1] = 0;
	dummy[2] = 0;
	if (PTS){
		dummy[1] |= PTS_ONLY;
		dummy[2] = 0x05;
	}
	memcpy(obuf+c,dummy,3);
	c += 3;

	if (PTS){
		memcpy(obuf+c,ppts,5);
		c += 5;
	}
	
	return c;
}


int write_ps_header(u8 *buf, long SCR, long muxr)
{
	ps_packet p;
	u8 *pts;
	long lpts;
	init_ps(&p);
	
	lpts = htonl(SCR);
	pts = (u8 *) &lpts;

	
	p.mpeg = 2;
// SCR = 0
	p.scr[0] = 0x44;
	p.scr[1] = 0x00;
	p.scr[2] = 0x04;
	p.scr[3] = 0x00;
	p.scr[4] = 0x04;
	p.scr[5] = 0x01;
	
// SCR = PTS
	p.scr[0] = 0x44 | ((pts[0] >> 3)&0x18) | ((pts[0] >> 4)&0x03);
	p.scr[1] = 0x00 | ((pts[0] << 4)&0xF0) | ((pts[1] >> 4)&0x0F);
	p.scr[2] = 0x04 | ((pts[1] << 4)&0xF0) | ((pts[2] >> 4)&0x08)
		| ((pts[2] >> 5)&0x03);
	p.scr[3] = 0x00 | ((pts[2] << 3)&0xF8) | ((pts[3] >> 5)&0x07);
	p.scr[4] = 0x04 | ((pts[3] << 3)&0xF8);
	p.scr[5] = 0x01;
	
// MUX Rate = 10080000 bits/sec (DVD)
	p.mux_rate[0] = ((muxr & 0x3FC000) >> 12) & 0xFF;
	p.mux_rate[1] = ((muxr & 0x003FC0) >>  6) & 0xFF;
	p.mux_rate[2] = (((muxr & 0x00003F) <<  2) & 0xFF) | 0x03;
	p.stuff_length = 0xF8;
	
	p.rate_bound[0] = 0x80;
	p.rate_bound[1] = 0xC4;
	p.rate_bound[2] = 0xE1;

	
	p.sheader_llength[0] = 0x00;
	p.sheader_llength[1] = 0x12;

	setl_ps(&p);
	
	p.audio_bound = 0x00;
	p.video_bound = 0xe1;
	p.reserved = 0xFF;

	p.data[0] =  0xB9;
	p.data[1] =  0xE0;
	p.data[2] =  0xE8;
	p.data[3] =  0xB8;
	p.data[4] =  0xC0;
	p.data[5] =  0x20;
	p.data[6] =  0xBD;
	p.data[7] =  0xE0;
	p.data[8] =  0x44;
	p.data[9] =  0xBF;
	p.data[10] = 0xE0;
	p.data[11] = 0x02;

	
	cwrite_ps(buf, &p, 38);
	return 38;
}

void init_remux(Remux *rem, int fin, int fout, int mult)
{
	rem->fin = fin;
	rem->fout = fout;
	ring_init(&rem->vid_buffy, 40*BUFFYSIZE*mult);
	ring_init(&rem->aud_buffy, BUFFYSIZE*mult);
	init_ptsl(rem->vpts_list);
	init_ptsl(rem->apts_list);
	init_framel(rem->vframe_list);
	init_framel(rem->aframe_list);

	rem->vptsn     = 0;
	rem->aptsn     = 0;
	rem->vframen   = 0;
	rem->aframen   = 0;
	rem->vframe    = 0;
	rem->aframe    = 0;
	rem->vcframe   = 0;
	rem->acframe   = 0;
	rem->vpts      = 0;
	rem->apts      = 0;
	rem->vpes      = 0;
	rem->apes      = 0;
	rem->vpts_old  = 0;
	rem->apts_old  = 0;
	rem->SCR       = 0;
	rem->SCR_off   = 0;
	rem->vwrite    = 0;
	rem->awrite    = 0;
	rem->vread     = 0;
	rem->aread     = 0;
	rem->muxr      = 0;
	rem->pack_size = 0;
}

u32 bytes2pts(int bytes, int rate)
{
	if (bytes < 0xFFFFFFFFUL/720000UL)
		return (u32)(bytes*720000UL/rate);
	else
		return (u32)(bytes/rate*720000UL);
}

long pts2bytes( u32 pts, int rate)
{
	if (pts < 0xEFFFFFFFUL/rate)
		return (pts*rate/720000);
	else 
		return (pts* (rate/720000));
}

int write_audio_pes( Remux *rem, u8 *buf, int length)
{
	int add;
	int pos = 0;
	int p   = 0;
	u32 pts = 0;

	if (!length) return 0;

	if (rem->apts_old != rem->apts){
		pts = rem->apts;
		p = 5;
	}
	if ( length+47+p >= rem->pack_size){
		length = rem->pack_size;
	} else length = length+47+p;
	if (rem->pack_size-length <= 7) length -= 10; 

	pos = write_ps_header(buf,rem->SCR,rem->muxr);

	pos += write_pes_header( 0xC0, length-pos, pts, buf+pos);
	add = aring_read( rem, buf+pos, length-pos);
	if (add < 0) return -1;
	pos += add;
	rem->apts_old = rem->apts;
	rem->apts = rem->apts_list[0].PTS;
	if (pos+7 < rem->pack_size){
		pos += write_pes_header( PADDING_STREAM, rem->pack_size-pos, 0,
					 buf+pos);
		memset(buf+pos,0xff,rem->pack_size-pos);
		pos = rem->pack_size;
	}		

	return pos;
}

int write_video_pes( Remux *rem, u8 *buf, int length)
{
	int add;
	int pos = 0;
	int p   = 0;
	u32 pts = 0;
	
	rem->SCR = rem->vpts - rem->SCR_off;
	if (! length) return 0;

	if (rem->vpts_old != rem->vpts){
		pts = rem->vpts;
		p = 5;
	}
	if ( length+47+p >= rem->pack_size){
		length = rem->pack_size;
	} else length = length+47+p;
	if (rem->pack_size-length <= 7) length -= 10; 

	pos = write_ps_header(buf,rem->SCR,rem->muxr);
	pos += write_pes_header( 0xE0, length-pos, pts, buf+pos);
	add = vring_read( rem, buf+pos, length-pos);
	if (add < 0) return -1;
	pos += add;
	rem->vpts_old = rem->vpts;
	rem->vpts = rem->vpts_list[0].PTS;
	if (pos+7 < rem->pack_size){
		pos += write_pes_header( PADDING_STREAM, rem->pack_size-pos, 0,
					 buf+pos);
		memset(buf+pos,0xff,rem->pack_size-pos);
		pos = rem->pack_size;
	}		
	return pos;
}

void remux(int fin, int fout, int pack_size, int mult)
{
	Remux rem;
	long ptsdiff;
	u8 buf[MAX_PACK_L];
	long pos = 0;
	u32 time_off = 0;
	int ratio = 0;
	long apos, vpos;
	int r = 0;
	int i;
	long packets = 0;
	u8 mpeg_end[4] = { 0x00, 0x00, 0x01, 0xB9 };

	init_remux(&rem, fin, fout, mult);
	rem.pack_size = pack_size;
	refill_buffy(&rem);
	fprintf(stderr,"Package size: %d\n",pack_size);
	
	if ( get_video_info(&rem) < 0 ){
		fprintf(stderr,"ERROR: Can't find valid video stream\n");
		exit(1);
	}

	i = 0;
	while(! time_off && i < rem.vframen) {
		if( (time_off = rem.vframe_list[i].time)) break;
		i++;
	}

	if ( get_audio_info(&rem) < 0 ){
		fprintf(stderr,"ERROR: Can't find valid audio stream\n");
		exit(1);
	}
	
	rem.vpts = rem.vpts_list[0].PTS;
	fprintf(stderr,"Video start PTS = %fs \n",rem.vpts/90000.);
	rem.apts = rem.apts_list[0].PTS;
	fprintf(stderr,"Audio start PTS = %fs\n",rem.apts/90000.);
	fprintf(stderr,"Difference Video - Audio = %fs\n",ptsdiff/90000.);
	fprintf(stderr,"Time offset = %ds\n",time_off);
	if (ptsdiff < 1500)  ptsdiff = 0;
	rem.SCR_off = rem.vpts;
	rem.muxr = (rem.video_info.bit_rate + rem.audio_info.bit_rate)/50/8;
	ratio = rem.video_info.bit_rate/rem.audio_info.bit_rate;


	while( ring_rest(&rem.aud_buffy) && ring_rest(&rem.vid_buffy) ){
		int pn=0;
		int time;

		r = 0;
		while ( rem.vptsn < 2 && !r) r = refill_buffy(&rem);
		vpos = rem.vpts_list[1].pos;

		while ( rem.aptsn < 2 && !r) r = refill_buffy(&rem);
		apos = rem.apts_list[1].pos;

		while( vpos <= rem.vread+pack_size ){
			r = 0;
			while ( rem.vptsn < pn && !r) r = refill_buffy(&rem);
			pn++;
			vpos = rem.vpts_list[pn].pos;
		}
		if (r) break;

		i = 0;
		time = 0;
		while(! time && i < rem.vframen) {
			if( (time = rem.vframe_list[i].time)) break;
			i++;
		}

		if( time ){
			time -= time_off;
			fprintf(stderr,"                                    ");
			fprintf(stderr,"                          ");
			fprintf(stderr,"TIME %2d:", time/3600);
			fprintf(stderr,"%2d:", (time%3600)/60);
			fprintf(stderr,"%2d\r", (time%3600)%60);
		}
		fprintf(stderr,"Packs: %7d  VFrames: %7d  AFrames: %7d\r", 
			(int) packets, rem.vframe_list[0].FRAME,
			rem.aframe_list[0].FRAME);
                /*
		  fprintf(stderr,"SCR: %2dh %2dmin %2ds  #Packets: %d\r",
			(int)(rem.SCR/90000.)/3600,
			((int)(rem.SCR/90000.)%3600)/60,
			((int)(rem.SCR/90000.)%3600)%60, (int) packets);
		*/
		while ( vpos-rem.vread ){
			if((pos = write_video_pes( &rem, buf, vpos-rem.vread))
			   < 0) break;
			if (pos){
				write( fout, buf, pos);
				packets++;
			}
			while(  rem.apts+ptsdiff  < rem.vpts ){
				pn = 0;
				while( apos <= rem.aread){
					r = 0;
					while ( rem.aptsn < pn && !r) 
						r = refill_buffy(&rem);
					pn++;
					apos = rem.apts_list[pn].pos;
				}
				if ((pos = write_audio_pes( &rem, buf, 
						       apos-rem.aread)) < 0)
					break;
				if (pos){
					write( fout, buf, pos);
					packets++;
				}
			}
		}
	}
/*
	fprintf(stderr,"SCR: %2dh %2dmin %2ds  #Packets: %d\n",
		(int)(rem.SCR/90000.)/3600,
		((int)(rem.SCR/90000.)%3600)/60,
		((int)(rem.SCR/90000.)%3600)%60, (int) packets);
*/
	pos = write_ps_header(buf,rem.SCR,rem.muxr);
	pos += write_pes_header( PADDING_STREAM, pack_size-pos-4, 0, buf+pos);
	memset(buf+pos,0xff,pack_size-pos-4);
	pos = rem.pack_size-4;
	write( fout, buf, pos);
	write( fout, mpeg_end, 4);
}

