#ifndef _DVB_DEV_HH_
#define _DVB_DEV_HH_


extern "C" {
#include <stdint.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <strstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <pthread.h>

#include <sys/time.h>
#include <dvb_v4l.h>

#ifdef NAPI
#include <ost/dmx.h>
#include <ost/frontend.h>
#include <ost/sec.h>
#include <ost/video.h>
#endif

#include <si/dvb.h>
#include <dvb/dvb.h>
#include <si/decode.h>
#include <si/tbl.h>
#include <si/free_si.h>
#include <channel.h>
}

#include <osd.hh>
#include <devices.hh>


#define DEC(N) dec << setw(N) << setfill(int('0')) 
#define HEX(N) hex << setw(N) << setfill(int('0')) 

#define MAXSECSIZE 4096

#define NK 10
enum {LNB=0,DIS,ROTOR,TRANS,CHAN,BOU,SAT,PICS,SWI,NTW};
static const int nums[]={LNB,DIS,ROTOR,TRANS,CHAN,BOU,SAT,PICS,SWI,NTW};
static const int maxs[]={  8,  8,    8,  512,16384,512,100, 50, 10, 100};


#define VIDEO_DEV   "/dev/ost/video"
#define AUDIO_DEV   "/dev/ost/audio"
#define FRONT_DEV   "/dev/ost/qpskfe"
#define SEC_DEV     "/dev/ost/sec"
#define DEMUX_DEV   "/dev/ost/demux"


class DVB {
public:

int udelay(unsigned long usec)
{
        struct timeval timeout; 
  
	timeout.tv_usec = usec % (unsigned long) 1000000;
	timeout.tv_sec = usec / (unsigned long) 1000000;
	
	if(select(0, 0, 0, 0, &timeout) < 0) exit(1);
	return 0;
}

#ifdef NAPI
	int fd_video;
	int fd_audio;
	int fd_frontend;
	int fd_sec;
	int fd_demuxa;
	int fd_demuxv;
#else
        int fsi;
#endif
        int fdvb;
	


#ifdef NAPI
	struct secCommand scmd;
	struct secCmdSequence scmds;
 	struct dmxPesFilterParams pesFilterParamsV; 
	struct dmxPesFilterParams pesFilterParamsA; 
	struct qpskParameters qpsk;
#else
        frontend front;
#endif
        OSD osd;


        struct Lnb *lnbs;
        struct DiSEqC *diseqcs;
        struct Rotor *rotors;
        struct Transponder *tps;
        struct Channel *chans;
        struct Bouquet *bouqs;
        struct Sat *sats;
        struct Picture *pics;
        struct Switch *swis;
        struct Network *ntws;
        int num[NK];

	DVB(){
#ifdef NAPI
		fd_video = -1;
		fd_audio = -1;
		fd_frontend = -1;
		fd_sec = -1;
		fd_demuxa = -1;
		fd_demuxv = -1;
#else
		fsi = -1;
#endif
		fdvb = -1;
	        init("","",0);
	}

        DVB(char *a, char *b) {
#ifdef NAPI
		fd_video = -1;
		fd_audio = -1;
		fd_frontend = -1;
		fd_sec = -1;
		fd_demuxa = -1;
		fd_demuxv = -1;
#else
		fsi = -1;

#endif
		fdvb = -1;
	        init(a,b,0);
	}

	~DVB();
	
	void use_osd(int fd = -1){
		if (fd >= 0) fdvb = fd;
		else  fdvb = open("/dev/video0", O_RDWR);
		
		if (fdvb >= 0){
			cerr << "opening dev/video0 for OSD" << endl;
			osd.init(fdvb);
		} else perror("osd");

		
	}

        void init(char *a="/dev/video0", char *b="/dev/vbi0",int minor = 0);

	inline void init(char *a, char *b){
		init(a,b,0);
	}


	int set_tp(uint *freq, int ttk, int pol, uint srate);
	void set_apid(ushort apid);
	void set_vpid(ushort vpid); 
	int set_lnb(int dis); 
	void set_diseqc_nb(int nr); 
	void set_front(void); 
	void get_front(void); 
	int GetNIT(dvb_nit_t *nit); 
	int GetPMT(dvb_pmt_t *pmt, unsigned short pid);
	int GetPAT(dvb_pat_t *pat); 
	int GetSDT(dvb_sdt_t *sdt); 
	int setBitFilter(bitfilter bfilter, uint16_t PID,
			 uint8_t TID, uint16_t TIDExt, 
			 uint16_t FilterTIDExt);
	int GetSection(uint8_t *buf, 
		       uint16_t PID, uint8_t TID, uint16_t TIDExt, 
		       uint16_t FilterTIDExt, 
		       uint8_t secnum, uint8_t &msecnum);
	int GetSection(uint8_t *buf, 
		       uint16_t PID, uint8_t *filter, uint8_t *mask,
		       uint8_t secnum, uint8_t &msecnum); 
	int GetSection(unsigned char *buf, ushort PID, unsigned char sec,
		       unsigned char secnum, unsigned char &msecnum); 
	int SetFilter(uint16_t pid, unsigned char *filter, 
		      unsigned char *mask,
		      uint32_t timeout, uint32_t flags); 
	__u16 SetFilter(__u16 pid, __u16 section, __u16 mode); 
	int CloseFilter(int h);
	

	void bar2(int x, int y, int w, int h, int val, int col1, int col2);

        int SetTP(unsigned int, unsigned int);
        int scan(void);
        int scan_all_tps(void);
        int scan_lnb(struct Lnb &);
        int scan_cable(Sat &sat);
        int scan_sat(struct Sat &);
        int scan_tp(struct Transponder &);

        int AddLNB(int id, int t, uint l1, uint l2, uint sl,
		   int dnr, int dis, int sw);

        int AddSat(int satid, unsigned int lnbid, char *name, uint fmin, uint fmax);
        int AddTP(Transponder &tp);
        int AddChannels(Transponder &tp, dvb_pat_t &pat, dvb_sdt_t &sdt);
        int AddChannel(Channel &chan);

        pthread_t fthread;
        int start_fthread(void); 
        int stop_fthread(void); 
        void *fthread_start(void *);
  
        friend ostream &operator<<(ostream &stream, DVB &x);
        friend istream &operator>>(istream &stream, DVB &x);

        int channel_num(void) {
	        return num[CHAN];
	};
	  
        int channel_change(int n) {
	        return 0;
	};
        int SetChannel(uint16_t, uint16_t, uint16_t, uint16_t);
        bool Tune(int &newProgNr, int &oldProgNr, uint16_t ONID,
		  uint16_t TSID, int SID);
        int SetChannel(int chnr);
        int GetChannel(int chnr, struct channel *);
        int NumChannel(void) {
	        return num[CHAN];
	}
#ifdef NAPI
	feStatus tune_it(struct qpskParameters *qpsk);
#endif
};

void hdump(unsigned char *buf, int n);

#endif
